@@ -1,7 +1,7 @@
 /*
  * abcm2ps: a program to typeset tunes written in abc format using PostScript
  *
- * Copyright (C) 1998-2004 Jean-Franois Moine
+ * Copyright (C) 1998-2005 Jean-Franois Moine
  *
  * Adapted from abc2ps-1.2.5:
  *  Copyright (C) 1996,1997  Michael Methfessel
@@ -42,6 +42,9 @@
 #include <string.h>
 #include <ctype.h>
 #include <sys/stat.h>
+#ifdef linux
+#include <unistd.h>
+#endif
 
 #include "abcparse.h"
 #include "abc2ps.h"
@@ -114,7 +117,6 @@ static struct str_a {
 } *str_r[MAXAREAL], *str_c[MAXAREAL];	/* root and current area pointers */
 
 /* -- local functions -- */
-
 static void do_filter(struct abctune *t,
 		      char *sel);
 static void do_select(struct abctune *t,
@@ -125,6 +127,9 @@ static void read_def_format(void);
 static char *read_file(void);
 static void set_page_format(void);
 static void usage(void);
+#ifdef linux
+static void wherefmtdir(void);
+#endif
 static void whistle_parse(char *p);
 static void write_version(void);
 
@@ -136,12 +141,11 @@ int main(int argc,
 	char *p;
 	char c, *aaa;
 
-	/* -- set default options and parse arguments -- */
 	fprintf(stderr, "abcm2ps-" VERSION " (" VDATE ")\n");
 	if (argc <= 1)
 		usage();
 
-	/* -- initialize -- */
+	/* initialize */
 	outf[0] = '\0';
 	clrarena(0);			/* global desription */
 	clrarena(1);			/* tune description */
@@ -157,10 +161,18 @@ int main(int argc,
 	s_argc = argc;
 	s_argv = argv;
 
-	/* parse the arguments - as soon as a file ends, it is treated */
+#ifdef linux
+	/* if not set, try to find where is the default format directory */
+	if (styd[0] == '\0')
+		wherefmtdir();
+#endif
+
+	/* parse the arguments - finding a new file, treat the previous one */
 	while (--argc > 0) {
 		argv++;
 		p = *argv;
+		if (*p == 0)
+			continue;
 		if (*p == '+') {	/* switch off flags with '+' */
 			while (*++p != '\0') {
 				switch (*p) {
@@ -229,20 +241,6 @@ int main(int argc,
 				case 'l': landscape = 1; break;
 				case 'M': music_only = 1; break;
 				case 'n': write_history = 1; break;
-/*				case 'b': */
-				case 'C':
-				case 'g':
-				case 'P':
-				case 'p':
-				case 'R':
-				case 'S':
-				case 'T':
-				case 'o':
-				case 'v':
-					fprintf(stderr,
-						"'-%c' is obsolete - option ignored\n",
-						c);
-					break;
 				case 'Q': printtempo = 1; break;
 				case 'u': deco_old = 1; break;
 				case 'V':
@@ -277,7 +275,7 @@ int main(int argc,
 				case 'N':
 					if (p[1] == 0
 					    && (argc <= 1
-					 	|| !isdigit(argv[1][0]))) {
+					 	|| !isdigit((unsigned) argv[1][0]))) {
 						pagenumbers = 2; /* old behaviour */
 						break;
 					}
@@ -407,7 +405,7 @@ int main(int argc,
 					break;
 				default:
 					fprintf(stderr,
-						"++++ Unknown flag: -%c\n", c);
+						"++++ Unknown flag: -%c ignored\n", c);
 					severity = 1;
 					break;
 				}
@@ -751,6 +749,36 @@ static void usage(void)
 	exit(0);
 }
 
+#ifdef linux
+/* -- where is the default format directory -- */
+static void wherefmtdir(void)
+{
+	char exe[512], *p;
+	FILE *f;
+	int l;
+
+	if ((l = readlink("/proc/self/exe", exe, sizeof exe)) <= 0)
+		return;
+	if ((p = strrchr(exe, '/')) == 0)
+		return;
+	p++;
+	if (p > &exe[5] && strncmp(p - 5, "/bin", 4) == 0) {
+		strcpy(p - 4, "share/abcm2ps/");
+		p += -4 + 14;
+	} /* else, assume this is the source directory */
+
+	/* check if a format file is present */
+	strcpy(p, "tight.fmt");
+	if ((f = fopen(exe, "r")) == 0)
+		return;
+	fclose(f);
+
+	/* change the format directory */
+	p[-1] = '\0';
+	styd = strdup(exe);
+}
+#endif
+
 /* -- parse the whistle command ('-Wvk') -- */
 static void whistle_parse(char *p)
 {
@@ -791,8 +819,7 @@ static void whistle_parse(char *p)
 			pitch += 12;
 		else	pitch -= 12;
 	}
-	whistle_tb[nwhistle].pitch = pitch;
-	nwhistle++;
+	whistle_tb[nwhistle++].pitch = pitch;
 }
 
 /* -- write the program version -- */
@@ -815,9 +842,9 @@ static void write_version(void)
 #endif
 		"\n");
 
-	if (strlen(DEFAULT_FDIR) > 0)
+	if (strlen(styd) > 0)
 		fprintf(stderr,
-			"Default format directory: %s\n", DEFAULT_FDIR);
+			"Default format directory: %s\n", styd);
 }
 
 /* -- arena routines -- */
