@@ -9,15 +9,19 @@
 #define DIRSEP '\\'
 #endif
 
+#define CM		* 28.35	/* factor to transform cm to pt */
+#define PT			/* factor to transform pt to pt */
+#define IN		* 72.0	/* factor to transform inch to pt */
+
 /* basic page dimensions */
 #ifdef A4_FORMAT
-#define PAGEHEIGHT	(29.7 * CM)
-#define PAGEWIDTH	(21.0 * CM)
-#define MARGIN		(1.8 * CM)
+#define PAGEHEIGHT	(29.7 CM)
+#define PAGEWIDTH	(21.0 CM)
+#define MARGIN		(1.8 CM)
 #else
-#define PAGEHEIGHT	(11.0 * IN)
-#define PAGEWIDTH	(8.5 * IN)
-#define MARGIN		(0.7 * IN)
+#define PAGEHEIGHT	(11.0 IN)
+#define PAGEWIDTH	(8.5 IN)
+#define MARGIN		(0.7 IN)
 #endif
 
 /* -- macros controlling music typesetting -- */
@@ -50,7 +54,7 @@
 
 #define BETA_C		0.1	/* max expansion for flag -c */
 #define ALFA_X		1.0	/* max compression before complaining */
-#define BETA_X		1.2	/* max expansion before complaining */
+#define BETA_X		1.0	/* max expansion before complaining */
 
 #define VOCPRE		0.4	/* portion of vocals word before note */
 #define GCHPRE		0.4	/* portion of guitar chord before note */
@@ -69,10 +73,6 @@
 
 /* -- macros for program internals -- */
 
-#define CM		28.35	/* factor to transform cm to pt */
-#define PT		1.0	/* factor to transform pt to pt */
-#define IN		72.0	/* factor to transform inch to pt */
-
 #define STRL1		256	/* string length for file names */
 #define MAXSTAFF	16	/* max staves */
 #define BSIZE		512	/* buffer size for one input string */
@@ -96,6 +96,8 @@
 #define OBEYCENTER	3
 #define SKIP		4
 
+#define YSTEP	100		/* number of steps for y offsets */
+
 extern unsigned char deco_glob[256], deco_tune[256];
 
 struct FONTSPEC {
@@ -111,7 +113,7 @@ struct lyl {
 	struct FONTSPEC* f;	/* font */
 	float w;		/* width */
 	float s;		/* shift / note */
-	unsigned char t[1];	/* word */
+	char t[1];		/* word */
 };
 struct lyrics {
 	struct lyl *lyl[MAXLY];	/* ptr to lyric lines */
@@ -137,6 +139,7 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 #define GRACE		12
 #define FMTCHG		13
 #define TUPLET		14
+#define WHISTLE		15
 	unsigned char seq;	/* sequence # - see parse.c */
 	unsigned char voice;	/* voice (0..nvoice) */
 	unsigned char staff;	/* staff (0..nstaff) */
@@ -158,11 +161,13 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 #define S_TI1		0x0400		/* some chord tie start */
 #define S_DYNUP		0x0800		/* dynamic marks above the staff */
 #define S_DYNDN		0x1000		/* dynamic marks below the staff */
+#define S_TREM		0x2000		/* tremolo */
+#define S_XSTEM		0x4000		/* cross-staff stem */
 	unsigned char nhd;	/* number of notes in chord - 1 */
 	signed char stem;	/* 1 / -1 for stem up / down */
 	signed char nflags;	/* number of note flags when > 0 */
 	char dots;		/* number of dots */
-	char head;		/* head type */
+	unsigned char head;	/* head type */
 #define H_FULL		0
 #define H_EMPTY 	1
 #define H_OVAL		2
@@ -176,16 +181,18 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 				 *	- KEYSIG: old key signature
 				 *	- BAR: new bar number
 				 *	- TUPLET: tuplet format
-				 *	- FMTCHG (format change): subtype
-				 *	  with value in xmx: */
-#define STBRK 0				/* staff break */
+				 *	- FMTCHG (format change): subtype */
+#define STBRK 0				/* staff break
+					 *	xmx: width */
 #define PSSEQ 1				/* postscript sequence */
 	float x;		/* position */
 	short y;
 	short ymn, ymx, yav;	/* min, max, avg note head height */
 	float xmx;		/* max h-pos of a head rel to top */
+#ifndef YSTEP
 	float dc_top;		/* max offset needed for decorations */
 	float dc_bot;		/* min offset for decoration */
+#endif
 	float xs, ys;		/* position of stem end */
 	float wl, wr;		/* left, right min width */
 	float pl, pr;		/* left, right preferred width */
@@ -215,21 +222,22 @@ struct FORMAT { 		/* struct for page layout */
 	float scale, maxshrink, lineskipfac, parskipfac, sysstaffsep;
 	float indent, infospace, slurheight, notespacingfactor;
 	float maxstaffsep, maxsysstaffsep, stemheight;
-	int landscape, titleleft, continueall, writehistory;
-	int stretchstaff, stretchlast, withxrefs, barsperstaff;
-	int oneperpage, musiconly, titlecaps, graceslurs, straightflags;
-	int splittune, encoding, partsbox, infoline, printtempo, autoclef;
-	int measurenb, measurefirst, measurebox, flatbeams, squarebreve;
-	int exprabove, exprbelow, breathlow, vocalabove, freegchord;
-	int printparts, gchordbox, contbarnb, halfbeam, tuplets, bstemdown;
-	int shifthnote, aligncomposer, pslevel, hyphencont, textoption;
-	int dynalign;
+	int aligncomposer, autoclef, barsperstaff, breathlow, bstemdown;
+	int combinevoices, contbarnb, continueall, dynalign;
+	int encoding, exprabove, exprbelow, flatbeams, freegchord;
+	int infoline, gchordbox, graceslurs, halfbeam, hyphencont;
+	int landscape, measurenb, measurefirst, measurebox, musiconly;
+	int oneperpage, partsbox, printparts, printtempo, pslevel;
+	int setdefl, shifthnote, splittune, squarebreve;
+	int straightflags, stretchstaff, stretchlast;
+	int textoption, titlecaps, titleleft, tuplets;
+	int vocalabove, withxrefs, writehistory;
 	char *dateformat, *header, *footer;
-	struct FONTSPEC titlefont, subtitlefont, vocalfont, textfont;
-	struct FONTSPEC tempofont, composerfont, partsfont, gchordfont;
-	struct FONTSPEC wordsfont, footerfont, headerfont, infofont;
-	struct FONTSPEC repeatfont, measurefont, annotationfont;
-	struct FONTSPEC voicefont;
+	struct FONTSPEC annotationfont, composerfont, footerfont;
+	struct FONTSPEC gchordfont, headerfont, historyfont, infofont;
+	struct FONTSPEC measurefont, partsfont, repeatfont, subtitlefont;
+	struct FONTSPEC tempofont, textfont, titlefont;
+	struct FONTSPEC vocalfont, voicefont, wordsfont;
 #define DFONT_MIN 5		/* index of dynamic fonts */
 #define DFONT_MAX 16		/* max number of dynamic fonts */
 	struct FONTSPEC font_tb[DFONT_MAX];
@@ -253,7 +261,7 @@ struct ISTRUCT {		/* information fields */
 	struct SYMBOL *src;
 	struct SYMBOL *tempo;
 	struct SYMBOL *title;
-	unsigned char *xref;
+	char *xref;
 	struct SYMBOL *trans;
 };
 
@@ -269,9 +277,11 @@ extern int tunenum;		/* number of current tune */
 extern int pagenum;		/* current page number */
 extern int nbar;		/* current measure number */
 extern int nbar_rep;		/* last repeat bar number */
-
 extern int in_page;
-extern int defl;		/* decoration flags (see deco.c for values) */
+extern int defl;		/* decoration flags */
+#define DEF_NOST 0x01		/* long deco with no start */
+#define DEF_NOEN 0x02		/* long deco with no end */
+#define DEF_STEMUP 0x04		/* stem up (1) or down (0) */
 
 		/* switches modified by flags: */
 extern int pagenumbers; 	/* write page numbers ? */
@@ -297,18 +307,22 @@ extern int nwhistle;
 extern int s_argc;		/* command line arguments */
 extern char **s_argv;
 
-struct STAFF {
+struct STAFF_S {
 	struct clef_s clef;	/* base clef */
 	unsigned brace:1;	/* 1st staff of a brace */
 	unsigned brace_end:1;	/* 2nd staff of a brace */
 	unsigned bracket:1;	/* 1st staff of a bracket */
 	unsigned bracket_end:1;	/* last staff of a bracket */
 	unsigned forced_clef:1;	/* explicit clef */
-	unsigned stop_bar:1;	/* stop drawing bar on this staff */
+	unsigned stop_bar:1;	/* stop drawing measure bar on this staff */
+	unsigned empty:1;	/* no symbol on this staff */
 	float y;		/* y position */
-	short nvocal;		/* number of vocals (0..n) */
+	float scale;
+#ifdef YSTEP
+	float top[YSTEP], bot[YSTEP];	/* top/bottom y offsets */
+#endif
 };
-extern struct STAFF staff_tb[MAXSTAFF];
+extern struct STAFF_S staff_tb[MAXSTAFF];
 extern int nstaff;		/* (0..MAXSTAFF-1) */
 
 struct VOICE_S {
@@ -319,14 +333,14 @@ struct VOICE_S {
 	char *name;		/* voice id */
 	char *nm;		/* voice name */
 	char *snm;		/* voice subname */
+	char *bar_text;		/* bar text at start of staff when bar_start */
+	struct SYMBOL *tie;	/* note with ties of previous line */
+	struct SYMBOL *rtie;	/* note with ties before 1st repeat bar */
+	float scale;		/* scale */
+	int time;		/* current time while parsing */
 	struct clef_s clef;	/* current clef */
 	struct key_s key;	/* current key signature */
 	struct meter_s meter;	/* current time signature */
-	float yvocal;		/* current vocal vertical offset */
-	char *bar_text;		/* bar text at start of staff when bar_start */
-	int time;		/* current time while parsing */
-	struct SYMBOL *tie;	/* note with ties of previous line */
-	struct SYMBOL *rtie;	/* note with ties before 1st repeat bar */
 	unsigned hy_st;		/* lyrics hyphens at start of line (bit array) */
 	unsigned forced_clef:1;	/* explicit clef */
 	unsigned second:1;	/* secondary voice in a brace/parenthesis */
@@ -334,9 +348,10 @@ struct VOICE_S {
 	unsigned selected:1;	/* selected while sorting by time (music.c) */
 	unsigned bar_repeat:1;	/* bar at start of staff is a repeat bar */
 	unsigned norepbra:1;	/* display the repeat brackets */
+	unsigned have_ly:1;	/* some lyrics in this voice */
+	unsigned whistle:1;	/* tin whistle for this voice */
 	short wmeasure;		/* measure duration while parsing */
 	signed char bar_start;	/* bar type at start of staff / -1 */
-	unsigned char nvocal;	/* number of vocals (0..n) */
 	signed char clone;	/* duplicate from this voice number */
 	unsigned char staff;	/* staff (0..n-1) */
 	unsigned char cstaff;	/* staff while parsing */
@@ -344,10 +359,10 @@ struct VOICE_S {
 	signed char stem;	/* stem direction while parsing */
 	signed char gstem;	/* grace stem direction while parsing */
 	signed char dyn;	/* place of dynamic marks while parsing */
+	signed char ly_pos;	/* place of lyrics (above / below) */
 	unsigned char slur_st;	/* slurs at start of staff */
 };
 extern struct VOICE_S voice_tb[MAXVOICE]; /* voice table */
-extern int current_voice;	/* current voice while parsing */
 extern struct VOICE_S *first_voice; /* first_voice */
 
 extern struct SYMBOL *tsnext;	/* next line when cut */
@@ -392,26 +407,41 @@ unsigned char deco_intern(unsigned char deco);
 void deco_update(struct SYMBOL *s, float dx);
 float deco_width(struct SYMBOL *s);
 void draw_all_deco(void);
-int draw_deco_head(int deco, float x, float y);
+int draw_deco_head(int deco, float x, float y, int stem);
 void draw_all_deco_head(struct SYMBOL *s, float x, float y);
 void draw_deco_near(void);
 void draw_deco_note(void);
 void draw_deco_staff(void);
 float draw_partempo(float top,
 		    int any_part,
-		    int any_tempo,
-		    int any_vocal);
+		    int any_tempo);
 void draw_measnb(void);
 void reset_deco(int deco_old);
+void set_defl(int new_defl);
+float y_get(struct SYMBOL *s,
+	    int up,
+	    float x,
+	    float w,
+	    float h);
+void y_set(struct SYMBOL *s,
+	   int up,
+	   float x,
+	   float w,
+	   float y);
 /* draw.c */
-void draw_staff(int mline, float indent);
 void draw_sym_near(void);
 void draw_symbols(struct VOICE_S *p_voice);
+void draw_vname(int mline, float indent);
 void draw_whistle(void);
+void set_scale(int staff);
+void putf(float f);
+void putx(float x);
+void puty(float y);
+void putxy(float x, float y);
 /* format.c */
 int interpret_format_line(char *w, char *p);
 void define_fonts(void);
-int get_textopt(unsigned char *p);
+int get_textopt(char *p);
 void make_font_list(void);
 void print_format(void);
 int read_fmt_file(char *filename);
@@ -434,7 +464,7 @@ void identify_note(struct SYMBOL *s,
 /* subs.c */
 void bug(char *msg, int fatal);
 void error(int sev, struct SYMBOL *s, char *fmt, ...);
-void cap_str(unsigned char *c);
+void cap_str(char *c);
 float scan_u(char *str);
 void add_to_text_block(char *s, int job);
 float cwid(unsigned char c);
@@ -445,15 +475,15 @@ void str_font(struct FONTSPEC *font);
 #define A_LEFT -1
 #define A_CENTER 0
 #define A_RIGHT 1
-void str_out(unsigned char *p, int action);
-void put_str(unsigned char *str, int action);
+void str_out(char *p, int action);
+void put_str(char *str, int action);
 float tex_str(char *s);
-extern unsigned char tex_buf[];	/* result of tex_str() */
+extern char tex_buf[];	/* result of tex_str() */
 #define TEX_BUF_SZ 512
 void user_ps_add(char *s);
 void user_ps_write(void);
 void write_title(struct SYMBOL *s);
-void write_heading(void);
+void write_heading(struct abctune *t);
 void write_user_ps(void);
 void write_text_block(int job, int abc_state);
 /* syms.c */
