@@ -1,7 +1,7 @@
 /*++
  * Generic ABC parser.
  *
- * Copyright (C) 1998-2004 Jean-Franois Moine
+ * Copyright (C) 1998-2005 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996, 1997  Michael Methfessel
  *
  * Contact: mailto:moinejf@free.fr
@@ -40,7 +40,7 @@ static int keep_comment;
 
 /* global values */
 char *deco_tb[128];		/* decoration names */
-#define MAXMICRO 16
+#define MAXMICRO 32
 unsigned short micro_tb[MAXMICRO]; /* microtone values [ (n-1) | (d-1) ] */
 int severity;			/* error severity */
 
@@ -57,10 +57,10 @@ static int vover;		/* voice overlay (1: single bar, -1: multi-bar */
 
 #define VOICE_NAME_SZ 64	/* max size of a voice name */
 
-static unsigned char *file;	/* remaining abc file */
+static char *file;		/* remaining abc file */
 static int linenum;		/* current source line number */
 static int colnum;		/* current source column number */
-static unsigned char *scratch_line;	/* parse line */
+static char *scratch_line;	/* parse line */
 static int scratch_length = 0;	/* allocated length */
 
 static short nvoice;		/* number of voices (0..n-1) */
@@ -133,22 +133,24 @@ static char char_tb[256] = {
 
 static char all_notes[] = "CDEFGABcdefgab^=_";
 
-static unsigned char *get_line(void);
-static unsigned char *parse_len(unsigned char *p,
-				int *p_len);
-static unsigned char *parse_basic_note(unsigned char *p,
-				       int *pitch,
-				       int *length,
-				       int *accidental,
-				       int *stemless);
+static char *get_line(void);
+static char *parse_len(char *p,
+			int *p_len);
+static char *parse_basic_note(char *p,
+			      int *pitch,
+			      int *length,
+			      int *accidental,
+			      int *stemless);
 static void parse_info(struct abctune *t,
-		       unsigned char *p,
+		       char *p,
 		       char *comment);
+static char *parse_gchord(char *p,
+			  int *p_more_gch);
 static int parse_line(struct abctune *t,
-		      unsigned char *p);
-static unsigned char *parse_note(struct abctune *t,
-				 unsigned char *p);
-static void syntax(char *msg, unsigned char *q);
+		      char *p);
+static char *parse_note(struct abctune *t,
+			char *p);
+static void syntax(char *msg, char *q);
 static void vover_new(void);
 
 /* -- abcMIDI like errors -- */
@@ -201,7 +203,7 @@ void abc_delete(struct abcsym *as)
 	if (as->tune->first_sym == as)
 		as->tune->first_sym = as->next;
 	if (as->tune->last_sym == as)
-		if ((as->tune->last_sym = as->prev) == 0)
+		as->tune->last_sym = as->prev;
 	free_f(as);
 }
 
@@ -257,7 +259,7 @@ void abc_init(void *alloc_f_api(int size),
 void abc_insert(char *file_api,
 		struct abcsym *s)
 {
-	unsigned char *p;
+	char *p;
 	struct abctune *t;
 
 	/* initialize */
@@ -284,8 +286,8 @@ void abc_insert(char *file_api,
 
 /* -- new symbol -- */
 struct abcsym *abc_new(struct abctune *t,
-		       unsigned char *text,
-		       unsigned char *comment)
+		       char *text,
+		       char *comment)
 {
 	struct abcsym *s;
 
@@ -317,7 +319,7 @@ struct abcsym *abc_new(struct abctune *t,
 /* -- parse an ABC file -- */
 struct abctune *abc_parse(char *file_api)
 {
-	unsigned char *p;
+	char *p;
 	struct abctune *first_tune = 0;
 	struct abctune *t, *last_tune;
 
@@ -339,26 +341,26 @@ struct abctune *abc_parse(char *file_api)
 					    -1);
 				severity = 1;
 			}
+			if (t != 0)
+				t->abc_vers = abc_vers;
 			break;			/* done */
 		}
-		while (isspace(*p))		/* skip starting blanks */
+		while (isspace((unsigned) *p))	/* skip starting blanks */
 			p++;
 
 		/* start a new tune if not done */
 		if (t == 0) {
-			struct abctune *n;
-
 			if (*p == '\0')
 				continue;
-			n = alloc_f(sizeof *n);
-			memset(n, 0 , sizeof *n);
+			t = alloc_f(sizeof *t);
+			memset(t, 0 , sizeof *t);
 			if (last_tune == 0)
-				first_tune = n;
+				first_tune = t;
 			else	{
-				last_tune->next = n;
-				n->prev = last_tune;
+				last_tune->next = t;
+				t->prev = last_tune;
 			}
-			last_tune = t = n;
+			last_tune = t;
 			ulen = gulen;
 		}
 
@@ -370,10 +372,10 @@ struct abctune *abc_parse(char *file_api)
 }
 
 /* -- cut off after % and remove trailing blanks -- */
-static unsigned char *decomment_line(unsigned char *p)
+static char *decomment_line(char *p)
 {
 	int i;
-	unsigned char c, *comment = 0;
+	char c, *comment = 0;
 
 	i = 0;
 	for (;;) {
@@ -394,7 +396,7 @@ static unsigned char *decomment_line(unsigned char *p)
 	/* remove the trailing blanks */
 	while (--i > 0) {
 		c = *--p;
-		if (!isspace(c)) {
+		if (!isspace((unsigned) c)) {
 			p[1] = '\0';
 			break;
 		}
@@ -404,15 +406,14 @@ static unsigned char *decomment_line(unsigned char *p)
 
 /* -- define a voice by name -- */
 /* the voice is created if it does not exist */
-static char *def_voice(unsigned char *p,
+static char *def_voice(char *p,
 		       int *p_voice)
 {
-	char *name;
-	char sep;
+	char *name, sep;
 	int voice;
 
 	name = p;
-	while (isalnum(*p) || *p == '_')
+	while (isalnum((unsigned) *p) || *p == '_')
 		p++;
 	sep = *p;
 	*p = '\0';
@@ -465,7 +466,7 @@ static void broken_rhythm(struct note *note,
 }
 
 /* -- check for the '!' as end of line (ABC2Win) -- */
-static int check_nl(unsigned char *p)
+static int check_nl(char *p)
 {
 	while (*p != '\0') {
 		switch (*p++) {
@@ -484,9 +485,11 @@ static int check_nl(unsigned char *p)
 }
 
 /* -- skip a clef definition -- */
-static unsigned char *clef_skip(unsigned char *p,
-				unsigned char **p_name,
-				unsigned char **p_middle)
+static char *clef_skip(char *p,
+			char **p_name,
+			char **p_middle,
+			char **p_lines,
+			char **p_scale)
 {
 	for (;;) {
 		if (strncmp(p, "clef=", 5) == 0
@@ -502,14 +505,22 @@ static unsigned char *clef_skip(unsigned char *p,
 			   || strncmp(p, "m=", 2) == 0) {
 			if (*p_middle != 0)
 				syntax("Double clef middle", p);
-			*p_middle = p;
+			*p_middle = p + (p[1] == '=' ? 2 : 7);
+		} else if (strncmp(p, "stafflines=", 11) == 0) {
+			if (*p_lines != 0)
+				syntax("Double clef stafflines", p);
+			*p_lines = p + 11;
+		} else if (strncmp(p, "staffscale=", 11) == 0) {
+			if (*p_scale != 0)
+				syntax("Double clef staffscale", p);
+			*p_scale = p + 11;
 		} else if (strncmp(p, "transpose=", 10) == 0
 			   || strncmp(p, "t=", 2) == 0) {
 			;		/* ignored */
 		} else	break;
-		while (!isspace(*p) && *p != '\0')
+		while (!isspace((unsigned) *p) && *p != '\0')
 			p++;
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 		if (*p == '\0')
 			break;
@@ -518,11 +529,10 @@ static unsigned char *clef_skip(unsigned char *p,
 }
 
 /* -- parse a decoration '[!|+]xxx[!|+]' -- */
-static unsigned char *get_deco(unsigned char *p,
-			       unsigned char *p_deco)
+static char *get_deco(char *p,
+		      unsigned char *p_deco)
 {
-	unsigned char *q, sep;
-	char **t;
+	char *q, sep, **t;
 	int i, l;
 
 	*p_deco = 0;
@@ -565,8 +575,8 @@ static unsigned char *get_deco(unsigned char *p,
 }
 
 /* -- parse a list of accidentals (K:) -- */
-static unsigned char *parse_acc(unsigned char *p,
-				struct abcsym *s)
+static char *parse_acc(char *p,
+			struct abcsym *s)
 {
 	int pit, len, acc, nostem, nacc;
 
@@ -590,18 +600,17 @@ static unsigned char *parse_acc(unsigned char *p,
 
 /* -- parse a clef (K: or V:) -- */
 static void parse_clef(struct clef_s *p_clef,
-		       unsigned char *name,
-		       unsigned char *middle)
+		       char *name,
+		       char *middle,
+		       char *lines,
+		       char *scale)
 {
 	int clef = -1;
 	int transpose = 0;
 	int clef_line = 2;
-	int clef_name;
 
-	clef_name = 0;
 	if (name != 0 && strncmp(name, "clef=", 5) == 0) {
 		name += 5;
-		clef_name = 1;
 		switch (*name) {
 		case 'g':
 			transpose = -7;
@@ -627,10 +636,8 @@ static void parse_clef(struct clef_s *p_clef,
 		case 'P':
 			clef = PERC;
 			break;
-		default:
-			clef_name = 0;
 		}
-		if (clef_name) {
+		if (clef >= 0) {
 			name++;
 			while (*name == ',') {
 				transpose += 7;
@@ -642,7 +649,7 @@ static void parse_clef(struct clef_s *p_clef,
 			}
 		}
 	}
-	if (name != 0 && !clef_name) {
+	if (name != 0 && clef < 0) {
 		if (!strncmp(name, "bass", 4)) {
 			clef = BASS;
 			clef_line = 4;
@@ -652,11 +659,9 @@ static void parse_clef(struct clef_s *p_clef,
 			p_clef->check_pitch = 1;
 #endif
 			name += 4;
-			clef_name = 1;
 		} else if (!strncmp(name, "treble", 6)) {
 			clef = TREBLE;
 			name += 6;
-			clef_name = 1;
 		} else if (!strncmp(name, "alto", 4)
 			   || !strncmp(name, "tenor", 5)) {
 			clef = ALTO;
@@ -669,7 +674,6 @@ static void parse_clef(struct clef_s *p_clef,
 			if (*name == 'a')
 				name += 4;
 			else	name += 5;
-			clef_name = 1;
 		} else if (!strncmp(name, "perc", 4)) {
 			clef = PERC;
 			name += 4;
@@ -680,7 +684,7 @@ static void parse_clef(struct clef_s *p_clef,
 		} else	syntax("Unknown clef", name);
 	}
 
-	if (clef_name) {
+	if (clef >= 0) {
 		switch (*name) {
 		case '1':
 		case '2':
@@ -702,13 +706,11 @@ static void parse_clef(struct clef_s *p_clef,
 		int pit, len, acc, nostem, l;
 
 		/* 'middle=<note pitch>' */
-		while (*middle++ != '=')
-			;
-		if (clef < 0)
-			clef = TREBLE;
 		curvoice->add_pitch = 0;
 		parse_basic_note(middle, &pit, &len, &acc, &nostem);
 		l = 20;
+		if (clef < 0)
+			clef = TREBLE;
 		switch (clef) {
 		case ALTO:
 			l = 16;
@@ -728,24 +730,41 @@ static void parse_clef(struct clef_s *p_clef,
 	p_clef->type = clef;
 	p_clef->line = clef_line;
 	p_clef->transpose = curvoice->add_pitch = transpose;
+	p_clef->stafflines = -1;
+	p_clef->staffscale = 0;
+	if (lines != 0) {
+		int l;
+
+		l = atoi(lines);
+		if ((unsigned) l < 10)
+			p_clef->stafflines = l;
+		else	syntax("Bad value of stafflines", lines);
+	}
+	if (scale != 0) {
+		float sc;
+
+		sc = atof(scale);
+		if (sc >= 0.5 && sc <= 2)
+			p_clef->staffscale = sc;
+		else	syntax("Bad value of staffscale", scale);
+	}
 }
 
 /* -- parse a 'K:' -- */
-static void parse_key(unsigned char *p,
+static void parse_key(char *p,
 		      struct abcsym *s)
 {
 	int sf;
-	unsigned char *clef_name, *clef_middle;
+	char *clef_name, *clef_middle, *clef_lines, *clef_scale;
 
-	clef_name = 0;
-	clef_middle = 0;
+	clef_name = clef_middle = clef_lines = clef_scale = 0;
 	if (strncmp(p, "none", 4) == 0) {
 		s->u.key.empty = 2;
 		p += 4;
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 	}
-	p = clef_skip(p, &clef_name, &clef_middle);
+	p = clef_skip(p, &clef_name, &clef_middle, &clef_lines, &clef_scale);
 	sf = 0;
 	switch (*p++) {
 	case 'F': sf = -1; break;
@@ -788,11 +807,11 @@ static void parse_key(unsigned char *p,
 	}
 
 	while (*p != '\0') {
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 		if (*p == '\0')
 			break;
-		p = clef_skip(p, &clef_name, &clef_middle);
+		p = clef_skip(p, &clef_name, &clef_middle, &clef_lines, &clef_scale);
 		if (*p == '\0')
 			break;
 		switch (*p) {
@@ -834,7 +853,7 @@ static void parse_key(unsigned char *p,
 			else if (strncasecmp(p, "mix", 3) == 0)
 				sf -= 1;
 			else if (strncasecmp(p, "min", 3) == 0
-				 || !isalpha(p[1])) {		/* 'm' alone */
+				 || !isalpha((unsigned char) p[1])) { /* 'm' alone */
 				sf -= 3;
 				s->u.key.minor = 1;
 			} else	goto unk;
@@ -843,7 +862,8 @@ static void parse_key(unsigned char *p,
 		case 'O':
 			if (strncasecmp(p, "octave", 6) == 0) {	/* (abcMIDI) */
 				p += 6;
-				while (!isspace(*p) && *p != '\0')
+				while (!isspace((unsigned char) *p)
+				       && *p != '\0')
 					p++;
 				continue;
 			}
@@ -862,11 +882,11 @@ static void parse_key(unsigned char *p,
 		default:
 		unk:
 			syntax("Unknown token in key specifier", p);
-			while (!isspace(*p) && *p != '\0')
+			while (!isspace((unsigned) *p) && *p != '\0')
 				p++;
 			continue;
 		}
-		while (isalpha(*p))
+		while (isalpha((unsigned) *p))
 			p++;
 	}
 
@@ -878,10 +898,10 @@ static void parse_key(unsigned char *p,
 	}
 	s->u.key.sf = sf;
 
-	if (clef_name != 0 || clef_middle != 0) {
+	if (clef_name != 0 || clef_middle != 0 || clef_lines != 0 || clef_scale != 0) {
 		s = abc_new(s->tune, 0, 0);
 		s->type = ABC_T_CLEF;
-		parse_clef(&s->u.clef, clef_name, clef_middle);
+		parse_clef(&s->u.clef, clef_name, clef_middle, clef_lines, clef_scale);
 		if (abc_state == ABC_S_HEAD) {	/* if first K: */
 			int i;
 
@@ -892,7 +912,7 @@ static void parse_key(unsigned char *p,
 }
 
 /* -- set default length from 'L:' -- */
-static char *get_len(unsigned char *p,
+static char *get_len(char *p,
 		     struct abcsym *s)
 {
 	int l1, l2, d;
@@ -923,10 +943,10 @@ static char *get_len(unsigned char *p,
 }
 
 /* -- get a new line from the current file in memory -- */
-static unsigned char *get_line(void)
+static char *get_line(void)
 {
 	int l;
-	unsigned char *p, *line;
+	char *p, *line;
 
 	p = file;
 	if (*p == '\0')
@@ -970,7 +990,7 @@ static unsigned char *get_line(void)
 				if (*p == '\\'
 				    && p[1] != '%') {	/* (??) */
 					line = p + 1;
-					while (isspace(*line))
+					while (isspace((unsigned) *line))
 						line++;
 					if (*line == '\0' || *line == '%')
 						break;
@@ -1017,12 +1037,10 @@ static unsigned char *get_line(void)
 }
 
 /* -- parse a 'M:' -- */
-static char *parse_meter(unsigned char *p,
+static char *parse_meter(char *p,
 			 struct abcsym *s)
 {
-	int m1, m2, d, wmeasure;
-	int nm, i;
-	int in_parenth;
+	int m1, m2, d, wmeasure, nm, i, in_parenth;
 
 	if (*p == '\0')
 		return "Empty meter string";
@@ -1030,7 +1048,7 @@ static char *parse_meter(unsigned char *p,
 	in_parenth = 0;
 	wmeasure = 0;
 	if (*p == 'N' || *p == 'n')
-		;				/* no meter */
+		p++;				/* no meter */
 	else if (*p == 'C') {
 		s->u.meter.meter[0].top[0] = *p++;
 		if (*p == '|')
@@ -1046,13 +1064,17 @@ static char *parse_meter(unsigned char *p,
 			nm++;
 			continue;
 		}
+		if (*p == '=')
+			break;
 		if (sscanf(p, "%d", &m1) != 1
 		    || m1 <= 0)
 			return "Cannot identify meter top";
+		if (nm >= MAX_MEASURE)
+			return "Too many values in M:";
 		i = 0;
 		m2 = 2;			/* default when no bottom value */
 		for (;;) {
-			while (isdigit(*p)
+			while (isdigit((unsigned) *p)
 			       && i < sizeof s->u.meter.meter[0].top)
 				s->u.meter.meter[nm].top[i++] = *p++;
 			if (*p == '/') {
@@ -1061,7 +1083,7 @@ static char *parse_meter(unsigned char *p,
 				    || m2 <= 0)
 					return "Cannot identify meter bottom";
 				i = 0;
-				while (isdigit(*p)
+				while (isdigit((unsigned) *p)
 				       && i < sizeof s->u.meter.meter[0].bot)
 					s->u.meter.meter[nm].bot[i++] = *p++;
 				break;
@@ -1085,6 +1107,14 @@ static char *parse_meter(unsigned char *p,
 			nm++;
 		}
 	}
+	if (*p == '=') {
+		if (sscanf(++p, "%d/%d", &m1, &m2) != 2
+		    || m1 <= 0
+		    || m2 <= 0)
+			return "Cannot identify meter explicit duration";
+		wmeasure = m1 * BASE_LEN / m2;
+		s->u.meter.expdur = 1;
+	}
 
 	s->u.meter.wmeasure = wmeasure;
 	s->u.meter.nmeter = nm;
@@ -1101,7 +1131,7 @@ static char *parse_meter(unsigned char *p,
 }
 
 /* -- treat %%staves -- */
-static void parse_staves(unsigned char *p,
+static void parse_staves(char *p,
 			 struct abcsym *s)
 {
 	int voice;
@@ -1156,7 +1186,7 @@ static void parse_staves(unsigned char *p,
 			staff->flags |= STOP_BAR;
 			break;
 		default:
-			if (!isalnum(*p) && *p != '_')
+			if (!isalnum((unsigned) *p) && *p != '_')
 				goto err;
 			if (voice >= MAXVOICE) {
 				syntax("Too many voices in %%staves", p);
@@ -1223,14 +1253,14 @@ err:
 }
 
 /* -- get a possibly quoted string -- */
-char *get_str(unsigned char *d,		/* destination */
-	      unsigned char *s,		/* source */
-	      int maxlen)		/* max length */
+char *get_str(char *d,		/* destination */
+	      char *s,		/* source */
+	      int maxlen)	/* max length */
 {
 	char sep, c;
 
 	maxlen--;		/* have place for the EOS */
-	while (isspace(*s))
+	while (isspace((unsigned) *s))
 		s++;
 	if (*s == '"') {
 		sep = '"';
@@ -1252,18 +1282,17 @@ char *get_str(unsigned char *d,		/* destination */
 		s++;
 	}
 	*d = '\0';
-	while (isspace(*s))
+	while (isspace((unsigned) *s))
 		s++;
 	return s;
 }
 
 /* -- parse a tempo (Q:) -- */
-static char *parse_tempo(unsigned char *p,
+static char *parse_tempo(char *p,
 			 struct abcsym *s)
 {
-	int have_error = 0;
-	unsigned char *q;
-	int l;
+	int l, have_error = 0;
+	char *q;
 
 	/* string before */
 	if (*p == '"') {
@@ -1276,7 +1305,7 @@ static char *parse_tempo(unsigned char *p,
 		s->u.tempo.str1[l] = '\0';
 		if (*p == '"')
 			p++;
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 	}
 
@@ -1289,13 +1318,13 @@ static char *parse_tempo(unsigned char *p,
 		if (len <= 0)
 			have_error++;
 		else	s->u.tempo.length[0] = len;
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
-	} else if (isdigit(*p) && strchr(p, '/') != 0) {
+	} else if (isdigit((unsigned) *p) && strchr(p, '/') != 0) {
 		int i;
 
 		i = 0;
-		while (isdigit(*p)) {
+		while (isdigit((unsigned) *p)) {
 			int top, bot, n;
 
 			if (sscanf(p, "%d /%d%n", &top, &bot, &n) != 2
@@ -1310,7 +1339,7 @@ static char *parse_tempo(unsigned char *p,
 				have_error++;
 			else	s->u.tempo.length[i++] = l;
 			p += n;
-			while (isspace(*p))
+			while (isspace((unsigned) *p))
 				p++;
 		}
 	}
@@ -1318,20 +1347,20 @@ static char *parse_tempo(unsigned char *p,
 	/* tempo value ('Q:beat=value' or 'Q:value') */
 	if (*p == '=') {
 		p++;
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 	}
 	if (*p != '\0' && *p != '"') {
 		q = p;
 		while (*p != '"' && *p != '\0')
 			p++;
-		while (isspace(p[-1]))
+		while (isspace((unsigned) p[-1]))
 			p--;
 		l = p - q;
 		s->u.tempo.value = alloc_f(l + 1);
 		strncpy(s->u.tempo.value, q, l);
 		s->u.tempo.value[l] = '\0';
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 	}
 
@@ -1350,17 +1379,17 @@ static char *parse_tempo(unsigned char *p,
 }
 
 /* -- get a user defined accent (U:) -- */
-static char *get_user(unsigned char *p,
+static char *get_user(char *p,
 		      struct abcsym *s)
 {
 /*fixme: may have 'U: <char> = "text" */
-	if (char_tb[*p] != CHAR_DECO)	/* accept any character */
+	if (char_tb[(unsigned) *p] != CHAR_DECO) /* accept any character */
 /*fixme: should be for the current tune only */
-		char_tb[*p] = CHAR_DECO;
+		char_tb[(unsigned) *p] = CHAR_DECO;
 	s->u.user.symbol = *p++;
 
 	/* skip '=' */
-	while (isspace(*p) || *p == '=')
+	while (isspace((unsigned) *p) || *p == '=')
 		p++;
 	if (*p == '!' || *p == '+')
 		p++;
@@ -1369,13 +1398,13 @@ static char *get_user(unsigned char *p,
 }
 
 /* -- parse the voice parameters (V:) -- */
-static char *parse_voice(unsigned char *p,
+static char *parse_voice(char *p,
 			 struct abcsym *s)
 {
 	int voice;
 	char *error_txt = 0;
 	char name[VOICE_NAME_SZ];
-	unsigned char *clef_name, *clef_middle;
+	char *clef_name, *clef_middle, *clef_lines, *clef_scale;
 	signed char *p_stem;
 static struct kw_s {
 	char *name;
@@ -1384,7 +1413,7 @@ static struct kw_s {
 } kw_tb[] = {
 	{"name=", 5, 0},
 	{"nm=", 3, 0},
-	{"subname=", 7, 1},
+	{"subname=", 8, 1},
 	{"sname=", 6, 1},
 	{"snm=", 4, 1},
 	{"merge", 5, 2},
@@ -1394,6 +1423,8 @@ static struct kw_s {
 	{"gstem=", 6, 6},
 	{"auto", 4, 7},
 	{"dyn=", 4, 8},
+	{"lyrics=", 7, 9},
+	{"scale=", 6, 10},
 	{0, 0, 0}
 };
 	struct kw_s *kw;
@@ -1428,15 +1459,14 @@ static struct kw_s {
 		ulen = curvoice->ulen;
 
 	/* parse the other parameters */
-	clef_name = 0;
-	clef_middle = 0;
+	clef_name = clef_middle = clef_lines = clef_scale = 0;
 	p_stem = &s->u.voice.stem;
 	for (;;) {
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 		if (*p == '\0')
 			break;
-		p = clef_skip(p, &clef_name, &clef_middle);
+		p = clef_skip(p, &clef_name, &clef_middle, &clef_lines, &clef_scale);
 		if (*p == '\0')
 			break;
 		for (kw = kw_tb; kw->name; kw++) {
@@ -1444,7 +1474,7 @@ static struct kw_s {
 				break;
 		}
 		if (!kw->name) {
-			while (!isspace(*p) && *p != '\0')
+			while (!isspace((unsigned char) *p) && *p != '\0')
 				p++;	/* ignore unknown keywords */
 			continue;
 		}
@@ -1481,19 +1511,33 @@ static struct kw_s {
 		case 8:			/* dyn= */
 			p_stem = &s->u.voice.dyn;
 			break;
+		case 9:			/* lyrics= */
+			p_stem = &s->u.voice.lyrics;
+			break;
+		case 10: {		/* scale= */
+			float sc;
+
+			sc = atof(p);
+			if (sc >= 0.5 && sc <= 2)
+				s->u.voice.scale = sc;
+			else	error_txt = "Bad value for voice scale";
+			while (!isspace((unsigned) *p) && *p != '\0')
+				p++;
+			break;
+		    }
 		}
 	}
-	if (clef_name != 0 || clef_middle != 0) {
+	if (clef_name != 0 || clef_middle != 0 || clef_lines != 0 || clef_scale != 0) {
 		s = abc_new(s->tune, 0, 0);
 		s->type = ABC_T_CLEF;
-		parse_clef(&s->u.clef, clef_name, clef_middle);
+		parse_clef(&s->u.clef, clef_name, clef_middle, clef_lines, clef_scale);
 	}
 	return error_txt;
 }
 
 /* -- parse a bar -- */
 static char *parse_bar(struct abctune *t,
-		       unsigned char *p)
+		       char *p)
 {
 	struct abcsym *s;
 	int bar_type;
@@ -1551,7 +1595,7 @@ static char *parse_bar(struct abctune *t,
 		memcpy(&s->u.bar.dc, &dc, sizeof s->u.bar.dc);
 		dc.n = dc.h = dc.s = 0;
 	}
-	if (!isdigit(*p)			/* if not a repeat bar */
+	if (!isdigit((unsigned) *p)		/* if not a repeat bar */
 	    && (*p != '"' || p[-1] != '['))	/* ('["' only) */
 		return p;
 
@@ -1561,10 +1605,10 @@ static char *parse_bar(struct abctune *t,
 		char *q;
 
 		q = repeat_value;
-		while (isdigit(*p)
+		while (isdigit((unsigned) *p)
 		       || *p == ','
 		       || *p == '-'
-		       || (*p == '.' && isdigit(p[1]))) {
+		       || (*p == '.' && isdigit((unsigned) p[1]))) {
 			if (q < &repeat_value[sizeof repeat_value - 1])
 				*q++ = *p++;
 			else	p++;
@@ -1586,14 +1630,14 @@ static char *parse_bar(struct abctune *t,
 }
 
 /* -- parse note or rest with pitch and length -- */
-static unsigned char *parse_basic_note(unsigned char *p,
-				       int *pitch,
-				       int *length,
-				       int *accidental,
-				       int *stemless)
+static char *parse_basic_note(char *p,
+			      int *pitch,
+			      int *length,
+			      int *accidental,
+			      int *stemless)
 {
 	int pit, len, acc, nostem;
-	unsigned char *q;
+	char *q;
 
 	acc = pit = nostem = 0;
 
@@ -1602,8 +1646,8 @@ static unsigned char *parse_basic_note(unsigned char *p,
 	case '^':
 		p++;
 		if (*p == '^') {
-			acc = A_DS;
 			p++;
+			acc = A_DS;
 		} else	acc = A_SH;
 		break;
 	case '=':
@@ -1613,15 +1657,15 @@ static unsigned char *parse_basic_note(unsigned char *p,
 	case '_':
 		p++;
 		if (*p == '_') {
-			acc = A_DF;
 			p++;
+			acc = A_DF;
 		} else	acc = A_FT;
 		break;
 	}
 
 	/* look for microtone value */
 	if (acc != 0
-	    && (isdigit(*p) || *p == '/')) {
+	    && (isdigit((unsigned) *p) || *p == '/')) {
 		int n, d;
 
 		n = d = 1;
@@ -1631,7 +1675,7 @@ static unsigned char *parse_basic_note(unsigned char *p,
 		}
 		if (*p == '/') {
 			p++;
-			if (!isdigit(*p))
+			if (!isdigit((unsigned) *p))
 				d = 2;
 			else {
 				d = strtol(p, (char **) &q, 10);
@@ -1648,7 +1692,11 @@ static unsigned char *parse_basic_note(unsigned char *p,
 				break;
 			}
 		}
-		acc += (n << 4);
+		if (n == MAXMICRO) {
+			syntax("Too many microtone accidentals", p);
+			n = 0;
+		}
+		acc += (n << 3);
 	}
 
 	/* get the pitch */
@@ -1689,23 +1737,21 @@ static unsigned char *parse_basic_note(unsigned char *p,
 	*length = len;
 	*accidental = acc;
 	*stemless = nostem;
-
 	return p;
 }
 
 /* -- parse the decorations of notes and bars -- */
-unsigned char *parse_deco(unsigned char *p,
-			  struct deco *deco)
+char *parse_deco(char *p,
+		 struct deco *deco)
 {
 	int n;
 	unsigned char c, d;
 
 	n = deco->n;
 	for (;;) {
-		c = *p++;
+		c = (unsigned char) *p++;
 		if (char_tb[c] != CHAR_DECO && char_tb[c] != CHAR_DECOS)
 			break;
-
 		if (char_tb[c] == CHAR_DECOS) {
 			p = get_deco(p, &d);
 			c = d;
@@ -1720,7 +1766,7 @@ unsigned char *parse_deco(unsigned char *p,
 }
 
 /* -- parse a decoration line (d: or s:) -- */
-static char *parse_decoline(unsigned char *p)
+static char *parse_decoline(char *p)
 {
 	struct abcsym *is;
 	unsigned char d;
@@ -1732,7 +1778,7 @@ static char *parse_decoline(unsigned char *p)
 
 	/* scan the decoration line */
 	while (*p != '\0') {
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 		if (*p == '\0')
 			break;
@@ -1768,43 +1814,110 @@ static char *parse_decoline(unsigned char *p)
 			syntax("'\\' ignored", p);
 			p++;
 			continue;
+		case '"': {
+			int more_gch;
+
+			p = parse_gchord(p + 1, &more_gch);
+			if (more_gch)
+				syntax("No end of guitar chord / annotation in deco line",
+				       p);
+			break;
+		    }
 		case '!':
 		case '+':
 			p = get_deco(p + 1, &d);
 			break;
 		default:
-			d = *p++;
+			d = (unsigned char) *p++;
 			break;
 		}
 
-		/* store the decoration in the next note */
+		/* store the decoration / gchord/annotation in the next note */
 		while (is != 0 && (is->type != ABC_T_NOTE
 				   || is->u.note.grace))
 			is = is->next;
 		if (is == 0)
 			return "Not enough notes for deco line";
 
-		n = is->u.note.dc.n;
-		if (n >= MAXDC)
-			syntax("Too many decorations for the note", p);
-		else if (d != 0) {
-			is->u.note.dc.t[n] = d;
-			is->u.note.dc.n = n + 1;
+		if (gchord != 0) {
+			if (is->text != 0) {
+				char *gch;
+
+				gch = alloc_f(strlen(is->text) + strlen(gchord) + 1);
+				strcpy(gch, is->text);
+				strcpy(gch + strlen(is->text), gchord);
+				if (free_f) {
+					free_f(gchord);
+					free_f(is->text);
+				}
+				gchord = gch;
+			}
+			is->text = gchord;
+			gchord = 0;
+		} else {
+			n = is->u.note.dc.n;
+			if (n >= MAXDC)
+				syntax("Too many decorations for the note", p);
+			else if (d != 0) {
+				is->u.note.dc.t[n] = d;
+				is->u.note.dc.n = n + 1;
+			}
 		}
 		is = is->next;
 	}
 	return 0;
 }
 
+/* -- parse a guitar chord / annotation -- */
+static char *parse_gchord(char *p,
+			  int *p_more_gch)
+{
+	char *q;
+	int l, l2, more_gch;
+
+	more_gch = 0;
+	q = p;
+	while (*p != '"') {
+		if (*p == '\0') {
+			more_gch = 1;
+			break;
+		}
+		p++;
+	}
+	l = p - q;
+	if (gchord) {
+		char *gch;
+
+		/* many guitar chords: concatenate with '\n' or ';' */
+		l2 = strlen(gchord);
+		gch = alloc_f(l2 + 1 + l + 1);
+		strcpy(gch, gchord);
+		gch[l2++] = more_gch ? ';' : '\n';
+		strncpy(&gch[l2], q, l);
+		gch[l2 + l] = '\0';
+		if (free_f)
+			free_f(gchord);
+		gchord = gch;
+	} else {
+		gchord = alloc_f(l + 1);
+		strncpy(gchord, q, l);
+		gchord[l] = '\0';
+	}
+	if (*p != '\0')
+		p++;
+	*p_more_gch = more_gch;
+	return p;
+}
+
 /* -- parse a note length -- */
-static unsigned char *parse_len(unsigned char *p,
-				int *p_len)
+static char *parse_len(char *p,
+			int *p_len)
 {
 	int len, fac;
-	unsigned char *q;
+	char *q;
 
 	len = BASE_LEN;
-	if (isdigit(*p)) {
+	if (isdigit((unsigned char) *p)) {
 		len *= strtol(p, (char **) &q, 10);
 		if (len <= 0) {
 			syntax("Bad length", p);
@@ -1815,7 +1928,7 @@ static unsigned char *parse_len(unsigned char *p,
 	fac = 1;
 	while (*p == '/') {
 		p++;
-		if (isdigit(*p)) {
+		if (isdigit((unsigned char) *p)) {
 			fac *= strtol(p, (char **) &q, 10);
 			p = q;
 		} else	fac *= 2;
@@ -1832,12 +1945,11 @@ static unsigned char *parse_len(unsigned char *p,
 /* -- parse a music line -- */
 /* return 1 at end of tune */
 static int parse_line(struct abctune *t,
-		      unsigned char *p)
+		      char *p)
 {
 	struct abcsym *s;
-	unsigned char *comment;
+	char *comment, *q, c;
 	struct abcsym *last_note_sav = 0;
-	unsigned char *q, c;
 	int i;
 	char sappo = 0;
 	static char qtb[10] = {0, 1, 3, 2, 3, 0, 2, 0, 3, 0};
@@ -1856,6 +1968,7 @@ again:					/* for history */
 			}
 			return 0;
 		}
+		t->abc_vers = abc_vers;
 		abc_state = ABC_S_GLOBAL;
 		abc_vers = abc_vers_g;
 		if (level_f)
@@ -1870,7 +1983,7 @@ again:					/* for history */
 			s->type = ABC_T_PSCOM;
 			s->state = abc_state;
 			p += 2;				/* skip '%%' */
-			while (isspace(*p))
+			while (isspace((unsigned) *p))
 				p++;
 			if (strncasecmp(p, "abc-version ", 12) == 0) {
 				abc_vers = atoi(p + 12);
@@ -1878,7 +1991,7 @@ again:					/* for history */
 			}
 			if (strncasecmp(p, "fmt ", 4) == 0) {
 				p += 4;			/* skip 'fmt' */
-				while (isspace(*p))
+				while (isspace((unsigned) *p))
 					p++;
 			}
 			begintype = '\0';
@@ -1899,11 +2012,11 @@ again:					/* for history */
 					if (*p != '%' || p[1] != '%')
 						continue;
 					p += 2;
-					while (isspace(*p))
+					while (isspace((unsigned) *p))
 						p++;
 					if (strncasecmp(p, "fmt ", 4) == 0) {
 						p += 4;
-						while (isspace(*p))
+						while (isspace((unsigned) *p))
 							p++;
 					}
 					if (begintype == 'p') {
@@ -1960,9 +2073,9 @@ again:					/* for history */
 		c = p[strlen(p) - 1];
 		if (c != '|' && c != ']')
 			return 0;
-		while (!isspace(*p) && *p != '\0')
+		while (!isspace((unsigned) *p) && *p != '\0')
 			p++;
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 	}
 	if (abc_state != ABC_S_TUNE) {
@@ -1974,60 +2087,29 @@ again:					/* for history */
 		return 0;
 	}
 
-	if (isspace(scratch_line[0]) && curvoice->last_note != 0)
+	if (isspace((unsigned) scratch_line[0]) && curvoice->last_note != 0)
 		curvoice->last_note->u.note.word_end = 1;
 
 	lyric_started = 0;
 	deco_start = deco_cont = 0;
 	while (*p != '\0') {
 		colnum = p - scratch_line;
-		switch (char_tb[*p++]) {
+		switch (char_tb[(unsigned) *p++]) {
 		case CHAR_GCHORD: {			/* " */
-			int l;
 			int more_gch;
 
-		gch_continue:
-			more_gch = 0;
-			q = p;
-			while (*p != '"') {
-				if (*p == '\0') {
-					more_gch = 1;
+			for (;;) {
+				p = parse_gchord(p, &more_gch);
+				if (!more_gch)
 					break;
-				}
-				p++;
-			}
-			l = p - q;
-			if (gchord) {
-				int l2;
-				char *gch;
-
-				/* many guitar chord: concatenate with '\n' or ';' */
-				l2 = strlen(gchord);
-				gch = alloc_f(l2 + 1 + l + 1);
-				strcpy(gch, gchord);
-				gch[l2++] = more_gch ? ';' : '\n';
-				strncpy(&gch[l2], q, l);
-				gch[l2 + l] = '\0';
-				if (free_f)
-					free_f(gchord);
-				gchord = gch;
-			} else {
-				gchord = alloc_f(l + 1);
-				strncpy(gchord, q, l);
-				gchord[l] = '\0';
-			}
-			if (*p != '\0')
-				p++;
-			else if (more_gch) {
 				if ((p = get_line()) == 0) {
 					syntax("EOF reached while parsing guitar chord",
-					       q);
-					return 1;
+					       p);
+					break;
 				}
-				goto gch_continue;
 			}
-		}
 			break;
+		    }
 		case CHAR_GRACE:		/* '{' or '}' */
 			if (p[-1] == '{') {
 				if (*p == '/') {
@@ -2088,7 +2170,7 @@ again:					/* for history */
 			q = p;
 			while (*q == '/')
 				q++;
-			if (char_tb[*q] != CHAR_BAR)
+			if (char_tb[(unsigned) *q] != CHAR_BAR)
 				goto bad_char;
 			syntax("Obsolete measure repeat syntax", p - 1);
 			s = abc_new(t, 0, 0);
@@ -2106,7 +2188,7 @@ again:					/* for history */
 			break;
 		case CHAR_OBRA:			/* '[' */
 			if (*p == '|' || *p == ']'
-			    || isdigit(*p) || *p == '"') {
+			    || isdigit((unsigned) *p) || *p == '"') {
 				p = parse_bar(t, p);
 				break;
 			}
@@ -2140,7 +2222,7 @@ again:					/* for history */
 			p = parse_bar(t, p);
 			break;
 		case CHAR_OPAR:			/* '(' */
-			if (isdigit(*p)) {
+			if (isdigit((unsigned) *p)) {
 				int pplet, qplet, rplet;
 
 				pplet = strtol(p, (char **) &q, 10);
@@ -2155,13 +2237,13 @@ again:					/* for history */
 				rplet = pplet;
 				if (*p == ':') {
 					p++;
-					if (isdigit(*p)) {
+					if (isdigit((unsigned) *p)) {
 						qplet = strtol(p, (char **) &q, 10);
 						p = q;
 					}
 					if (*p == ':') {
 						p++;
-						if (isdigit(*p)) {
+						if (isdigit((unsigned) *p)) {
 							rplet = strtol(p, (char **) &q, 10);
 							p = q;
 						}
@@ -2228,7 +2310,7 @@ again:					/* for history */
 			    || vover == 0) {		/*??*/
 				s = abc_new(t, 0, 0);
 				s->type = ABC_T_V_OVER;
-				/*s->u.v_over.type = V_OVER_N; */
+				/*s->u.v_over.type = V_OVER_V; */
 				vover_new();
 				s->u.v_over.voice = curvoice - voice_tb;
 				if (vover == 0)
@@ -2267,8 +2349,10 @@ again:					/* for history */
 				tie_pos = SL_AUTO;
 				break;
 			}
-			for (i = 0; i <= curvoice->last_note->u.note.nhd; i++)
-				curvoice->last_note->u.note.ti1[i] = tie_pos;
+			for (i = 0; i <= curvoice->last_note->u.note.nhd; i++) {
+				if (curvoice->last_note->u.note.ti1[i] == 0)
+					curvoice->last_note->u.note.ti1[i] = tie_pos;
+			}
 			if (curvoice->dotted_tie) {
 				curvoice->last_note->u.note.dotted_tie = 1;
 				curvoice->dotted_tie = 0;
@@ -2315,16 +2399,15 @@ again:					/* for history */
 	s = abc_new(t, 0, 0);
 	s->type = ABC_T_EOLN;
 	s->state = abc_state;
-
 	return 0;
 }
 
 /* -- parse a note or a rest -- */
-static unsigned char *parse_note(struct abctune *t,
-				 unsigned char *p)
+static char *parse_note(struct abctune *t,
+			char *p)
 {
 	struct abcsym *s;
-	unsigned char *q;
+	char *q;
 	int pit, len, acc, nostem, chord, j, m;
 
 	if (char_tb['{'] == CHAR_BAD) {	/* in a grace note sequence */
@@ -2353,8 +2436,8 @@ static unsigned char *parse_note(struct abctune *t,
 		s->type = ABC_T_MREST;
 		p++;
 		len = 1;
-		if (isdigit(*p)) {
-			len = strtol(p, (char **) &q, 10);
+		if (isdigit((unsigned) *p)) {
+			len = strtol(p, &q, 10);
 			p = q;
 		}
 		s->u.bar.type = 0;
@@ -2364,8 +2447,8 @@ static unsigned char *parse_note(struct abctune *t,
 		s->type = ABC_T_REST;
 		s->u.note.invis = 1;
 		p++;
-		if (isdigit(*p)) {	/* number of points */
-			s->u.note.lens[1] = strtol(p, (char **) &q, 10);
+		if (isdigit((unsigned) *p)) {	/* number of points */
+			s->u.note.lens[1] = strtol(p, &q, 10);
 			p = q;
 		} else	s->u.note.lens[1] = -1;
 		goto add_slurs;
@@ -2477,7 +2560,11 @@ static unsigned char *parse_note(struct abctune *t,
 			break;
 		if (*p == ']') {
 			p++;
-			if (*p == '/' || isdigit(*p)) {
+			if (*p == '0') {
+				nostem |= 1;
+				p++;
+			}
+			if (*p == '/' || isdigit((unsigned) *p)) {
 				p = parse_len(p, &len);
 				s->u.note.chlen = len;
 				for (j = 0; j < m; j++) {
@@ -2523,11 +2610,11 @@ add_deco:
 
 /* -- parse an information field -- */
 static void parse_info(struct abctune *t,
-		       unsigned char *p,
+		       char *p,
 		       char *comment)
 {
 	struct abcsym *s;
-	unsigned char info_type = *p;
+	char info_type = *p;
 	char *error_txt = 0;
 
 	s = abc_new(t, p, comment);
@@ -2535,7 +2622,7 @@ static void parse_info(struct abctune *t,
 	s->state = abc_state;
 
 	p += 2;
-	while (isspace(*p))
+	while (isspace((unsigned) *p))
 		p++;
 	switch (info_type) {
 	case 'd':
@@ -2608,7 +2695,7 @@ static void parse_info(struct abctune *t,
 
 /* -- print a syntax error message -- */
 static void syntax(char *msg,
-		   unsigned char *q)
+		   char *q)
 {
 	int n, len, m1, m2, pp;
 	int maxcol = 73;
@@ -2657,8 +2744,8 @@ static void vover_new(void)
 {
 	int voice, mvoice;
 
-	mvoice = curvoice - voice_tb;
-	for (voice = mvoice + 1; voice <= nvoice; voice++)
+	mvoice = curvoice->mvoice;
+	for (voice = curvoice - voice_tb + 1; voice <= nvoice; voice++)
 		if (voice_tb[voice].mvoice == mvoice)
 			break;
 	if (voice > nvoice) {
