@@ -107,7 +107,8 @@ struct abcsym {
 		} length;
 		struct meter_s {	/* M: info */
 			short wmeasure;		/* duration of a measure */
-			short nmeter;		/* number of meter elements */
+			char nmeter;		/* number of meter elements */
+			char expdur;		/* explicit measure duration */
 #define MAX_MEASURE 6
 			struct {
 				char top[8];	/* top value */
@@ -124,11 +125,13 @@ struct abcsym {
 			char *name;		/* name */
 			char *fname;		/* full name */
 			char *nname;		/* nick name */
+			float scale;		/* != 0 when change */
 			unsigned char voice;	/* voice number */
 			char merge;		/* merge with previous voice */
 			signed char stem;	/* have stems up or down (2 = auto) */
 			signed char gstem;	/* have grace stems up or down (2 = auto) */
 			signed char dyn;	/* have dynamic marks above or below the staff */
+			signed char lyrics;	/* have lyrics above or below the staff */
 		} voice;
 		struct {		/* bar, mrest or mrep */
 			struct deco dc;		/* decorations */
@@ -137,8 +140,10 @@ struct abcsym {
 			char len;		/* len if mrest or mrep */
 			char dotted;
 		} bar;
-		struct clef_s {		/* clef */
-			char type;
+		struct clef_s {		/* clef (and staff!) */
+			float staffscale;	/* != 0 when change */
+			signed char stafflines;	/* >= 0 when change */
+			signed char type;	/* no clef if < 0 */
 #define TREBLE 0
 #define ALTO 1
 #define BASS 2
@@ -188,6 +193,7 @@ struct abctune {
 	struct abctune *prev;	/* previous tune */
 	struct abcsym *first_sym; /* first symbol */
 	struct abcsym *last_sym; /* last symbol */
+	int abc_vers;		/* ABC version */
 	int client_data;	/* client data */
 };
 
@@ -212,14 +218,14 @@ void abc_init(void *alloc_f_api(int size),
 void abc_insert(char *file_api,
 		struct abcsym *s);
 struct abcsym *abc_new(struct abctune *t,
-		       unsigned char *p,
-		       unsigned char *comment);
+		       char *p,
+		       char *comment);
 struct abctune *abc_parse(char *file_api);
-char *get_str(unsigned char *d,
-	      unsigned char *s,
+char *get_str(char *d,
+	      char *s,
 	      int maxlen);
-unsigned char *parse_deco(unsigned char *p,
-			  struct deco *deco);
+char *parse_deco(char *p,
+		 struct deco *deco);
 #if defined(__cplusplus)
 }
 #endif
