@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2004 Jean-Franois Moine
+ * Copyright (C) 1998-2005 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -53,7 +53,7 @@ int use_buffer;			/* 1 if lines are being accumulated */
 /* -- convert a date -- */
 static void cnv_date(time_t *ltime)
 {
-	unsigned char buf[TEX_BUF_SZ];
+	char buf[TEX_BUF_SZ];
 
 	tex_str(cfmt.dateformat);
 	strcpy(buf, tex_buf);
@@ -74,7 +74,8 @@ static void init_ps(char *str, int is_epsf)
 				- cfmt.rightmargin + 20,
 			-bposy);
 		cur_lmarg = cfmt.leftmargin - 10;
-	} else	fprintf(fout, "%%!PS-Adobe-3.0\n");
+	} else	fprintf(fout, "%%!PS-Adobe-%d.0\n",
+			cfmt.pslevel);
 
 	fprintf(fout, "%%%%Title: %s\n", str);
 
@@ -165,6 +166,7 @@ void close_output_file(void)
 	fout = 0;
 	file_initialized = 0;
 	nbpages = tunenum = 0;
+	defl = 0;
 }
 
 /* -- close the PS page -- */
@@ -178,7 +180,6 @@ void close_page(void)
 		"showpage\n");
 	cur_lmarg = 0;
 	cur_scale = 1.0;
-	defl = -1;
 }
 
 /* -- output a header/footer element -- */
@@ -243,7 +244,7 @@ static void format_hf(char *p)
 			break;
 		case 'T':		/* tune title */
 			q = &info.title->as.text[2];
-			while (isspace(*q))
+			while (isspace((unsigned char) *q))
 				q++;
 			tex_str(q);
 			fprintf(fout, "%s", tex_buf);
@@ -489,7 +490,7 @@ void write_eps(void)
 	i = strlen(fnm) - 1;
 	if (fnm[i] == '=') {
 		p = &info.title->as.text[2];
-		while (isspace(*p))
+		while (isspace((unsigned char) *p))
 			p++;
 		strncpy(&fnm[i], p, sizeof fnm - i - 4);
 		fnm[sizeof fnm - 5] = '\0';
@@ -509,8 +510,8 @@ void write_eps(void)
 	m = ftell(fout);
 	fclose(fout);
 	fout = 0;
-	fprintf(stderr, "Output written on %s (%ld byte%s)\n",
-		fnm, m, m == 1 ? "" : "s");
+	fprintf(stderr, "Output written on %s (%ld bytes)\n",
+		fnm, m);
 	cur_lmarg = 0;
 	cur_scale = 1.0;
 }
@@ -571,7 +572,7 @@ void write_buffer(void)
 	int i, l, b2, np;
 	float p1, dp;
 
-	if (nbuf == 0)
+	if (nbuf == 0 || multicol_start != 0)
 		return;
 	i = 0;
 	p1 = 0;
@@ -611,6 +612,12 @@ void buffer_eob(void)
 {
 	if (bposy == 0 && nbuf == 0)
 		return;
+	if (ln_num >= BUFFLN) {
+		error(1, 0, "max number of buffer lines exceeded"
+			" -- check BUFFLN");
+		write_buffer();
+		use_buffer = 0;
+	}
 	ln_buf[ln_num] = nbuf;
 	ln_pos[ln_num] = bposy;
 	ln_lmarg[ln_num] = cfmt.leftmargin;
@@ -620,14 +627,8 @@ void buffer_eob(void)
 		write_buffer();
 		return;
 	}
-	if (ln_num >= BUFFLN && posy + bposy >= 0) {
-		error(1, 0, "max number of buffer lines exceeded"
-			" -- check BUFFLN");
-		bposy = -posy - 1;	/* force write_buffer() */
-	}
-	if ((posy + bposy < 0 || cfmt.oneperpage)
-	    && !epsf) {
-		if (tunenum != 1)
+	if (posy + bposy < 0 && !epsf && multicol_start == 0) {
+		if (tunenum > 1)
 			newpage();
 		write_buffer();
 		use_buffer = 0;
