@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 2000-2004, Jean-Franois Moine.
+ * Copyright (C) 2000-2005, Jean-Franois Moine.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,24 +30,23 @@
 
 int nbar;		/* current measure number */
 int nbar_rep;		/* last repeat bar number */
+
 int defl;		/* decoration flags */
-#define DEF_NOST 0x01		/* long deco with no start */
-#define DEF_NOEN 0x02		/* long deco with no end */
-#define DEF_STEMUP 0x04		/* stem up (1) or down (0) */
 
 static struct deco_elt {
 	struct deco_elt *next, *prev;	/* next/previous decoration */
 	struct SYMBOL *s;	/* symbol */
+	struct deco_elt *start;	/* start a long decoration ending here */
 	unsigned char t;	/* decoration index */
 	unsigned char staff;	/* staff */
 	unsigned char flags;
 #define DE_VAL	0x01		/* put extra value if 1 */
 #define DE_UP	0x02		/* above the staff */
-#define DE_TREATED 0x04		/* (for !decox(! - !decox)!) */
 #define DE_BELOW 0x08		/* below the staff */
 #define DE_GRACE 0x10		/* in grace note */
-#define DE_INV 0x20		/* invert the glyph if 1 */
-#define DE_FLAGS 0x40		/* output the decoration flags */
+#define DE_INV 0x20		/* invert the glyph */
+#define DE_LDST 0x40		/* start of long decoration */
+#define DE_LDEN 0x80		/* end of long decoration */
 	unsigned char defl;	/* decorations flags - see DEF_xx */
 	float x, y;		/* x, y */
 	float v;		/* extra value */
@@ -109,6 +108,7 @@ static struct deco_def_s {
 	{"fff",		6, 13, 18, 7, 17},
 	{"ffff",	6, 13, 18, 8, 24},
 	{"mf",		6, 13, 18, 5, 11},
+	{"mp",		6, 13, 18, 5, 11},
 	{"mordent",	3,  5, 10},
 	{"open",	3, 30, 10},
 	{"p",		6, 13, 18, 4, 4},
@@ -124,7 +124,7 @@ static struct deco_def_s {
 	{"trill)",	5, 28, 8},
 	{"snap",	3, 31, 14},
 	{"thumb",	3, 15, 14},
-	{"arpeggio",	2, 27, 0, 10},
+	{"arpeggio",	2, 27, 12, 10},
 	{"crescendo(",	7, -1, 18},
 	{"crescendo)",	7, 19, 18},
 	{"<(",		7, -1, 18},
@@ -136,6 +136,11 @@ static struct deco_def_s {
 	{">",		3, 21, 8},
 	{"invisible",	32},
 	{"beamon",	33},
+	{"trem1",	34},
+	{"trem2",	34},
+	{"trem3",	34},
+	{"trem4",	34},
+	{"xstem",	35},
 };
 
 /* c function table */
@@ -194,15 +199,50 @@ static char *str_tb[32] = {
 
 static struct SYMBOL *first_note;	/* first note/rest of the line */
 
-static void draw_gchord(struct SYMBOL *s, float gchy, int *fonttype);
+static void draw_gchord(struct SYMBOL *s, float gchy);
 
 /* -- get the max/min vertical offset -- */
-static float get_y(struct SYMBOL *s,
-		   int up,
-		   float x,
-		   float w,
-		   float h)
+float y_get(struct SYMBOL *s,
+	    int up,
+	    float x,
+	    float w,
+	    float h)
 {
+#ifdef YSTEP
+	struct STAFF_S *p_staff;
+	int i, j;
+	float y;
+
+	p_staff = &staff_tb[s->staff];
+	i = (int) (x / realwidth * YSTEP);
+if (i < 0) {
+fprintf(stderr, "y_get i:%d\n", i);
+i = 0;
+}
+	j = (int) ((x + w) / realwidth * YSTEP);
+	if (j >= YSTEP) {
+/*fprintf(stderr, "y_get i:%d", j);*/
+		j = YSTEP - 1;
+		if (i > j)
+			i = j;
+	}
+	if (up) {
+		y = p_staff->top[i++];
+		while (i <= j) {
+			if (y < p_staff->top[i])
+				y = p_staff->top[i];
+			i++;
+		}
+	} else {
+		y = p_staff->bot[i++];
+		while (i <= j) {
+			if (y > p_staff->bot[i])
+				y = p_staff->bot[i];
+			i++;
+		}
+		y -= h;
+	}
+#else
 	struct SYMBOL *s2, *s_start;
 	int staff, start_seen;
 	float y;
@@ -214,7 +254,15 @@ static float get_y(struct SYMBOL *s,
 	else {
 		while (s->ts_prev != 0 && s->ts_prev->x >= x)
 			s = s->ts_prev;
-		start_seen = 0;
+		while (s->staff != staff) {
+			if (s->ts_prev == 0) {
+				while (s->staff != staff)
+					s = s->ts_next;
+				break;
+			}
+			s = s->ts_prev;
+		}
+		start_seen = s == s_start;
 	}
 	y = up ? s->dc_top : s->dc_bot;
 
@@ -230,8 +278,6 @@ static float get_y(struct SYMBOL *s,
 				y = s->ymn;
 		}
 		s = s->ts_next;
-		if (s == s_start)
-			start_seen = 1;
 	}
 	x += w;		/* right offset */
 	if (up) {
@@ -260,16 +306,48 @@ static float get_y(struct SYMBOL *s,
 		}
 		y -= h;
 	}
+#endif
 	return y;
 }
 
 /* -- adjust the vertical offsets -- */
-static void set_y(struct SYMBOL *s,
-		  int up,
-		  float x,
-		  float w,
-		  float y)
+void y_set(struct SYMBOL *s,
+	   int up,
+	   float x,
+	   float w,
+	   float y)
 {
+#ifdef YSTEP
+	struct STAFF_S *p_staff;
+	int i, j;
+
+	p_staff = &staff_tb[s->staff];
+	i = (int) (x / realwidth * YSTEP);
+if (i < 0) {
+fprintf(stderr, "y_set i:%d\n", i);
+i = 0;
+}
+	j = (int) ((x + w) / realwidth * YSTEP);
+	if (j >= YSTEP) {
+/*fprintf(stderr, "y_get i:%d", j);*/
+		j = YSTEP - 1;
+		if (i > j)
+			i = j;
+	}
+	if (up) {
+		while (i <= j) {
+			if (p_staff->top[i] < y)
+				p_staff->top[i] = y;
+			i++;
+		}
+	} else {
+		while (i <= j) {
+			if (p_staff->bot[i] > y)
+				p_staff->bot[i] = y;
+			i++;
+		}
+	}
+#else
 	struct SYMBOL *s_start;
 	int staff, start_seen;
 
@@ -307,6 +385,7 @@ static void set_y(struct SYMBOL *s,
 				s->dc_bot = y;
 		}
 	}
+#endif
 }
 
 /* -- get the staff position of the dynamic marks -- */
@@ -320,7 +399,8 @@ static int dyn_p(struct SYMBOL *s)
 		return s->multi > 0 ? 1 : 0;
 	if (cfmt.exprabove
 	    || (!cfmt.exprbelow
-		&& staff_tb[s->staff].nvocal != 0))
+/*fixme: only if the lyrics are below the staff*/
+		&& voice_tb[s->voice].have_ly))
 		return 1;
 	return 0;
 }
@@ -353,8 +433,9 @@ static void d_arp(struct deco_elt *de)
 			xc = dx;
 	}
 	h = 3 * (s->pits[s->nhd] - s->pits[0]) + 4;
-	if (h < 18)
-		h = 18;
+	m = dd->h;		/* minimum height */
+	if (h < m)
+		h = m;
 
 	de->flags |= DE_VAL;
 	de->v = h;
@@ -362,55 +443,36 @@ static void d_arp(struct deco_elt *de)
 	de->y = (float) (3 * (s->pits[0] - 18)) - 3;
 }
 
-/* special case for (de)crescendo */
+/* special case for crescendo/diminuendo */
 static void d_cresc(struct deco_elt *de)
 {
 	struct SYMBOL *s, *s2;
 	struct deco_def_s *dd, *dd2;
 	struct deco_elt *de1;
-	int voice, up;
+	int up;
 	float x, dx, x2;
 
-	if (de->flags & DE_TREATED)
+	if (de->flags & DE_LDST)
 		return;
-	s = de->s;
+	s2 = de->s;
 	dd = &deco_def_tb[de->t];
-	voice = s->voice;
-	de1 = de;
-
-	if (dd->ps_func < 0) {		/* start of (de)crescendo */
-		int t;
-
-		/* !! works when '(de)crescendo)' is '(de)crescendo(' + 1 !! */
-		t = de->t + 1;
-		for (de = de->next; de != 0; de = de->next)
-			if (de->t == t && de->s->voice == voice)
-				break;
-		if (de == 0) {		/* no end, insert one */
-			de = (struct deco_elt *) getarena(sizeof *de);
-			memset(de, 0, sizeof *de);
-			deco_tail->next = de;
-			deco_tail = de;
-			de->s = s;
-			de->t = t;
-			de->defl = DEF_NOEN;
-		}
-		s2 = de->s;
+	de1 = de->start;		/* start of the deco */
+	if (de1 != 0) {
+		s = de1->s;
 		x = s->x + 3;
 	} else {			/* end without start */
-		s2 = s;
 		s = first_note;
 		x = s->x - s->wl - 4;
-		de->defl = DEF_NOST;
 	}
 	de->staff = s2->staff;
-	de->flags |= DE_VAL|DE_TREATED|DE_FLAGS;
+	de->flags &= ~DE_LDEN;		/* old behaviour */
+	de->flags |= DE_VAL;
 	up = dyn_p(s2);
 	if (up)
 		de->flags |= DE_UP;
 
 	/* shift the starting point if any dynamic mark on the left */
-	if (de1->prev != 0 && de1->prev->s == s
+	if (de1 != 0 && de1->prev != 0 && de1->prev->s == s
 	    && ((de->flags ^ de1->prev->flags) & DE_UP) == 0) {
 		dd2 = &deco_def_tb[de1->prev->t];
 		if (dd2->func >= 6) {
@@ -420,26 +482,33 @@ static void d_cresc(struct deco_elt *de)
 		}
 	}
 
-	if (s2 == s
-	    && dd->ps_func < 0) {	/* if no decoration end */
-		dx = realwidth - x - 6;
+	if (de->defl & DEF_NOEN) {	/* if no decoration end */
+		dx = de->x - x;
 		if (dx < 20) {
-			x = realwidth - 20 - 6;
+			x = de->x - 20 - 3;
 			dx = 20;
 		}
 	} else {
-		dx = s2->x - x - 4;
-		if (s2->type == NOTE
-		    && s2->as.u.note.dc.n > 1)
-			dx -= 6;
-		if (dx < 20)
+		x2 = s2->x;
+		if (de->next != 0 && de->next->s == s
+		    && ((de->flags ^ de->next->flags) & DE_UP) == 0) {
+			dd2 = &deco_def_tb[de->next->t];
+			if (dd2->func >= 6)	/* if dynamic mark */
+			x2 -= 5;
+		}
+		dx = x2 - x - 4;
+		if (dx < 20) {
+			x -= (20 - dx) * 0.5;
+			if (de->start == 0)
+				x -= (20 - dx) * 0.5;
 			dx = 20;
+		}
 	}
 
 	de->v = dx;
 	de->x = x;
-	de->y = get_y(s, up, x, dx, dd->h);
-	/* (set_y is done in draw_deco_staff) */
+	de->y = y_get(s2, up, x, dx, dd->h);
+	/* (y_set is done later in draw_deco_staff) */
 }
 
 /* near the note (dot, tenuto) */
@@ -455,20 +524,20 @@ static void d_near(struct deco_elt *de)
 	if (s->multi)
 		up = !up;
 	if (up)
-		y = s->dc_top + 3;
-	else	y = s->dc_bot - 3;
+		y = s->ymx + 3;
+	else	y = s->ymn - 3;
 	if (y > -3 && y < 24 + 3) {
 		if (up)
 			y++;
 		y = (y + 5) / 6 * 6 - 3;	/* between lines */
 	}
-	if (s->dc_top < y + dd->h)
-		s->dc_top = y + dd->h;
-	if (s->dc_bot > y - dd->h)
-		s->dc_bot = y - dd->h;
+	if (s->ymx < y + dd->h)
+		s->ymx = y + dd->h;
+	if (s->ymn > y - dd->h)
+		s->ymn = y - dd->h;
 
 	de->x = s->x + s->shhd[s->stem >= 0 ? 0 : s->nhd];
-	de->y = y;
+	de->y = (float) y;
 }
 
 /* special case for piano/forte indications */
@@ -491,7 +560,7 @@ static void d_pf(struct deco_elt *de)
 	if (de->prev != 0 && de->prev->s == s
 	    && ((de->flags ^ de->prev->flags) & DE_UP) == 0) {
 		dd2 = &deco_def_tb[de->prev->t];
-		if (dd2->func >= 6) {
+		if (dd2->func >= 6) {	/* if dynamic mark */
 			x2 = de->prev->x + de->prev->v + 4;
 			if (x2 > x)
 				x = x2;
@@ -504,9 +573,9 @@ static void d_pf(struct deco_elt *de)
 
 	de->v = dd->wl + dd->wr;
 	de->x = x;
-	de->y = get_y(s, up, s->x, de->v, dd->h);
+	de->y = y_get(s, up, s->x, de->v, dd->h);
 	de->str = str;
-	/* (set_y is done in draw_deco_staff) */
+	/* (y_set is done later in draw_deco_staff) */
 }
 
 /* special case for slide */
@@ -545,78 +614,55 @@ static void d_trill(struct deco_elt *de)
 {
 	struct SYMBOL *s;
 	struct deco_def_s *dd;
-	int staff, voice, up;
+	int staff, up;
 	float x, y, dx;
 	struct SYMBOL *s2;
 
-	if (de->flags & DE_TREATED)
+	if (de->flags & DE_LDST)
 		return;
-	s = de->s;
+	s2 = de->s;
 	dd = &deco_def_tb[de->t];
-	voice = s->voice;
 
-	if (dd->ps_func < 0) {		/* start of trill */
-		int t;
-
-		/* !! works when 'trill)' is 'trill(' + 1 !! */
-		t = de->t + 1;
-		for (de = de->next; de != 0; de = de->next)
-			if (de->t == t && de->s->voice == voice)
-				break;
-		if (de == 0) {		/* no end, insert one */
-			de = (struct deco_elt *) getarena(sizeof *de);
-			memset(de, 0, sizeof *de);
-			deco_tail->next = de;
-			deco_tail = de;
-			de->s = s;
-			de->t = t;
-			de->defl = DEF_NOEN;
-		}
-		s2 = de->s;
+	if (de->start != 0) {		/* deco start */
+		s = de->start->s;
 		x = s->x;
 		if (s->type == NOTE
 		    && s->as.u.note.dc.n > 1)
 			x += 10;
 	} else {			/* end without start */
-		s2 = s;
 		s = first_note;
 		x = s->x - s->wl - 4;
-		de->defl = DEF_NOST;
 	}
 	de->staff = staff = s2->staff;
 
 	up = s2->multi >= 0;
-	if (s2 == s) {			/* if no decoration end */
-		dx = realwidth - x - 6;
+	if (de->defl & DEF_NOEN) {	/* if no decoration end */
+		dx = de->x - x;
 		if (dx < 20) {
-			x = realwidth - 20 - 6;
+			x = de->x - 20 - 3;
 			dx = 20;
 		}
 	} else {
 		dx = s2->x - x - 6;
 		if (s2->type == NOTE)
 			dx -= 6;
-		if (dx < 20)
+		if (dx < 20) {
+			x -= (20 - dx) * 0.5;
+			if (de->start == 0)
+				x -= (20 - dx) * 0.5;
 			dx = 20;
+		}
 	}
 
-	y = get_y(s, up, x, dx, dd->h);
-	if (up) {
-		if (y < 24 + 2)
-			y = 24 + 2;
-	} else {
-		if (y > -2)
-			y = -2;
-	}
-
-	de->flags |= DE_VAL|DE_TREATED|DE_FLAGS;
+	y = y_get(s2, up, x, dx, dd->h);
+	de->flags &= ~DE_LDEN;
+	de->flags |= DE_VAL;
 	de->v = dx;
 	de->x = x;
 	de->y = y;
-
 	if (up)
 		y += dd->h;
-	set_y(s, up, x, dx, y);
+	y_set(s2, up, x, dx, y);
 }
 
 /* above (or below) the staff */
@@ -637,20 +683,36 @@ static void d_upstaff(struct deco_elt *de)
 	case 2:		/* roll */
 		if (s->multi < 0
 		    || (s->multi == 0 && s->stem > 0)) {
+#ifdef YSTEP
+			yc = y_get(s, 0, s->x, 0, 0);
+#else
 			yc = s->dc_bot;
+#endif
 			if (yc > -2)
 				yc = -2;
 			yc -= dd->h;
+#ifdef YSTEP
+			y_set(s, 0, s->x, 0, yc);
+#else
 			s->dc_bot = yc;
+#endif
 			inv = 1;
 		} else {
+#ifdef YSTEP
+			yc = y_get(s, 1, s->x, 0, 0) + 3;
+#else
 			yc = s->dc_top + 3;
+#endif
 			if (yc < 24 + 2)
 				yc = 24 + 2;
 			if (s->stem <= 0
 			    && (s->dots == 0 || ((int) s->y % 6)))
 				yc -= 2;
+#ifdef YSTEP
+			y_set(s, 1, s->x, 0, yc + dd->h);
+#else
 			s->dc_top = yc + dd->h;
+#endif
 		}
 		break;
 	case 15:	/* breath */
@@ -678,16 +740,32 @@ static void d_upstaff(struct deco_elt *de)
 		if (s->multi >= 0
 		    && de->t != 19	/* invertedfermata */
 		    && !(de->flags & DE_BELOW)) {
+#ifdef YSTEP
+			yc = y_get(s, 1, s->x, 0, 0);
+#else
 			yc = s->dc_top;
+#endif
 			if (yc < 24 + 2)
 				yc = 24 + 2;
+#ifdef YSTEP
+			y_set(s, 1, s->x, 0, yc + dd->h);
+#else
 			s->dc_top = yc + dd->h;
+#endif
 		} else {
+#ifdef YSTEP
+			yc = y_get(s, 0, s->x, 0, 0);
+#else
 			yc = s->dc_bot;
+#endif
 			if (yc > -2)
 				yc = -2;
 			yc -= dd->h;
+#ifdef YSTEP
+			y_set(s, 0, s->x, 0, yc);
+#else
 			s->dc_bot = yc;
+#endif
 			switch (de->t) {
 			case 3:		/* fermata */
 			case 19:	/* invertedfermata */
@@ -697,11 +775,10 @@ static void d_upstaff(struct deco_elt *de)
 		}
 		break;
 	}
-	if (inv)
+	if (inv) {
 		yc += dd->h;
-
-	if (inv)
 		de->flags |= DE_INV;
+	}
 	de->x = x;
 	de->y = yc;
 }
@@ -803,7 +880,7 @@ void deco_add(char *text)
 void deco_cnv(struct deco *dc,
 	      struct SYMBOL *s)
 {
-	int i;
+	int i, j;
 	struct deco_def_s *dd;
 	unsigned char deco;
 	struct VOICE_S *p_voice;
@@ -823,17 +900,48 @@ void deco_cnv(struct deco *dc,
 
 		/* special decorations */
 		dd = &deco_def_tb[deco];
-		if (dd->func >= 32) {
-			if (dd->func == 32) {		/* 32 = invisible */
-				if (s->type != NOTE)
-					error(1, s,
-					      "Cannot have +%s+ on a rest or a bar",
-					       dd->name);
-				else	s->as.u.note.invis = 1;
-			} else	s->sflags |= S_BEAM_ON;	/* 33 = beamon */
-			dc->t[i] = 0;
+		switch (dd->func) {
+		default:
 			continue;
+		case 32:		/* 32 = invisible */
+			if (s->type != NOTE)
+				error(1, s,
+				      "Cannot have +%s+ on a rest or a bar",
+				       dd->name);
+			else	s->as.u.note.invis = 1;
+			break;
+		case 33:		/* 33 = beamon */
+			s->sflags |= S_BEAM_ON;
+			break;
+		case 34:		/* 34 = trem1..trem4 */
+			if (s->type != NOTE || s->prev->type != NOTE) {
+				error(1, s,
+				      "+%s+ must be on the last of a couple of notes",
+				       dd->name);
+				break;
+			}
+			s->sflags |= S_TREM;
+			s->sflags &= ~S_WORD_ST;
+			s->as.u.note.word_end = 1;
+			s->prev->sflags |= (S_TREM | S_WORD_ST);
+			s->prev->as.u.note.word_end = 0;
+			s->nflags = s->prev->nflags = dd->name[4] - '0';
+			for (j = 0; j <= s->nhd; j++)
+				s->as.u.note.lens[j] *= 2;
+			for (j = 0; j <= s->prev->nhd; j++)
+				s->prev->as.u.note.lens[j] *= 2;
+			break;
+		case 35:		/* 35 = xstem */
+			if (s->type != NOTE) {
+				error(1, s,
+				      "Cannot have +%s+ on a rest or a bar",
+				       dd->name);
+				break;
+			}
+			s->sflags |= S_XSTEM;
+			break;
 		}
+		dc->t[i] = 0;
 	}
 	p_voice = &voice_tb[s->voice];
 	if (p_voice->dyn != 0)
@@ -906,13 +1014,13 @@ void draw_all_deco(void)
 {
 	struct deco_elt *de;
 	struct deco_def_s *dd;
-	int f, staff, fl;
-	float y, y2, ym;
+	int f, staff;
+	float x, y, y2, ym;
 	float ymid[MAXSTAFF];
 
 	if (!cfmt.dynalign) {
 		staff = nstaff;
-		y = staff_tb[nstaff].y;
+		y = staff_tb[staff].y;
 		while (--staff >= 0) {
 			y2 = staff_tb[staff].y;
 			ymid[staff] = (y + 24 + y2) * 0.5;
@@ -943,35 +1051,37 @@ void draw_all_deco(void)
 				s = de->s;
 				while (s->staff != staff)
 					s = s->ts_prev;
-				y = get_y(s, !(de->flags & DE_UP),
+				y2 = y_get(s, !(de->flags & DE_UP),
 					   de->x, de->v, dd->h)
 					+ staff_tb[staff].y;
-				if (de->flags & DE_UP) {
-					if (y > ym)
-						y = ym;
-				} else if (y < ym)
+				if (((de->flags & DE_UP) && y2 > ym)
+				    || (!(de->flags & DE_UP) && y2 < ym)) {
 					y = ym;
-				set_y(de->s, de->flags & DE_UP,
-				      de->x, de->v,
-				      ((de->flags & DE_UP) ? y + dd->h : y)
-					- staff_tb[de->staff].y);
+					y_set(de->s, de->flags & DE_UP,
+					      de->x, de->v,
+					      ((de->flags & DE_UP) ? y + dd->h : y)
+						- staff_tb[de->staff].y);
+				}
 			}
 		}
 
-		fl = (de->flags & DE_FLAGS) ? de->defl : defl;
-		if (de->s->stem >= 0)
-			fl |= DEF_STEMUP;
-		else	fl &= ~DEF_STEMUP;
-
-		if (defl != fl) {
-			defl = fl;
-			PUT1("/defl %d def ", fl);
-		}
+		set_scale(de->s->staff);
+		set_defl(de->defl);
+/*fixme: scaled or not?*/
 		if (de->flags & DE_VAL)
-			PUT1("%.1f ", de->v);
+			putf(de->v);
 		if (de->str)
 			PUT1("(%s)", de->str);
-		PUT2("%.1f %.1f ", de->x, y);
+		putxy(de->x, y);
+		if (de->flags & DE_LDEN) {
+			if (de->start != 0) {
+				x = de->start->x;
+				y = de->start->y + staff_tb[de->start->staff].y;
+			} else	x = first_note->x - first_note->wl - 4;
+			if (x > de->x - 20)
+				x = de->x - 20;
+			putxy(x, y);
+		}
 		if (de->flags & DE_GRACE) {
 			if (de->flags & DE_INV)
 				PUT1("gsave T 0.7 -0.7 scale 0 0 %s grestore\n", ps_func_tb[f]);
@@ -982,11 +1092,12 @@ void draw_all_deco(void)
 			else	PUT1("%s\n", ps_func_tb[f]);
 		}
 	}
+	set_scale(-1);			/* restore the scale */
 }
 
 /* -- draw a decoration relative to a note head -- */
 /* return 1 if the decoration is a head */
-int draw_deco_head(int deco, float x, float y)
+int draw_deco_head(int deco, float x, float y, int stem)
 {
 	struct deco_def_s *dd;
 	char *str;
@@ -996,6 +1107,15 @@ int draw_deco_head(int deco, float x, float y)
 	dd = &deco_def_tb[deco];
 	if (dd->ps_func < 0)
 		return 0;
+	if (cfmt.setdefl) {
+		int fl;
+
+		fl = stem >= 0 ? DEF_STEMUP : 0;
+		if (defl != fl) {
+			defl = fl;
+			PUT1("/defl %d def ", fl);
+		}
+	}
 	switch (dd->func) {
 	case 2:
 	case 5:
@@ -1014,7 +1134,8 @@ int draw_deco_head(int deco, float x, float y)
 		PUT1("(%s)", str);
 		break;
 	}
-	PUT3("%.1f %.1f %s ", x, y, ps_func_tb[dd->ps_func]);
+	putxy(x, y);
+	PUT1("%s ", ps_func_tb[dd->ps_func]);
 	return strncmp(dd->name, "head-", 5) == 0;
 }
 
@@ -1036,7 +1157,7 @@ void draw_all_deco_head(struct SYMBOL *s, float x, float y)
 
 		if (strncmp(dd->name, "head-", 5) != 0)
 			continue;
-		draw_deco_head(deco, x, y);
+		draw_deco_head(deco, x, y, s->stem);
 	}
 }
 
@@ -1044,7 +1165,7 @@ void draw_all_deco_head(struct SYMBOL *s, float x, float y)
 static void deco_create(struct SYMBOL *s,
 			struct deco *dc)
 {
-	int k;
+	int k, l;
 	unsigned char deco;
 	struct deco_def_s *dd;
 	struct deco_elt *de;
@@ -1116,6 +1237,22 @@ static void deco_create(struct SYMBOL *s,
 		de->staff = s->staff;
 		if (s->type == NOTE && s->as.u.note.grace)
 			de->flags = DE_GRACE;
+		l = strlen(dd->name) - 1;
+		if (l > 0) {
+			switch (dd->name[l]) {
+			case '(':
+				de->flags |= DE_LDST;
+				break;
+			case ')':
+				if (strchr(dd->name, '(') != 0)
+					break;
+				de->flags |= DE_LDEN;
+				de->defl = DEF_NOST;
+				break;
+			}
+		}
+		if (cfmt.setdefl && s->stem >= 0)
+			de->defl |= DEF_STEMUP;
 
 		if (dd->func >= 3)	/* if not near the note */
 			continue;
@@ -1177,13 +1314,35 @@ void draw_deco_near(void)
 /* (the staves are not yet defined) */
 void draw_deco_note(void)
 {
-	struct deco_elt *de;
+	struct deco_elt *de, *de2;
+	struct deco_def_s *dd;
+	int f, t, voice;
 
 	for (de = deco_head; de != 0; de = de->next) {
-		struct deco_def_s *dd;
-		int f;
-
-		dd = &deco_def_tb[de->t];
+		t = de->t;
+		dd = &deco_def_tb[t];
+		if (de->flags & DE_LDST) {	/* start of long decoration */
+			t++;
+			voice = de->s->voice;
+			for (de2 = de->next; de2 != 0; de2 = de2->next)
+				if (de2->t == t && de2->s->voice == voice)
+					break;
+			if (de2 == 0) {		/* no end, insert one */
+				de2 = (struct deco_elt *) getarena(sizeof *de2);
+				memset(de2, 0, sizeof *de2);
+				de2->prev = deco_tail;
+				deco_tail->next = de2;
+				deco_tail = de2;
+				de2->s = de->s;
+				de2->t = t;
+				de2->defl = DEF_NOEN;
+				de2->flags = DE_LDEN;
+				de2->x = realwidth - 6;
+				de2->y = de->s->y;
+			}
+			de2->start = de;
+			de2->defl &= ~DEF_NOST;
+		}
 		f = dd->func;
 		if (f < 3 || f >= 6)
 			continue;	/* not tied to the note */
@@ -1201,24 +1360,15 @@ void draw_deco_staff(void)
 	struct VOICE_S *p_voice;
 	float x, y, w;
 	struct deco_elt *de;
-	int voice;
 	int some_gchord;
 	struct {
 		float ymin, ymax;
 	} minmax[MAXSTAFF];
 
-	/* set the top and bottom of all symbols out of the staves */
-	for (s = first_voice->sym; s != 0; s = s->ts_next) {
-		if (s->dc_top < 24 + 2)
-			s->dc_top = 24 + 2;
-		if (s->dc_bot > -2)
-			s->dc_bot = -2;
-	}
-
 	/* search the vertical offset for the guitar chords */
 	memset(minmax, 0, sizeof minmax);
 	some_gchord = 0;
-	for (s = first_voice->sym->next; s != 0; s = s->ts_next) {
+	for (s = first_voice->sym; s != 0; s = s->ts_next) {
 		char *p;
 
 		if (s->as.text == 0)
@@ -1240,21 +1390,22 @@ void draw_deco_staff(void)
 		    || (p = strchr(s->as.text, ';')) != 0)
 			w *= p - s->as.text;
 		else	w *= strlen(s->as.text);
-		y = get_y(s, 1, s->x, w, 0);
+		y = y_get(s, 1, s->x, w, 0);
 		if (y > minmax[s->staff].ymax)
 			minmax[s->staff].ymax = y;
 	}
 
 	/* draw the guitar chords if any */
 	if (some_gchord) {
-		int i, fonttype = 0;
+		int i;
 
 		for (i = nstaff; i >= 0; i--) {
 			minmax[i].ymax += 4;
 			if (minmax[i].ymax < 34)
 				minmax[i].ymax = 34;
 		}
-		for (s = first_voice->sym->next; s != 0; s = s->ts_next) {
+		strcf = -1;
+		for (s = first_voice->sym; s != 0; s = s->ts_next) {
 			if (s->as.text == 0)
 				continue;
 			switch (s->type) {
@@ -1268,30 +1419,10 @@ void draw_deco_staff(void)
 			default:
 				continue;
 			}
-			draw_gchord(s, minmax[s->staff].ymax, &fonttype);
+			draw_gchord(s, minmax[s->staff].ymax);
 		}
 	}
 
-	/* compute the max number of lyrics */
-	if (!cfmt.musiconly) {
-		for (s = first_voice->sym->next; s != 0; s = s->ts_next) {
-			struct lyrics *ly;
-			int nlyric;
-
-			if ((ly = s->ly) == 0)
-				continue;
-			for (nlyric = MAXLY; --nlyric >= 0; )
-				if (ly->lyl[nlyric] != 0)
-					break;
-			nlyric++;
-			voice = s->voice;
-			if (voice_tb[voice].nvocal < nlyric)
-				voice_tb[voice].nvocal = nlyric;
-		}
-		for (p_voice = first_voice; p_voice; p_voice = p_voice->next)
-			staff_tb[p_voice->staff].nvocal += p_voice->nvocal;
-	}
-
 	/* draw the repeat brackets */
 	for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
 		struct SYMBOL *s1, *s2, *first_repeat;
@@ -1331,17 +1462,37 @@ void draw_deco_staff(void)
 						break;
 				}
 			}
-			y2 = get_y(s1, 1, s1->x, s->x - s1->x, 0);
-			if (y2 > y)
+			y2 = y_get(s1, 1, s1->x, s->x - s1->x, 0);
+			if (y < y2)
 				y = y2;
 
 			/* have room for the repeat numbers */
-			y2 = s1->next->dc_top;
-			if (s1->next->as.text != 0)	/* if guitar chord */
-				y2 -= cfmt.font_tb[s->gcf].size; /* got already */
-			if (y2 > 24 + 6 + 4)
-				y = y2 + 10 + 4;
-
+			if (s1->as.text != 0)
+			    for (s2 = s1->next; s2 != s; s2 = s2->next) {
+				switch (s2->type) {
+				case PART:
+				case TEMPO:
+				case FMTCHG:
+				case TUPLET:
+				case WHISTLE:
+					continue;
+				case NOTE:
+				case REST:
+#ifdef YSTEP
+					y2 = y_get(s2, 1, s2->x, 3, 0);
+#else
+					y2 = s2->dc_top;
+#endif
+					if (s2->as.text != 0)	/* if guitar chord */
+						y2 -= cfmt.font_tb[s->gcf].size; /* got already */
+					y2 += cfmt.repeatfont.size;
+					if (y < y2)
+						y = y2;
+				default:
+					break;
+				}
+				break;
+			}
 			if (s->as.u.bar.repeat_bar)
 				s = s->prev;
 		}
@@ -1385,14 +1536,14 @@ void draw_deco_staff(void)
 			x = s1->x;
 			if ((s1->as.u.bar.type & 0x0f) == B_COL)
 				x -= 4;
-			i = 0;			/* no end of bracket */
+			i = 0;			/* no bracket end */
 			w = s2->x - x - 8;
 			if (s2->type != BAR)
 				w = realwidth - x - 4;
 			else if (((s2->as.u.bar.type & 0xf0)	/* if complex bar */
 				   && s2->as.u.bar.type != (B_OBRA << 4) + B_CBRA)
 				 || s2->as.u.bar.type == B_CBRA) {
-				i =  2;
+				i =  2;		/* bracket start and stop */
 				if ((s2->as.u.bar.type & 0x0f) == B_COL)
 					w -= 4;
 				else if (!(s2->sflags & S_RRBAR)
@@ -1405,19 +1556,20 @@ void draw_deco_staff(void)
 			}
 			p = s1->as.text;
 			if (p == 0) {
-				i--;		/* no start of bracket */
+				i--;		/* no bracket start (1) or not draw */
 				p = "";
 			}
 			if (i == 0 && s2->next == 0) {	/* 2nd ending at end of line */
 				if (p_voice->bar_start != 0)
-					i = 2;
+					i = 2;		/* bracket start and stop */
 				else	repnl = 1;	/* continue on next line */
 			}
 			if (i >= 0) {
 				PUT2("(%s)%d ", p, i);
-				PUT4("%.1f %.1f %.1f y%d repbra\n",
-				     w, x, y, s1->staff);
-				set_y(s1, 1, x, w, y + 2.);
+				putf(w);
+				putxy(x, y);
+				PUT1("y%d repbra\n", s1->staff);
+				y_set(s1, 1, x, w, y + 2.);
 			}
 			if (s->as.u.bar.repeat_bar)
 				s = s->prev;
@@ -1466,15 +1618,14 @@ void draw_deco_staff(void)
 		} else	y = de->y;
 		if (de->flags & DE_UP)
 			y += dd->h;
-		set_y(de->s, de->flags & DE_UP, de->x, de->v, y);
+		y_set(de->s, de->flags & DE_UP, de->x, de->v, y);
 	}
 }
 
 /* -- draw the guitar chords and annotations -- */
 /* (the staves are not yet defined) */
 static void draw_gchord(struct SYMBOL *s,
-			float gchy,
-			int *fonttype)
+			float gchy)
 {
 	float x, y, xspc, yspca, yspcc, gchyb, gchyl, gchyr;
 	float xmin, xmax, ymin, ymax;
@@ -1482,11 +1633,15 @@ static void draw_gchord(struct SYMBOL *s,
 	char *p, *q, *psf, sep, antype;
 
 	/* compute the y offset of all position types */
+	ymax = gchy;
 	yspcc = cfmt.font_tb[s->gcf].size;
 	yspca = cfmt.font_tb[s->anf].size;
-	ymax = gchy;
 	gchy -= yspcc;
+#ifdef YSTEP
+	gchyb = y_get(s, 0, s->x, 3, 0) - yspca;
+#else
 	gchyb = s->dc_bot - yspca;
+#endif
 	gchyl = gchyr = s->yav - yspca * 0.75;
 	p = s->as.text;
 	antype = '\0';
@@ -1516,16 +1671,23 @@ static void draw_gchord(struct SYMBOL *s,
 			break;
 		}
 		for (;;) {
-			p++;
 			if (*p == '\0' || *p == ';' || *p == '\n')
 				break;
+			p++;
+			if (*p == ';' && p[-1] == '\\')
+				p++;
 		}
 		sep = *p;
 		if (sep == '\0')
 			break;
 		p++;
 	}
+#ifdef YSTEP
+	ymin = gchy + yspcc;
+	y_set(s, 1, s->x, 3, ymin);
+#else
 	ymin = s->dc_top = gchy + yspcc;
+#endif
 	xmin = xmax = s->x;
 	box = cfmt.gchordbox;
 	psf = 0;
@@ -1559,6 +1721,8 @@ static void draw_gchord(struct SYMBOL *s,
 			else	antype = '\0';
 		}
 		for (q = p; ; q++) {
+			if (*q == ';' && q > p && q[-1] == '\\')
+				continue;
 			if (*q == '\0' || *q == ';' || *q == '\n') {
 				sep = *q;
 				*q = '\0';
@@ -1566,17 +1730,15 @@ static void draw_gchord(struct SYMBOL *s,
 			}
 		}
 		if (antype == '\0') {
-			if (*fonttype != 1) {
+			if (strcf != s->gcf) {
 				strcf = s->gcf;
 				set_font(&cfmt.font_tb[s->gcf]);
-				*fonttype = 1;
 			}
 			psf = "gc";
 		} else {
-			if (*fonttype != 2) {
+			if (strcf != s->anf) {
 				strcf = s->anf;
 				set_font(&cfmt.font_tb[s->anf]);
-				*fonttype = 2;
 			}
 			psf = "an";
 		}
@@ -1594,7 +1756,11 @@ static void draw_gchord(struct SYMBOL *s,
 			x = s->x - xspc;
 			if (antype == '_') {
 				y = gchyb;
+#ifdef YSTEP
+				y_set(s, 0, x, xspc, gchyb - 2);
+#else
 				s->dc_bot = gchyb - 2;
+#endif
 				gchyb -= yspca;
 			} else {
 				y = gchy;
@@ -1620,12 +1786,15 @@ static void draw_gchord(struct SYMBOL *s,
 		case '<':		/* left */
 /*fixme: what symbol space?*/
 			x = s->x - xspc - 6;
+			if (s->as.u.note.accs[0])
+				x -= s->shac[0];
 			y = gchyl;
 			gchyl -= yspca;
 			break;
 		case '>':		/* right */
-/*fixme: what symbol space?*/
-			x = s->x + 6;
+			x = s->x + s->xmx + 6;
+			if (s->dots > 0)
+				x += 1.5 + 3.5 * s->dots;
 			y = gchyr;
 			gchyr -= yspca;
 			break;
@@ -1636,8 +1805,9 @@ static void draw_gchord(struct SYMBOL *s,
 			y -= yspca;
 			break;
 		}
-		PUT5("%.1f %.1f y%d M(%s)%sshow ",
-		     x, y, s->staff, tex_buf, psf);
+		putxy(x, y);
+		PUT3("y%d M(%s)%sshow ",
+		     s->staff, tex_buf, psf);
 		if (sep == '\0')
 			break;
 		*q = sep;
@@ -1658,21 +1828,27 @@ void draw_measnb(void)
 {
 	struct SYMBOL *s;
 	char *showm;
-	int bar_time, any_nb;
-	float x, y, w, wmeasure;
+	int bar_time, any_nb, wmeasure;
+	float x, y, w;
 
 	showm = cfmt.measurebox ? "showb" : "show";
 	any_nb = 0;
 
 	/* get the current bar number */
-	for (s = first_voice->sym->next; s->next != 0; s = s->next) {
+/*fixme: what to do if no symbol in the 1st voice?*/
+	if ((s = first_voice->sym->next) == 0)
+		return;
+	for ( ; s->next != 0; s = s->next) {
 		switch (s->type) {
 		case TIMESIG:
 			wmeasure = s->as.u.meter.wmeasure;
+			continue;
 		case CLEF:
 		case KEYSIG:
 		case PART:
 		case TEMPO:
+		case FMTCHG:
+		case WHISTLE:
 			continue;
 		case BAR:
 			if (s->u != 0)
@@ -1699,12 +1875,13 @@ void draw_measnb(void)
 			any_nb = 1;
 			x = 0;
 			w = 20;
-			y = get_y(s, 1, x, w, 0);
+			y = y_get(s, 1, x, w, 0);
 			if (y < 24 + 14)
 				y = 24 + 14;
-			PUT3("0 %.1f y0 M(%d)%s",
-			     y, nbar, showm);
-			set_y(s, 1, x, w, y + cfmt.measurefont.size + 2);
+			PUT0("0 ");
+			puty(y);
+			PUT2("y0 M(%d)%s",nbar, showm);
+			y_set(s, 1, x, w, y + cfmt.measurefont.size + 2);
 		} else if (nbar % cfmt.measurenb == 0) {
 			x = s->x - 8;
 			set_font(&cfmt.measurefont);
@@ -1717,12 +1894,12 @@ void draw_measnb(void)
 			}
 			if (cfmt.measurebox)
 				w += 4;
-			y = get_y(s, 1, x, w, 0);
+			y = y_get(s, 1, x, w, 0);
 			if (y < 24 + 7)
 				y = 24 + 7;
-			PUT4("%.1f %.1f y0 M(%d)%s",
-			     x, y, nbar, showm);
-			set_y(s, 1, x, w, y + cfmt.measurefont.size + 2);
+			putxy(x, y);
+			PUT2("y0 M(%d)%s", nbar, showm);
+			y_set(s, 1, x, w, y + cfmt.measurefont.size + 2);
 		}
 	}
 
@@ -1783,7 +1960,7 @@ void draw_measnb(void)
 		if (cfmt.measurebox)
 			w += 4;
 		x = s->x - w * 0.4;
-		y = get_y(s, 1, x, w, 0);
+		y = y_get(s, 1, x, w, 0);
 		if (y < 24 + 7)
 			y = 24 + 7;
 		if (s->next->type == NOTE) {
@@ -1795,9 +1972,10 @@ void draw_measnb(void)
 					y = s->next->y;
 			}
 		}
-		PUT4(" %.1f %.1f y0 M(%d)%s",
-		     x, y, nbar, showm);
-		set_y(s, 1, x, w, y + cfmt.measurefont.size + 2);
+		PUT0(" ");
+		putxy(x, y);
+		PUT2("y0 M(%d)%s",nbar, showm);
+		y_set(s, 1, x, w, y + cfmt.measurefont.size + 2);
 	}
 	if (any_nb)
 		PUT0("\n");
@@ -1828,8 +2006,7 @@ static void draw_notempo(struct SYMBOL *s, int len, float sc)
 	int head, dots, flags;
 	float dx;
 
-	identify_note(s, len,
-		      &head, &dots, &flags);
+	identify_note(s, len, &head, &dots, &flags);
 	PUT1("gsave %.2f dup scale 8 3 RM currentpoint ", sc);
 	switch (head) {
 	case H_OVAL:
@@ -1859,7 +2036,7 @@ static void draw_notempo(struct SYMBOL *s, int len, float sc)
 			break;
 		}
 		while (--dots >= 0) {
-			PUT1(" x %.1f add 0 dt", dotx);
+			PUT1(" %.1f 0 dt", dotx);
 			dx = dotx;
 			dotx += 3.5;
 		}
@@ -1880,8 +2057,7 @@ static void draw_notempo(struct SYMBOL *s, int len, float sc)
 /* (the staves are being defined) */
 float draw_partempo(float top,
 		    int any_part,
-		    int any_tempo,
-		    int any_vocal)
+		    int any_tempo)
 {
 	struct SYMBOL *s;
 	float h, ht, w, y, ymin, nw, dy;
@@ -1894,40 +2070,36 @@ float draw_partempo(float top,
 		float sc;
 
 		ht = cfmt.tempofont.size + 2 + 2;
-		if (any_vocal)
-			dy = ht;
-		else {
-			str_font(&cfmt.tempofont);
-			nw = 6. + cwid(' ') * cfmt.tempofont.size * 6;
+		str_font(&cfmt.tempofont);
+		nw = 6. + cwid(' ') * cfmt.tempofont.size * 6;
 /*fixme:have tempo on other voices but the 1st?*/
 
-			/* get the minimal y offset */
-			ymin = 24 + 12;
-			for (s = first_voice->sym; s != 0; s = s->next) {
-				if (s->type != TEMPO)
-					continue;
+		/* get the minimal y offset */
+		ymin = 24 + 12;
+		for (s = first_voice->sym->next; s != 0; s = s->next) {
+			if (s->type != TEMPO)
+				continue;
 
-				if (s->as.u.tempo.str1 != 0)
-					nw += tex_str(s->as.u.tempo.str1);
-				if (s->as.u.tempo.value != 0) {
-					i = 1;
-					while (i < sizeof s->as.u.tempo.length
-							/ sizeof s->as.u.tempo.length[0]
-					       && s->as.u.tempo.length[i] > 0) {
-						nw += 10;
-						i++;
-					}
-					nw += 10 + 10;
+			if (s->as.u.tempo.str1 != 0)
+				nw += tex_str(s->as.u.tempo.str1);
+			if (s->as.u.tempo.value != 0) {
+				i = 1;
+				while (i < sizeof s->as.u.tempo.length
+						/ sizeof s->as.u.tempo.length[0]
+				       && s->as.u.tempo.length[i] > 0) {
+					nw += 10;
+					i++;
 				}
-				if (s->as.u.tempo.str2 != 0)
-					nw += tex_str(s->as.u.tempo.str2);
-				y = get_y(s, 1, s->x - 5, nw + 30 + 10, 0) + 2;
-				if (y > ymin)
-					ymin = y;
+				nw += 10 + 10;
 			}
-			if (top < ymin + ht)
-				dy = ymin + ht - top;
+			if (s->as.u.tempo.str2 != 0)
+				nw += tex_str(s->as.u.tempo.str2);
+			y = y_get(s, 1, s->x - 5, nw + 30 + 10, 0) + 2;
+			if (y > ymin)
+				ymin = y;
 		}
+		if (top < ymin + ht)
+			dy = ymin + ht - top;
 
 		/* draw the tempo indications */
 		str_font(&cfmt.tempofont);
@@ -1942,8 +2114,8 @@ float draw_partempo(float top,
 				continue;
 			}
 
-			/*fixme: cf left shift (-5.)*/
-			PUT2("%.1f %.1f M ", s->x - 5., 2. - ht);
+			/*fixme: cf left shift (-5)*/
+			PUT2("%.1f %.1f M ", s->x - 5, 2 - ht);
 			if (s->as.u.tempo.str1 != 0)
 				put_str(s->as.u.tempo.str1, A_LEFT);
 
@@ -1980,24 +2152,20 @@ float draw_partempo(float top,
 	h = cfmt.partsfont.size + 2 + 2;	/* + cfmt.partsspace; */
 	
 	str_font(&cfmt.partsfont);
-	if (any_vocal)
-		dy += h;
-	else {
-		ymin = 24 + 14;
-		for (s = first_voice->sym; s != 0; s = s->next) {
-			if (s->type != PART)
-				continue;
-			w = tex_str(&s->as.text[2]);
-			y = get_y(s, 1, s->x - 10, w + 15, 0) + 5;
-			if (ymin < y)
-				ymin = y;
-		}
-		if (top < ymin + h + ht)
-			dy = ymin + h + ht - top;
+	ymin = 24 + 14;
+	for (s = first_voice->sym->next; s != 0; s = s->next) {
+		if (s->type != PART)
+			continue;
+		w = tex_str(&s->as.text[2]);
+		y = y_get(s, 1, s->x - 10, w + 15, 0) + 5;
+		if (ymin < y)
+			ymin = y;
 	}
+	if (top < ymin + h + ht)
+		dy = ymin + h + ht - top;
 
 	set_font(&cfmt.partsfont);
-	for (s = first_voice->sym; s != 0; s = s->next) {
+	for (s = first_voice->sym->next; s != 0; s = s->next) {
 		if (s->type != PART)
 			continue;
 		tex_str(&s->as.text[2]);
@@ -2040,3 +2208,12 @@ void reset_deco(int deco_old)
 		deco_glob['M'] = 14;
 	}
 }
+
+/* -- set the decoration flags -- */
+void set_defl(int new_defl)
+{
+	if (defl == new_defl)
+		return;
+	defl = new_defl;
+	PUT1("/defl %d def ", new_defl);
+}
