@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2004 Jean-Franois Moine
+ * Copyright (C) 1998-2005 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -59,6 +59,7 @@ static struct format {
 	{"barsperstaff", FORMAT_I, 0, &cfmt.barsperstaff},
 	{"botmargin", FORMAT_U, 0, &cfmt.botmargin},
 	{"bstemdown", FORMAT_B, 0, &cfmt.bstemdown},
+	{"combinevoices", FORMAT_B, 0, &cfmt.combinevoices},
 	{"composerfont", FORMAT_F, 0, &cfmt.composerfont},
 	{"composerspace", FORMAT_U, 0, &cfmt.composerspace},
 	{"contbarnb", FORMAT_B, 0, &cfmt.contbarnb},
@@ -78,6 +79,7 @@ static struct format {
 	{"halfbeam", FORMAT_B, 0, &cfmt.halfbeam},
 	{"header", FORMAT_S, 0, &cfmt.header},
 	{"headerfont", FORMAT_F, 0, &cfmt.headerfont},
+	{"historyfont", FORMAT_F, 0, &cfmt.historyfont},
 	{"hyphencont", FORMAT_B, 0, &cfmt.hyphencont},
 	{"indent", FORMAT_U, 0, &cfmt.indent},
 	{"infofont", FORMAT_F, 0, &cfmt.infofont},
@@ -109,6 +111,7 @@ static struct format {
 	{"repeatfont", FORMAT_F, 0, &cfmt.repeatfont},
 	{"rightmargin", FORMAT_U, 0, &cfmt.rightmargin},
 	{"scale", FORMAT_R, 0, &cfmt.scale},
+	{"setdefl", FORMAT_B, 0, &cfmt.setdefl},
 	{"setfont-1", FORMAT_F, 0, &cfmt.font_tb[1]},
 	{"setfont-2", FORMAT_F, 0, &cfmt.font_tb[2]},
 	{"setfont-3", FORMAT_F, 0, &cfmt.font_tb[3]},
@@ -266,22 +269,23 @@ void make_font_list(void)
 	struct FORMAT *f;
 
 	f = &cfmt;
-	used_font[f->titlefont.fnum] = 1;
-	used_font[f->subtitlefont.fnum] = 1;
-	used_font[f->composerfont.fnum] = 1;
-	used_font[f->partsfont.fnum] = 1;
-	used_font[f->vocalfont.fnum] = 1;
-	used_font[f->textfont.fnum] = 1;
-	used_font[f->tempofont.fnum] = 1;
-	used_font[f->wordsfont.fnum] = 1;
-	used_font[f->gchordfont.fnum] = 1;
-	used_font[f->infofont.fnum] = 1;
-	used_font[f->footerfont.fnum] = 1;
-	used_font[f->headerfont.fnum] = 1;
-	used_font[f->repeatfont.fnum] = 1;
-	used_font[f->measurefont.fnum] = 1;
 	used_font[f->annotationfont.fnum] = 1;
+	used_font[f->composerfont.fnum] = 1;
+	used_font[f->footerfont.fnum] = 1;
+	used_font[f->gchordfont.fnum] = 1;
+	used_font[f->headerfont.fnum] = 1;
+	used_font[f->historyfont.fnum] = 1;
+	used_font[f->infofont.fnum] = 1;
+	used_font[f->measurefont.fnum] = 1;
+	used_font[f->partsfont.fnum] = 1;
+	used_font[f->repeatfont.fnum] = 1;
+	used_font[f->subtitlefont.fnum] = 1;
+	used_font[f->tempofont.fnum] = 1;
+	used_font[f->textfont.fnum] = 1;
+	used_font[f->titlefont.fnum] = 1;
+	used_font[f->vocalfont.fnum] = 1;
 	used_font[f->voicefont.fnum] = 1;
+	used_font[f->wordsfont.fnum] = 1;
 }
 
 /* -- set the default format -- */
@@ -296,20 +300,20 @@ void set_format(void)
 	f->pagewidth = PAGEWIDTH;
 	f->leftmargin = MARGIN;
 	f->rightmargin = MARGIN;
-	f->topmargin = 1.0 * CM;
-	f->botmargin = 1.0 * CM;
-	f->topspace = 0.8 * CM;
-	f->titlespace = 0.2 * CM;
-	f->subtitlespace = 0.1 * CM;
-	f->composerspace = 0.2 * CM;
-	f->musicspace = 0.2 * CM;
-	f->partsspace = 0.3 * CM;
-	f->staffsep = 46.0 * PT;
-	f->sysstaffsep = 34.0 * PT;
-	f->maxstaffsep = 800.0 * PT;
-	f->maxsysstaffsep = 800.0 * PT;
-	f->vocalspace = 23.0 * PT;
-	f->textspace = 0.5 * CM;
+	f->topmargin = 1.0 CM;
+	f->botmargin = 1.0 CM;
+	f->topspace = 0.8 CM;
+	f->titlespace = 0.2 CM;
+	f->subtitlespace = 0.1 CM;
+	f->composerspace = 0.2 CM;
+	f->musicspace = 0.2 CM;
+	f->partsspace = 0.3 CM;
+	f->staffsep = 46.0 PT;
+	f->sysstaffsep = 34.0 PT;
+	f->maxstaffsep = 2000.0 PT;
+	f->maxsysstaffsep = 2000.0 PT;
+	f->vocalspace = 23.0 PT;
+	f->textspace = 0.5 CM;
 	f->scale = 0.75;
 	f->slurheight = 1.0;
 	f->maxshrink = 0.65;
@@ -330,22 +334,23 @@ void set_format(void)
 	f->stemheight = STEM;
 	f->dateformat = strdup("\\%b \\%e, \\%Y \\%H:\\%M");
 	f->textoption = OBEYLINES;
-	fontspec(&f->titlefont, "Times-Roman", 0, 20.0);
-	fontspec(&f->subtitlefont, "Times-Roman", 0, 16.0);
-	fontspec(&f->composerfont, "Times-Italic", 0, 14.0);
-	fontspec(&f->partsfont, "Times-Roman", 0, 15.0);
-	fontspec(&f->tempofont, "Times-Bold", 0, 15.0);
-	fontspec(&f->vocalfont, "Times-Bold", 0, 13.0);
-	fontspec(&f->textfont,	"Times-Roman", 0, 16.0);
-	fontspec(&f->wordsfont,	"Times-Roman", 0, 16.0);
-	fontspec(&f->gchordfont, "Helvetica", 0, 12.0);
-	fontspec(&f->infofont,	"Times-Italic", 0, 14.0); /* same as composer by default */
-	fontspec(&f->footerfont, "Times-Roman", 0, 12.0); /* not scaled */
-	fontspec(&f->headerfont, "Times-Roman", 0, 12.0); /* not scaled */
-	fontspec(&f->repeatfont, "Times-Roman", 0, 13.0);
-	fontspec(&f->measurefont, "Times-Italic", 0, 14.0);
 	fontspec(&f->annotationfont, "Helvetica", 0, 12.0);
+	fontspec(&f->composerfont, "Times-Italic", 0, 14.0);
+	fontspec(&f->footerfont, "Times-Roman", 0, 12.0); /* not scaled */
+	fontspec(&f->gchordfont, "Helvetica", 0, 12.0);
+	fontspec(&f->headerfont, "Times-Roman", 0, 12.0); /* not scaled */
+	fontspec(&f->historyfont, "Times-Roman", 0, 16.0);
+	fontspec(&f->infofont,	"Times-Italic", 0, 14.0); /* same as composer by default */
+	fontspec(&f->measurefont, "Times-Italic", 0, 14.0);
+	fontspec(&f->partsfont, "Times-Roman", 0, 15.0);
+	fontspec(&f->repeatfont, "Times-Roman", 0, 13.0);
+	fontspec(&f->subtitlefont, "Times-Roman", 0, 16.0);
+	fontspec(&f->tempofont, "Times-Bold", 0, 15.0);
+	fontspec(&f->textfont,	"Times-Roman", 0, 16.0);
+	fontspec(&f->titlefont, "Times-Roman", 0, 20.0);
+	fontspec(&f->vocalfont, "Times-Bold", 0, 13.0);
 	fontspec(&f->voicefont, "Times-Bold", 0, 13.0);
+	fontspec(&f->wordsfont,	"Times-Roman", 0, 16.0);
 }
 
 /* -- print the current format -- */
@@ -394,12 +399,12 @@ static char yn[2][5]={"no","yes"};
 		}
 		case FORMAT_U:
 			if (fd->subtype == 0)
-				printf("%.2fcm\n", *((float *) fd->v) / CM);
+				printf("%.2fcm\n", *((float *) fd->v) / (1 CM));
 			else	printf("%.2fcm\n",
 					(cfmt.pagewidth
 						- cfmt.leftmargin
 						- cfmt.rightmargin)
-					/ CM);
+					/ (1 CM));
 			break;
 		case FORMAT_B:
 			printf("%s\n", yn[*((int *) fd->v)]);
@@ -418,7 +423,7 @@ static int get_encoding(char *p)
 	char **e;
 
 	l = 0;
-	while (p[l] != '\0' && !isspace(p[l]))
+	while (p[l] != '\0' && !isspace((unsigned char) p[l]))
 		l++;
 	if (strncasecmp(p, "ASCII", l) == 0)	/* backward compatibility */
 		return 0;
@@ -442,7 +447,7 @@ static int get_encoding(char *p)
 }
 
 /* -- get the option for text -- */
-int get_textopt(unsigned char *p)
+int get_textopt(char *p)
 {
 	int option;
 
@@ -504,13 +509,13 @@ static void g_fspc(char *p,
 	float fsize;
 
 	p = get_str(fname, p, sizeof fname);
-	if (!isdigit(*p)) {
+	if (!isdigit((unsigned char) *p)) {
 		if (*p == '*')
 			encoding = font_enc[fn->fnum];
 		else	encoding = get_encoding(p);
-		while (*p != '\0' && !isspace(*p))
+		while (*p != '\0' && !isspace((unsigned char) *p))
 			p++;
-		while (isspace(*p))
+		while (isspace((unsigned char) *p))
 			p++;
 	} else	encoding = cfmt.encoding;
 	fsize = *p != '\0' ? g_fltv(p) : fn->size;
@@ -548,7 +553,8 @@ int interpret_format_line(char *w,		/* keyword */
 		return 0;
 	}
 	if (strcmp(w, "format") == 0) {
-		read_fmt_file(p);
+		if (read_fmt_file(p) < 0)
+			error(1, 0, "No such format file '%s'", p);
 		return 1;
 	}
 	if (strcmp(w, "postscript") == 0) {
@@ -686,48 +692,68 @@ int interpret_format_line(char *w,		/* keyword */
 	return 0;
 }
 
+/* -- remove the spaces and comments of a format line -- */
+static char *clean_line(char *p)
+{
+	int i;
+	char c, *q;
+
+	while (isspace((unsigned char) *p))
+		p++;
+	q = p;
+	i = 0;
+	for (;;) {
+		if ((c = *p++) == '%') {
+			if (i > 0 && p[-2] != '\\')
+				c = '\0';
+		}
+		if (c == '\0') {
+			p--;
+			break;
+		}
+		i++;
+	}
+	while (--i > 0) {
+		c = *--p;
+		if (!isspace((unsigned char) c)) {
+			p[1] = '\0';
+			break;
+		}
+	}
+	return q;
+}
+
 /* -- read a format file -- */
 int read_fmt_file(char *filename)
 {
 	FILE *fp;
-	char fname[512];
+	char line[BSIZE], *p, *q;
 
-	strcpy(fname, filename);
-	strext(fname, "fmt");
-	if ((fp = fopen(fname, "r")) == 0) {
+	strcpy(line, filename);
+	strext(line, "fmt");
+	if ((fp = fopen(line, "r")) == 0) {
 		if (*styd == '\0')
 			return -1;
-                sprintf(fname, "%s%c%s", styd, DIRSEP, filename);
-		strext(fname, "fmt");
-		if ((fp = fopen(fname, "r")) == 0)
+                sprintf(line, "%s%c%s", styd, DIRSEP, filename);
+		strext(line, "fmt");
+		if ((fp = fopen(line, "r")) == 0)
 			return -1;
 	}
 
 	for (;;) {
-		char line[BSIZE], *p, *q;
-		int l;
-
 		p = line;
 		if (!fgets(p, sizeof line, fp))
 			break;
-		l = strlen(p) - 1;
-		if (p[l] == '\n')
-			p[l] = '\0';
-		while (isspace(*p))
-			p++;
-		if (*p == '\0' || *p == '%')
+		p = clean_line(p);
+		if (*p == '\0')
 			continue;
 		if (strcmp(p, "beginps") == 0) {
 			for (;;) {
 				p = line;
 				if (!fgets(p, sizeof line, fp))
 					break;
-				l = strlen(p) - 1;
-				if (p[l] == '\n')
-					p[l] = '\0';
-				while (isspace(*p))
-					p++;
-				if (*p == '\0' || *p == '%')
+				p = clean_line(p);
+				if (*p == '\0')
 					continue;
 				if (strcmp(p, "endps") == 0)
 					break;
@@ -738,11 +764,11 @@ int read_fmt_file(char *filename)
 		if (strcmp(p, "end") == 0)
 			break;
 		q = p;
-		while (*p != '\0' && !isspace(*p))
+		while (*p != '\0' && !isspace((unsigned char) *p))
 			p++;
 		if (*p != '\0') {
 			*p++ = '\0';
-			while (isspace(*p))
+			while (isspace((unsigned char) *p))
 				p++;
 		}
 		interpret_format_line(q, p);
