@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2004 Jean-Franois Moine
+ * Copyright (C) 1998-2005 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@
 #include "abcparse.h"
 #include "abc2ps.h" 
 
-struct STAFF staff_tb[MAXSTAFF];	/* staff table */
+struct STAFF_S staff_tb[MAXSTAFF];	/* staff table */
 int nstaff;				/* (0..MAXSTAFF-1) */
 
 struct VOICE_S voice_tb[MAXVOICE];	/* voice table */
@@ -56,22 +56,23 @@ static float multicol_max;
 static float lmarg, rmarg;
 
 /* sequence numbers for symbol grouping - index = symbol type */
-static unsigned char seq_tb[15] = {
+static unsigned char seq_tb[16] = {
 	0x60,		/* 0: notype */
 	0x78,		/* 1: note */
 	0x78,		/* 2: rest */
-	0x40,		/* 3: bar */
+	0x60,		/* 3: bar */
 	0x10,		/* 4: clef */
-	0x20,		/* 5: timesig */
-	0x18,		/* 6: keysig */
-	0x28,		/* 7: tempo */
+	0x30,		/* 5: timesig */
+	0x28,		/* 6: keysig */
+	0x38,		/* 7: tempo */
 	0x00,		/* 8: staves */
 	0x78,		/* 9: mrest */
-	0x30,		/* 10: part */
+	0x40,		/* 10: part */
 	0x78,		/* 11: mrep */
-	0x38,		/* 12: grace */
+	0x50,		/* 12: grace */
 	0x08,		/* 13: fmtchg */
-	0x60,		/* 14: tuplet */
+	0x70,		/* 14: tuplet */
+	0x00,		/* 15: whistle */
 };
 
 static void get_clef(struct SYMBOL *s);
@@ -81,26 +82,20 @@ static void get_voice(struct SYMBOL *s);
 static void get_note(struct SYMBOL *s);
 static struct abcsym *process_pscomment(struct abcsym *as);
 static void set_tuplet(struct SYMBOL *s);
-static void sym_link(struct SYMBOL *s);
+static void sym_link(struct SYMBOL *s, int type);
 
 /* -- add a new symbol at end of list -- */
-struct SYMBOL *add_sym(struct VOICE_S *p_voice,
-		       int type)
+struct SYMBOL *add_sym(struct VOICE_S *p_voice, int type)
 {
 	struct SYMBOL *s;
+	struct VOICE_S *p_voice2;
 
 	s = (struct SYMBOL *) getarena(sizeof *s);
 	memset(s, 0, sizeof *s);
-	if (p_voice->sym != 0) {
-		p_voice->last_symbol->next = s;
-		s->prev = p_voice->last_symbol;
-	} else	p_voice->sym = s;
-	p_voice->last_symbol = s;
-
-	s->type = type;
-	s->seq = seq_tb[type];
-	s->voice = p_voice - voice_tb;
-	s->staff = p_voice->cstaff;
+	p_voice2 = curvoice;
+	curvoice = p_voice;
+	sym_link(s, type);
+	curvoice = p_voice2;
 	return s;
 }
 
@@ -216,10 +211,10 @@ static void gchord_adjust(struct SYMBOL *s)
 }
 
 /* -- parse a lyric (vocal) definition -- */
-static char *get_lyric(unsigned char *p)
+static char *get_lyric(char *p)
 {
 	struct SYMBOL *s;
-	unsigned char word[81], *q;
+	char word[81], *q;
 	int ln;
 	struct FONTSPEC *f;
 
@@ -238,7 +233,7 @@ static char *get_lyric(unsigned char *p)
 
 	/* scan the lyric line */
 	while (*p != '\0') {
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 		if (*p == '\0')
 			break;
@@ -325,7 +320,7 @@ static char *get_lyric(unsigned char *p)
 		while (s != 0 && s->type != NOTE)
 			s = s->next;
 		if (s == 0)
-			return "Not enough notes for lyric line";
+			return "Too many words in lyric line";
 		if (word[0] != '*') {
 			struct lyl *lyl;
 			float w;
@@ -356,20 +351,22 @@ static void get_over(struct SYMBOL *s)
 {
 	struct VOICE_S *p_voice, *p_voice2;
 	struct SYMBOL *s1, *s2;
-	int ctime, linenum;
+	int ctime, linenum, colnum;
 
 	/* treat the end of overlay */
 	p_voice = curvoice;
 	linenum = s != 0 ? s->as.linenum : 0;
+	colnum = s != 0 ? s->as.colnum : 0;
+#if 0
 	if (over_bar) {
 		struct SYMBOL *s2;
 
-		s2 = add_sym(curvoice, BAR);
+		s2 = add_sym(p_voice, BAR);
 		s2->as.type = ABC_T_BAR;
 		s2->as.linenum = linenum;
-		s2->time = curvoice->time;
-		s2->seq = seq_tb[BAR];
+		s2->as.colnum = colnum;
 	}
+#endif
 	if (s == 0
 	    || s->as.u.v_over.type == V_OVER_E)  {
 		over_bar = 0;
@@ -379,7 +376,9 @@ static void get_over(struct SYMBOL *s)
 			return;
 		}
 		voice_over = 0;
-		for (p_voice = p_voice->prev; ; p_voice = p_voice->prev)
+		if (s != 0)
+			p_voice = &voice_tb[s->as.u.v_over.voice];
+		else for (p_voice = p_voice->prev; ; p_voice = p_voice->prev)
 			if (p_voice->name[0] != '&')
 				break;
 		curvoice = p_voice;
@@ -438,7 +437,7 @@ static void get_over(struct SYMBOL *s)
 		if (s1->time <= ctime)
 			break;
 
-	/* fill the secundary voice with invisible silences */
+	/* fill the secundary voice with invisible rests */
 	if (p_voice2->last_symbol == 0
 	    || p_voice2->last_symbol->type != BAR) {
 		for (s2 = s1; s2 != 0 && s2->time == ctime; s2 = s2->prev) {
@@ -450,9 +449,9 @@ static void get_over(struct SYMBOL *s)
 		if (s1->type == BAR) {
 			s2 = add_sym(p_voice2, BAR);
 			s2->as.linenum = linenum;
+			s2->as.colnum = colnum;
 			s2->as.u.bar.type = s1->as.u.bar.type;
 			s2->time = ctime;
-			s2->seq = seq_tb[BAR];
 		}
 	}
 	while (ctime < s->time) {
@@ -461,22 +460,22 @@ static void get_over(struct SYMBOL *s)
 			if (s1->type == BAR)
 				break;
 		}
-		if (s1->time - ctime != 0) {
+		if (s1->time != ctime) {
 			s2 = add_sym(p_voice2, REST);
 			s2->as.type = ABC_T_REST;
 			s2->as.linenum = linenum;
+			s2->as.colnum = colnum;
 			s2->as.u.note.invis = 1;
 			s2->len = s2->as.u.note.lens[0] = s1->time - ctime;
 			s2->time = ctime;
-			s2->seq = seq_tb[NOTE];
 			ctime = s1->time;
 		}
 		while (s1->type == BAR) {
 			s2 = add_sym(p_voice2, BAR);
 			s2->as.linenum = linenum;
+			s2->as.colnum = colnum;
 			s2->as.u.bar.type = s1->as.u.bar.type;
 			s2->time = ctime;
-			s2->seq = seq_tb[BAR];
 			if ((s1 = s1->next) == 0)
 				break;
 		}
@@ -488,15 +487,12 @@ static void get_over(struct SYMBOL *s)
 /* -- get staves definition (%%staves) -- */
 static void get_staves(struct SYMBOL *s)
 {
-	int i, staff, flags;
+	int i, staff, flags, dup_voice, v;
 	struct staff_s *p_staff;
 	struct VOICE_S *p_voice, *p_voice2;
-	int dup_voice;
 
 	/* clear, then link the voices */
-	for (i = 0, p_voice = voice_tb;
-	     i < MAXVOICE;
-	     i++, p_voice++) {
+	for (i = 0, p_voice = voice_tb; i < MAXVOICE; i++, p_voice++) {
 		p_voice->clone = -1;
 		p_voice->next = 0;
 		p_voice->prev = 0;
@@ -511,10 +507,7 @@ static void get_staves(struct SYMBOL *s)
 	for (i = 0, p_staff = s->as.u.staves;
 	     i < MAXVOICE && p_staff->name;
 	     i++, p_staff++) {
-		int voice;
-
-		voice = p_staff->voice;
-		p_voice = &voice_tb[voice];
+		p_voice = &voice_tb[p_staff->voice];
 
 		/* if voice already inserted, duplicate it */
 		if (p_voice == p_voice2 || p_voice->next || p_voice->prev) {
@@ -523,7 +516,7 @@ static void get_staves(struct SYMBOL *s)
 			dup_voice--;
 			p_voice3 = &voice_tb[dup_voice];
 			memcpy(p_voice3, p_voice, sizeof *p_voice3);
-			p_voice3->clone = voice;
+			p_voice3->clone = p_staff->voice;
 			p_voice3->next = 0;
 			p_voice3->second = 0;
 			p_voice3->floating = 0;
@@ -542,14 +535,10 @@ static void get_staves(struct SYMBOL *s)
 
 	/* define the staves */
 	memset(staff_tb, 0, sizeof staff_tb);
-	for (i = MAXSTAFF; --i >= 0; )
-		staff_tb[i].clef.line = 2;	/* treble clef on 2nd line */
 	staff = -1;
 	for (i = 0, p_staff = s->as.u.staves;
 	     i < MAXVOICE && p_staff->name;
 	     i++, p_staff++) {
-		int v;
-
 		flags = p_staff->flags;
 #if MAXSTAFF < MAXVOICE
 		if (staff >= MAXSTAFF - 1) {
@@ -560,13 +549,8 @@ static void get_staves(struct SYMBOL *s)
 		staff++;
 
 		p_voice = &voice_tb[p_staff->voice];
-
 		p_voice->staff = p_voice->cstaff = staff;
-		if (p_voice->forced_clef) {
-/*fixme*/
-			memcpy(&staff_tb[staff].clef, &p_voice->clef,
-			       sizeof staff_tb[0].clef);
-		}
+
 		if ((flags & (OPEN_PARENTH | CLOSE_PARENTH))
 				== (OPEN_PARENTH | CLOSE_PARENTH))
 			flags &= ~(OPEN_PARENTH | CLOSE_PARENTH);
@@ -661,7 +645,7 @@ err:
 	nstaff = staff;
 }
 
-/* -- initialize the general tune characteristics of all potential voices -- */
+/* -- re-initialize all potential voices -- */
 static void voice_init(void)
 {
 	struct VOICE_S *p_voice;
@@ -670,7 +654,7 @@ static void voice_init(void)
 	for (i = 0, p_voice = voice_tb;
 	     i < MAXVOICE;
 	     i++, p_voice++) {
-		p_voice->sym = 0;
+		p_voice->sym = p_voice->last_symbol = 0;
 		p_voice->clone = -1;
 		p_voice->bar_start = 0;
 		p_voice->time = 0;
@@ -706,7 +690,8 @@ static char *state_txt[4] = {
 };
 static void get_info(struct SYMBOL *s,
 		     int info_type,
-		     unsigned char *p)
+		     char *p,
+		     struct abctune *t)
 {
 	struct ISTRUCT *inf;
 
@@ -714,59 +699,61 @@ static void get_info(struct SYMBOL *s,
 	inf = s->as.state == ABC_S_GLOBAL ? &default_info : &info;
 	lvlarena(s->as.state != ABC_S_GLOBAL);
 
-	while (isspace(*p))
+	while (isspace((unsigned) *p))
 	       p++;
 
 	switch (info_type) {
 	case 'A':
 		add_info(s, &inf->area);
-		return;
+		break;
 	case 'B':
 		add_info(s, &inf->book);
-		return;
+		break;
 	case 'C':
 		add_info(s, &inf->comp);
-		return;
+		break;
 	case 'D':
 		add_info(s, &inf->disco);
-		return;
+		break;
 	case 'd':
 	case 'E':
 	case 'F':
 	case 'G':
-		return;
+		break;
 	case 'H':
 		add_info(s, &inf->histo);
-		return;
+		break;
 	case 'I':
 		process_pscomment(&s->as);	/* same as pseudo-comment */
-		return;
+		break;
 	case 'K':
 		get_key(s);
 		if (s->as.state != ABC_S_HEAD)
-			return;
+			break;
 		tunenum++;
 		PUT2("%% --- %s (%s) ---\n",
 		     info.xref, &info.title->as.text[2]);
 		if (!epsf)
 			bskip(cfmt.topspace);
-		write_heading();
+		write_heading(t);
 		reset_gen();
 		nbar = nbar_rep = cfmt.measurefirst;	/* measure numbering */
 		curvoice = first_voice;		/* switch to the 1st voice */
-		return;
+		break;
 	case 'L':
-		return;
+		break;
 	case 'M':
 		get_meter(s);
-		return;
+		break;
 	case 'N':
 		add_info(s, &inf->notes);
-		return;
+		break;
 	case 'O':
 		add_info(s, &inf->orig);
-		return;
+		break;
 	case 'P':
+		if (!cfmt.printparts)
+			break;
 		switch (s->as.state) {
 		case ABC_S_GLOBAL:
 		case ABC_S_HEAD:
@@ -777,46 +764,43 @@ static void get_info(struct SYMBOL *s,
 
 			p_voice = curvoice;
 			curvoice = first_voice;
-			sym_link(s);
-			s->type = PART;
+			sym_link(s, PART);
 			if (p_voice->time != curvoice->time) {
 				error(1, s, "Misplaced P:");
 				if (p_voice->time > curvoice->time)
-					curvoice->time = p_voice->time;
+					s->time = curvoice->time = p_voice->time;
 				else	p_voice->time = curvoice->time;
 			}
 			curvoice = p_voice;
 			break;
 		    }
 		default:
-			sym_link(s);
-			s->type = PART;
+			sym_link(s, PART);
 			break;
 		}
-		return;
+		break;
 	case 'Q':
 		if (curvoice != first_voice	/* tempo only for first voice */
 		    || !cfmt.printtempo)
-			return;
+			break;
 		switch (s->as.state) {
 		case ABC_S_GLOBAL:
 		case ABC_S_HEAD:
 			inf->tempo = s;
 			break;
 		default:
-			sym_link(s);
-			s->type = TEMPO;
+			sym_link(s, TEMPO);
 			break;
 		}
-		return;
+		break;
 	case 'R':
 		add_info(s, &inf->rhythm);
-		return;
+		break;
 	case 'S':
 		add_info(s, &inf->src);
-		return;
+		break;
 	case 's':
-		return;
+		break;
 	case 'T':
 		switch (s->as.state) {
 		case ABC_S_GLOBAL:	/* T: without X: */
@@ -828,40 +812,40 @@ static void get_info(struct SYMBOL *s,
 			output_music();
 			buffer_eob();
 			write_title(s);
-			bskip(cfmt.musicspace + 0.2 * CM);
+			bskip(cfmt.musicspace + 0.2 CM);
 			voice_init();
 			reset_gen();		/* (display the time signature) */
 			curvoice = first_voice;
 			break;
 		}
-		return;
+		break;
 	case 'U': {
 		unsigned char *deco;
 
 		deco = s->as.state == ABC_S_GLOBAL ? deco_glob : deco_tune;
 		deco[s->as.u.user.symbol] = deco_intern(s->as.u.user.value);
-		return;
+		break;
 	    }
 	case 'u':
-		return;
+		break;
 	case 'V':
 		get_voice(s);
-		return;
+		break;
 	case 'w':
-		if (cfmt.musiconly)
-			return;
-		if (s->as.state != ABC_S_TUNE)
-			return;
+		if (cfmt.musiconly
+		    || s->as.state != ABC_S_TUNE)
+			break;
 		if (lyric_start == 0)
 			break;
 		if ((p = get_lyric(p)) != 0)
 			error(1, s, "%s", p);
-		return;
+		curvoice->have_ly = 1;
+		break;
 	case 'W':
 		if (s->as.state == ABC_S_GLOBAL)
-			return;
+			break;
 		add_info(s, &words);
-		return;
+		break;
 	case 'X':
 newtune:
 		if (!epsf)
@@ -873,13 +857,15 @@ newtune:
 			info.title = s;
 		memcpy(&deco_tune, &deco_glob, sizeof deco_tune);
 		voice_init();		/* initialize all the voices */
-		return;
+		break;
 	case 'Z':
 		add_info(s, &inf->trans);
-		return;
+		break;
+	default:
+		error(1, s, "%s info '%c:' not treated",
+			state_txt[(int) s->as.state], info_type);
+		break;
 	}
-	error(1, s, "%s info '%c:' not treated",
-		state_txt[(int) s->as.state], info_type);
 }
 
 /* -- set head type, dots, flags for note -- */
@@ -984,8 +970,7 @@ static void get_bar(struct SYMBOL *s)
 		}
 	}
 
-	sym_link(s);
-	s->type = BAR;
+	sym_link(s, BAR);
 
 	if ((bar_type & 0xf0) != 0) {
 		do {
@@ -1028,11 +1013,11 @@ void do_tune(struct abctune *t,
 	     int header_only)
 {
 	struct abcsym *as;
+	struct SYMBOL *s, *s2;
 	int i;
 
 	/* initialize */
 	memset(voice_tb, 0, sizeof voice_tb);
-	voice_init();		/* initialize all the voices */
 	voice_tb[0].name = "1";	/* implicit voice */
 	voice_over = 0;
 	words = 0;
@@ -1042,23 +1027,28 @@ void do_tune(struct abctune *t,
 
 	for (i = MAXVOICE; --i >= 0; ) {
 		voice_tb[i].clef.line = 2;	/* treble clef on 2nd line */
+		voice_tb[i].clef.stafflines = 5;
+		voice_tb[i].clef.staffscale = 1;
 		voice_tb[i].meter.nmeter = 1;
 		voice_tb[i].meter.wmeasure = BASE_LEN;
 		voice_tb[i].meter.meter[0].top[0] = '4';
 		voice_tb[i].meter.meter[0].bot[0] = '4';
 		voice_tb[i].wmeasure = BASE_LEN;
+		voice_tb[i].scale = 1;
 	}
-	for (i = MAXSTAFF; --i >= 0; )
-		staff_tb[i].clef.line = 2;
+	voice_init();		/* initialize all the voices */
+	for (i = 0; i < nwhistle; i++)
+		voice_tb[whistle_tb[i].voice].whistle = 1;
 
 	curvoice = first_voice = voice_tb;
-	use_buffer = !cfmt.splittune;
+	if (cfmt.oneperpage) {
+		use_buffer = 0;
+		close_page();
+	} else	use_buffer = !cfmt.splittune;
 
 	/* set the note duration - this is needed for tuplets */
 	if (!header_only) {
 		for (as = t->first_sym; as != 0; as = as->next) {
-			struct SYMBOL *s;
-
 			if (as->type != ABC_T_NOTE
 			    && as->type != ABC_T_REST)
 				continue;
@@ -1070,16 +1060,15 @@ void do_tune(struct abctune *t,
 	/* scan the tune */
 	grace_head = 0;
 	for (as = t->first_sym; as != 0; as = as->next) {
-		struct SYMBOL *s = (struct SYMBOL *) as;
-
 		if (header_only && as->state != ABC_S_GLOBAL)
 			break;
 		if (grace_head != 0 && as->type != ABC_T_NOTE)
 			grace_head = 0;
+		s = (struct SYMBOL *) as;
 		switch (as->type) {
 		case ABC_T_INFO: {
 			int info_type;
-			unsigned char *p;
+			char *p;
 
 			if (header_only
 			    && (as->text[0] == 'X'
@@ -1088,7 +1077,7 @@ void do_tune(struct abctune *t,
 			info_type = as->text[0];
 			p = &as->text[2];
 			for (;;) {
-				get_info(s, info_type, p);
+				get_info(s, info_type, p, t);
 				if (as->next == 0
 				    || as->next->type != ABC_T_INFO2)
 					break;
@@ -1106,9 +1095,15 @@ void do_tune(struct abctune *t,
 			get_note(s);
 			break;
 		case ABC_T_BAR:
-			if (over_bar)
-				get_over(0);
 			get_bar(s);
+			if (over_bar) {
+				get_over(0);
+				s2 = add_sym(curvoice, BAR);
+				s2->as.linenum = s->as.linenum;
+				s2->as.colnum = s->as.colnum;
+				s2->as.u.bar.type = s->as.u.bar.type;
+				s2->time = s->time;
+			}
 			break;
 		case ABC_T_CLEF:
 			get_clef(s);
@@ -1124,28 +1119,27 @@ void do_tune(struct abctune *t,
 			len = curvoice->wmeasure * as->u.bar.len;
 			if (as->type == ABC_T_MREP
 			    && as->u.bar.len > 1) {
-				struct SYMBOL *s2;
 
-			/* repeat measure more than 1 time */
-			/* 2 times -> (bar - invisible rest - bar - mrep - bar) */
+				/* repeat measure more than 1 time */
+				/* 2 times -> (bar - invisible rest - bar - mrep - bar) */
 /*fixme: 3 or more times not treated*/
 				s2 = add_sym(curvoice, REST);
 				s2->as.type = ABC_T_REST;
 				s2->as.linenum = as->linenum;
+				s2->as.colnum = as->colnum;
 				s2->as.u.note.invis = 1;
 				len /= as->u.bar.len;
 				s2->len = len;
-				s2->time = curvoice->time;
 				curvoice->time += len;
 				s2 = add_sym(curvoice, BAR);
 				s2->as.linenum = as->linenum;
+				s2->as.colnum = as->colnum;
 				s2->as.u.bar.type = B_SINGLE;
-				s2->time = curvoice->time;
 			}
-			sym_link(s);
-			s->type = as->type == ABC_T_MREST ? MREST : MREP;
+			sym_link(s, as->type == ABC_T_MREST ? MREST : MREP);
 			s->len = len;
-			s->gcf = cfmt.gcf;
+			if (s->as.text != 0)		/* adjust the */
+				gchord_adjust(s);	/* guitar chords */
 			break;
 		    }
 		case ABC_T_V_OVER:
@@ -1157,21 +1151,30 @@ void do_tune(struct abctune *t,
 		default:
 			continue;
 		}
-		s->seq = seq_tb[s->type];
-		s->time = curvoice->time;
+		if (s->type == 0)
+			continue;
 		if (grace_head == 0)
 			curvoice->time += s->len;
+		if (s->prev != 0 && s->prev->time == s->time
+		    && s->prev->seq == s->seq) {
+			if (s->seq != 0)
+				s->seq = 0;
+			else	s->seq++;
+		}
 	}
 
 	output_music();
 	buffer_eob();
-	if (words != 0)
-		put_words(words);
-	if (cfmt.writehistory)
-		put_history();
-	if (epsf && nbuf > 0)
-		write_eps();
-	else	write_buffer();
+	if (!header_only) {
+		if (words != 0)
+			put_words(words);
+		if (cfmt.writehistory)
+			put_history();
+	}
+	if (epsf) {
+		if (nbuf > 0)
+			write_eps();
+	} else	write_buffer();
 
 	if (info.xref != 0)
 		cfmt = dfmt;	/* restore the format at start of tune */
@@ -1182,24 +1185,39 @@ static void get_clef(struct SYMBOL *s)
 {
 	struct VOICE_S *p_voice;
 	struct SYMBOL *s2;
+	int i, stafflines;
+	float staffscale;
 
 	p_voice = curvoice;
 	if (s->as.prev->type == ABC_T_INFO) {
 		switch (s->as.prev->text[0]) {
 		case 'K':
 			if (s->as.prev->state == ABC_S_HEAD) {
-				int i;
-
-				for (i = MAXVOICE, p_voice = voice_tb;
-				     --i >= 0;
-				     p_voice++) {
-					memcpy(&p_voice->clef, &s->as.u.clef,
-					       sizeof p_voice->clef);
-					p_voice->forced_clef = 1;
+				if (s->as.u.clef.type >= 0) {
+					for (i = MAXVOICE, p_voice = voice_tb;
+					     --i >= 0;
+					     p_voice++) {
+						stafflines = p_voice->clef.stafflines;
+						staffscale = p_voice->clef.staffscale;
+						memcpy(&p_voice->clef, &s->as.u.clef,
+						       sizeof p_voice->clef);
+						p_voice->clef.stafflines = stafflines;
+						p_voice->clef.staffscale = staffscale;
+						p_voice->forced_clef = 1;
+					}
+				}
+				if ((stafflines = s->as.u.clef.stafflines) >= 0) {
+					for (i = MAXVOICE, p_voice = voice_tb;
+					     --i >= 0;
+					     p_voice++)
+						p_voice->clef.stafflines = stafflines;
+				}
+				if ((staffscale = s->as.u.clef.staffscale) != 0) {
+					for (i = MAXVOICE, p_voice = voice_tb;
+					     --i >= 0;
+					     p_voice++)
+						p_voice->clef.staffscale = staffscale;
 				}
-				for (i = MAXSTAFF; --i >= 0; )
-					memcpy(&staff_tb[i].clef, &s->as.u.clef,
-					       sizeof s->as.u.clef);
 				return;
 			}
 			break;
@@ -1210,97 +1228,120 @@ static void get_clef(struct SYMBOL *s)
 	}
 
 	if (p_voice->sym == 0) {
-		memcpy(&staff_tb[p_voice->staff].clef, &s->as.u.clef, /* initial clef */
-		       sizeof s->as.u.clef);
+		i = p_voice->staff;
+		if ((stafflines = s->as.u.clef.stafflines) < 0)
+			stafflines = p_voice->clef.stafflines;
+		if ((staffscale = s->as.u.clef.staffscale) == 0)
+			staffscale = p_voice->clef.staffscale;
+		if (s->as.u.clef.type >= 0) {
+			memcpy(&p_voice->clef,
+			       &s->as.u.clef, 		/* initial clef */
+			       sizeof p_voice->clef);
+		}
+		p_voice->clef.stafflines = stafflines;
+		p_voice->clef.staffscale = staffscale;
 	} else {
-#if 0		
-		if (p_voice->clef.type != s->as.u.clef.type
-		    || p_voice->clef.line != s->as.u.clef.line
-		    || p_voice->clef.octave != s->as.u.clef.octave) {
-#endif
-			sym_link(s);
-			s->type = CLEF;
-			s->u = 1;	/* small clef */
+		sym_link(s, CLEF);
+		s->u = 1;	/* small clef */
 
-			/* the clef change must be before a key signature */
-			s2 = s->prev;
-			if (s2->type == KEYSIG) {
-				s2->next = 0;
-				p_voice->last_symbol = s2;
-				s->prev = s2->prev;
-				if (s->prev != 0)
-					s->prev->next = s;
-				s->next = s2;
-				s2->prev = s;
-			}
+		/* the clef change must be before a key signature */
+		s2 = s->prev;
+		if (s2->type == KEYSIG) {
+			s2->next = 0;
+			p_voice->last_symbol = s2;
+			if ((s->prev = s2->prev) != 0)
+				s->prev->next = s;
+			s->next = s2;
+			s2->prev = s;
+		}
 
-			/* the clef change must be before a bar */
-			s2 = s->prev;
-			if (s2 != 0
-			    && s2->type == BAR) {
-				s2->next = s->next;
-				if (s->next != 0)
-					s->next->prev = s2;
-				else	p_voice->last_symbol = s2;
-				s->prev = s2->prev;
-				if (s->prev != 0)
-					s->prev->next = s;
-				s->next = s2;
-				s2->prev = s;
-			}
-/*		} */
+		/* the clef change must be before a bar */
+		s2 = s->prev;
+		if (s2 != 0 && s2->type == BAR) {
+			if ((s2->next = s->next) != 0)
+				s->next->prev = s2;
+			else	p_voice->last_symbol = s2;
+			if ((s->prev = s2->prev) != 0)
+				s->prev->next = s;
+			s->next = s2;
+			s2->prev = s;
+		}
 	}
-	memcpy(&p_voice->clef, &s->as.u.clef,	/* current clef */
-	       sizeof p_voice->clef);
-	p_voice->forced_clef = 1;		/* don't change */
+	if (s->as.u.clef.type >= 0)
+		p_voice->forced_clef = 1;		/* don't change */
 }
 
 /* -- get a key signature definition (K:) -- */
 static void get_key(struct SYMBOL *s)
 {
 	struct VOICE_S *p_voice;
+	struct SYMBOL *s2;
 	int i;
 
 	if (s->as.u.key.empty == 1)
 		return;			/* clef only */
 	switch (s->as.state) {
-	case ABC_S_HEAD: {
+	case ABC_S_HEAD:
 		for (i = MAXVOICE, p_voice = voice_tb;
 		     --i >= 0;
 		     p_voice++) {
 			memcpy(&p_voice->key, &s->as.u.key, sizeof p_voice->key);
 			p_voice->sfp = s->as.u.key.sf;
+			if (p_voice->key.bagpipe
+			    && p_voice->stem == 0)
+				p_voice->stem = -1;
 		}
 		break;
-	    }
 	case ABC_S_TUNE:
 	case ABC_S_EMBED:
 		if (curvoice->sym == 0) {
-#if 0
-			/* if first symbol of the first voice, change all voices */
-			if (curvoice == first_voice) {
+
+			/* if first symbol of the first voice, apply to all voices */
+			if (curvoice == first_voice
+			    && s->as.state == ABC_S_TUNE) {	/* (not embedded) */
 				for (i = MAXVOICE, p_voice = voice_tb;
 				     --i >= 0;
 				     p_voice++) {
 					memcpy(&p_voice->key, &s->as.u.key,
 					       sizeof p_voice->key);
 					p_voice->sfp = s->as.u.key.sf;
+					if (p_voice->key.bagpipe
+					    && p_voice->stem == 0)
+						p_voice->stem = -1;
 				}
 			} else {
-#endif
 				memcpy(&curvoice->key, &s->as.u.key,
 				       sizeof curvoice->key);
 				curvoice->sfp = s->as.u.key.sf;
-/*			} */
+				if (curvoice->key.bagpipe
+				    && curvoice->stem == 0)
+					curvoice->stem = -1;
+			}
 			break;
 		}
 		if (curvoice->sfp == s->as.u.key.sf	/* if same key */
 		    && s->as.next->type != ABC_T_CLEF)	/* but not explicit clef */
 			break;				/* ignore */
-		sym_link(s);
-		s->type = KEYSIG;
+		sym_link(s, KEYSIG);
 		s->u = curvoice->sfp;		/* old key signature */
 		curvoice->sfp = s->as.u.key.sf;
+
+		/* the key signature must be just after a bar */
+		if ((s2 = s->prev) != 0 && s2->prev != 0) {
+			switch (s2->type) {
+			case TIMESIG:
+			case TEMPO:
+			case PART:
+/*			case FMTCHG: */
+				curvoice->last_symbol = s2;
+				s2->next = 0;
+				s2->prev->next = s;
+				s->prev = s2->prev;
+				s->next = s2;
+				s2->prev = s;
+				break;
+			}
+		}
 		break;
 	}
 }
@@ -1308,14 +1349,14 @@ static void get_key(struct SYMBOL *s)
 /* -- set meter from M: -- */
 static void get_meter(struct SYMBOL *s)
 {
+	struct VOICE_S *p_voice;
+	int i;
+
 	switch (s->as.state) {
 	case ABC_S_GLOBAL:
 		/*fixme: keep the values and apply to all tunes?? */
 		break;
 	case ABC_S_HEAD: {
-		struct VOICE_S *p_voice;
-		int i;
-
 		for (i = MAXVOICE, p_voice = voice_tb;
 		     --i >= 0;
 		     p_voice++) {
@@ -1327,18 +1368,27 @@ static void get_meter(struct SYMBOL *s)
 	    }
 	case ABC_S_TUNE:
 	case ABC_S_EMBED:
+		curvoice->wmeasure = s->as.u.meter.wmeasure;
 		if (curvoice->sym == 0) {
-			memcpy(&curvoice->meter, &s->as.u.meter,
-			       sizeof curvoice->meter);
-			curvoice->wmeasure = s->as.u.meter.wmeasure;
+
+			/* if first symbol of the first voice, apply to all voices */
+			if (curvoice == first_voice
+			    && s->as.state == ABC_S_TUNE) {	/* (not embedded) */
+				for (i = MAXVOICE, p_voice = voice_tb;
+				     --i >= 0;
+				     p_voice++) {
+					memcpy(&p_voice->meter, &s->as.u.meter,
+					       sizeof p_voice->meter);
+					p_voice->wmeasure = s->as.u.meter.wmeasure;
+				}
+			} else	memcpy(&curvoice->meter, &s->as.u.meter,
+				       sizeof curvoice->meter);
 			reset_gen();	/* (display the time signature) */
 			break;
 		}
-		curvoice->wmeasure = s->as.u.meter.wmeasure;
 		if (s->as.u.meter.nmeter == 0)
 			break;		/* M:none */
-		sym_link(s);
-		s->type = TIMESIG;
+		sym_link(s, TIMESIG);
 		break;
 	}
 }
@@ -1398,18 +1448,24 @@ static void get_voice(struct SYMBOL *s)
 	if (s->as.u.voice.dyn != 0)
 		p_voice->dyn = s->as.u.voice.dyn != 2
 			? s->as.u.voice.dyn : 0;
+	if (s->as.u.voice.lyrics != 0)
+		p_voice->ly_pos = s->as.u.voice.lyrics != 2
+			? s->as.u.voice.lyrics : 0;
+	if (s->as.u.voice.scale != 0)
+		p_voice->scale = s->as.u.voice.scale;
 }
 
 /* -- note or rest -- */
 static void get_note(struct SYMBOL *s)
 {
-	int i, m;
+	int i, m, type;
 
 	s->nhd = m = s->as.u.note.nhd;
+	type = s->as.type == ABC_T_NOTE ? NOTE : REST;
 	if (!s->as.u.note.grace) {	/* normal note/rest */
 		if (grace_head != 0)
 			grace_head = 0;
-		sym_link(s);
+		sym_link(s, type);
 		s->multi = curvoice->stem;
 	} else {			/* in a grace note sequence */
 		int div;
@@ -1418,9 +1474,8 @@ static void get_note(struct SYMBOL *s)
 			struct SYMBOL *s2;
 
 			s2 = add_sym(curvoice, GRACE);
-			s2->type = GRACE;
 			s2->as.linenum = s->as.linenum;
-			s2->time = curvoice->time;
+			s2->as.colnum = s->as.colnum;
 			grace_head = s2;
 			grace_head->grace = grace_tail = s;
 			s2->multi = curvoice->gstem;
@@ -1445,20 +1500,24 @@ static void get_note(struct SYMBOL *s)
 		for (i = 0; i <= m; i++)
 			s->as.u.note.lens[i] /= div;
 		s->len = s->as.u.note.lens[0];
+		s->type = type;
 	}
-	s->type = s->as.type == ABC_T_NOTE ? NOTE : REST;
 
 	/* convert the decorations */
 	if (s->as.u.note.dc.n > 0)
 		deco_cnv(&s->as.u.note.dc, s);
 
 	/* change the figure of whole measure rests */
-	if (s->type == REST && s->len == curvoice->wmeasure) {
-		if (s->len < BASE_LEN * 2)
-			s->as.u.note.lens[0] = BASE_LEN;
-		else if (s->len < BASE_LEN * 4)
-			s->as.u.note.lens[0] = BASE_LEN * 2;
-		else	s->as.u.note.lens[0] = BASE_LEN * 4;
+	if (s->type == REST) {
+		if (s->len == 0)		/* if space */
+			s->seq = 0;
+		else if (s->len == curvoice->wmeasure) {
+			if (s->len < BASE_LEN * 2)
+				s->as.u.note.lens[0] = BASE_LEN;
+			else if (s->len < BASE_LEN * 4)
+				s->as.u.note.lens[0] = BASE_LEN * 2;
+			else	s->as.u.note.lens[0] = BASE_LEN * 4;
+		}
 	}
 
 	/* sort by pitch the notes of the chord (lowest first) */
@@ -1498,24 +1557,28 @@ static void get_note(struct SYMBOL *s)
 			identify_note(s, l, &head, &dots, &nflags);
 			s->head = head;
 			s->dots = dots;
-			s->nflags = nflags;
-		}
-
-		for (i = 1; i <= m; i++)
-			if (s->as.u.note.lens[i] != l)
-				break;
-		if (i <= m) {
-			for (i = 1; i <= m; i++) {
-				if (s->as.u.note.lens[i] == l)
-					continue;
-				identify_note(s, s->as.u.note.lens[i],
-					      &head, &dots, &nflags);
-				if (head > s->head)
-					s->head = head;
-				if (dots > s->dots)
-					s->dots = dots;
-				if (nflags > s->nflags)
-					s->nflags = nflags;
+			if (!(s->sflags & S_TREM)) {
+				s->nflags = nflags;
+				for (i = 1; i <= m; i++) {
+					if (s->as.u.note.lens[i] == l)
+						continue;
+					identify_note(s, s->as.u.note.lens[i],
+						      &head, &dots, &nflags);
+					if (head > s->head)
+						s->head = head;
+					if (dots > s->dots)
+						s->dots = dots;
+					if (nflags > s->nflags)
+						s->nflags = nflags;
+				}
+				if (s->sflags & S_XSTEM)
+					s->nflags = 0;	/* word start+end */
+			} else {	/* 2nd tremolo note */
+				s->prev->head = head;
+				if (head >= H_OVAL) {
+					s->as.u.note.stemless = 1;
+					s->prev->as.u.note.stemless = 1;
+				}
 			}
 		}
 	}
@@ -1544,12 +1607,11 @@ static void get_note(struct SYMBOL *s)
 
 /* -- treat a postscript definition -- */
 static void ps_def(struct SYMBOL *s,
-		   unsigned char *p)
+		   char *p)
 {
 	if (s->as.state == ABC_S_TUNE
 	    || s->as.state == ABC_S_EMBED) {
-		sym_link(s);
-		s->type = FMTCHG;
+		sym_link(s, FMTCHG);
 		s->u = PSSEQ;
 		s->as.text = p;
 		return;
@@ -1564,17 +1626,16 @@ static void ps_def(struct SYMBOL *s,
 /* -- process a pseudo-comment (%% or I:) -- */
 static struct abcsym *process_pscomment(struct abcsym *as)
 {
-	unsigned char *p;
-	char w[32];
+	char w[32], *p;
 	float h1;
 	struct SYMBOL *s = (struct SYMBOL *) as;
 
 	p = as->text + 2;		/* skip '%%' */
-	while (isspace(*p))
+	while (isspace((unsigned) *p))
 		p++;
 	if (strncasecmp(p, "fmt ", 4) == 0) {
 		p += 4;			/* skip 'fmt' */
-		while (isspace(*p))
+		while (isspace((unsigned) *p))
 			p++;
 	}
 	p = get_str(w, p, sizeof w);
@@ -1588,11 +1649,11 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 				p = as->text;
 				if (*p == '%' && p[1] == '%') {
 					p += 2;
-					while (isspace(*p))
+					while (isspace((unsigned) *p))
 						p++;
 					if (strncasecmp(p, "fmt ", 4) == 0) {
 						p += 4;
-						while (isspace(*p))
+						while (isspace((unsigned) *p))
 							p++;
 					}
 					if (strncmp(p, "endps", 5) == 0)
@@ -1629,11 +1690,11 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 				p = as->text;
 				if (*p == '%' && p[1] == '%') {
 					p += 2;
-					while (isspace(*p))
+					while (isspace((unsigned) *p))
 						p++;
 					if (strncasecmp(p, "fmt ", 4) == 0) {
 						p += 4;
-						while (isspace(*p))
+						while (isspace((unsigned) *p))
 							p++;
 					}
 					if (strncmp(p, "endtext", 7) == 0) {
@@ -1657,8 +1718,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			output_music();
 			buffer_eob();
 			if ((epsf = fopen(p, "r")) == 0) {
-				error(1, s,
-				      "No such file: %s", p);
+				error(1, s, "No such file: %s", p);
 				return as;
 			}
 
@@ -1696,7 +1756,11 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			output_music();
 			if (strncmp(p, "start", 5) == 0) {
 				buffer_eob();
-				multicol_max = multicol_start = get_bposy();
+				if ((bposy = get_bposy()) == 0) {
+					PUT0("%%");	/* initialize the output */
+					bposy = get_bposy();
+				}
+				multicol_max = multicol_start = bposy;
 				lmarg = cfmt.leftmargin;
 				rmarg = cfmt.rightmargin;
 			} else if (strncmp(p, "new", 3) == 0) {
@@ -1723,6 +1787,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 					cfmt.leftmargin = lmarg;
 					cfmt.rightmargin = rmarg;
 					multicol_start = 0;
+					buffer_eob();
 				}
 			} else {
 				error(1, s,
@@ -1738,7 +1803,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			output_music();
 			write_buffer();
 			use_buffer = 0;
-			if (isdigit(*p))
+			if (isdigit((unsigned) *p))
 				pagenum = atoi(p);
 			close_page();
 			return as;
@@ -1777,30 +1842,30 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			h1 = h2 = len = 0;
 			if (*p != '\0') {
 				h1 = scan_u(p);
-				while (*p != '\0' && !isspace(*p))
+				while (*p != '\0' && !isspace((unsigned) *p))
 					p++;
-				while (isspace(*p))
+				while (isspace((unsigned) *p))
 					p++;
 			}
 			if (*p != '\0') {
 				h2 = scan_u(p);
-				while (*p != '\0' && !isspace(*p))
+				while (*p != '\0' && !isspace((unsigned) *p))
 					p++;
-				while (isspace(*p))
+				while (isspace((unsigned) *p))
 					p++;
 			}
 			if (*p != '\0')
 				len = scan_u(p);
 			if (h1 < 1)
-				h1 = 0.5 * CM;
+				h1 = 0.5 CM;
 			if (h2 < 1)
 				h2 = h1;
 			if (len < 1)
-				len = 3.0 * CM;
+				len = 3.0 CM;
 			bskip(h1);
 			PUT2("%.1f %.1f sep0\n",
-			     (lwidth - len) * 0.5 / cfmt.scale,
-			     (lwidth + len) * 0.5 / cfmt.scale);
+			     len / cfmt.scale,
+			     (lwidth - len) * 0.5 / cfmt.scale);
 			bskip(h2);
 			buffer_eob();
 			return as;
@@ -1824,12 +1889,11 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			if (as->state != ABC_S_TUNE
 			    && as->state != ABC_S_EMBED)
 				return as;
-			sym_link(s);
-			s->type = FMTCHG;
+			sym_link(s, FMTCHG);
 			s->u = STBRK;
 			if (*p != '\0')
 				s->xmx = scan_u(p);
-			else	s->xmx = 0.5 * CM;
+			else	s->xmx = 0.5 CM;
 			return as;
 		}
 		if (strcmp(w, "staves") == 0) {
@@ -1864,7 +1928,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			output_music();
 			h1 = scan_u(p);
 			if (h1 < 1)
-				h1 = 0.5 * CM;
+				h1 = 0.5 CM;
 			bskip(h1);
 			buffer_eob();
 			return as;
@@ -1956,14 +2020,13 @@ static void set_tuplet(struct SYMBOL *t)
 		error(1, t,
 		      "Tuplets in grace note sequence not yet treated");
 	else {
-		sym_link(t);
-		t->type = TUPLET;
+		sym_link(t, TUPLET);
 		t->u = cfmt.tuplets;
 	}
 }
 
 /* -- link a symbol in a voice -- */
-static void sym_link(struct SYMBOL *s)
+static void sym_link(struct SYMBOL *s, int type)
 {
 	struct VOICE_S *p_voice = curvoice;
 
@@ -1974,6 +2037,9 @@ static void sym_link(struct SYMBOL *s)
 	} else	p_voice->sym = s;
 	p_voice->last_symbol = s;
 
+	s->type = type;
+	s->seq = seq_tb[type];
 	s->voice = p_voice - voice_tb;
 	s->staff = p_voice->cstaff;
+	s->time = p_voice->time;
 }
