@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2004 Jean-Franois Moine
+ * Copyright (C) 1998-2005 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@
 #include "abcparse.h"
 #include "abc2ps.h" 
 
-unsigned char tex_buf[512];	/* result of tex_str() */
+char tex_buf[512];		/* result of tex_str() */
 struct FONTSPEC *font_init;	/* font in case of page break */
 int strcf;			/* current string font */
 
@@ -225,7 +225,7 @@ static struct SYMBOL *t;
 
 		t = info.title;
 		p = &t->as.text[2];
-		while (isspace(*p))
+		while (isspace((unsigned char) *p))
 			p++;
 		fprintf(stderr, "   - In tune `%s':\n", p);
 	}
@@ -248,32 +248,32 @@ float scan_u(char *str)
 
 	if (sscanf(str, "%f%n", &a, &nch) == 1) {
 		if (str[nch] == '\0' || str[nch] == ' ')
-			return a * PT;
+			return a PT;
 		if (!strncasecmp(str + nch, "cm", 2))
-			return a * CM;
+			return a CM;
 		if (!strncasecmp(str + nch, "in", 2))
-			return a * IN;
+			return a IN;
 		if (!strncasecmp(str + nch, "pt", 2))
-			return a * PT;
+			return a PT;
 	}
 	error(1, 0, "\n++++ Unknown unit value \"%s\"", str);
-	return 20 * PT;
+	return 20 PT;
 }
 
 /* -- capitalize a string -- */
-void cap_str(unsigned char *p)
+void cap_str(char *p)
 {
 	while (*p != '\0') {
 #if 1
 /* pb with toupper - works with ASCII only */
 		unsigned char c;
 
-		c = *p;
+		c = (unsigned char) *p;
 		if ((c >= 'a' && c <= 'z')
 		    || (c >= 0xe0 && c <= 0xfe))
 			*p = c & ~0x20;
 #else
-		*p = toupper(*p);
+		*p = toupper((unsigned char) *p);
 #endif
 		p++;
 	}
@@ -299,7 +299,7 @@ float cwid(unsigned char c)
  * Returns an estimate of the string width. */
 float tex_str(char *s)
 {
-	unsigned char *d, c1, c2, *p_enc, *p;
+	char *d, c1, c2, *p_enc, *p;
 	int maxlen, i;
 	float w, swfac;
 
@@ -337,7 +337,7 @@ float tex_str(char *s)
 				*d++ = c2;
 				*d++ = s[1];
 				c1 = ((c1 - '0') << 6) + ((c2 - '0') << 3) + s[1] - '0';
-				w += cwid(c1) * swfac;
+				w += cwid((unsigned char) c1) * swfac;
 				s += 2;
 				break;
 			}
@@ -375,7 +375,7 @@ float tex_str(char *s)
 			break;
 #endif
 		case '$':
-			if (isdigit(*s)
+			if (isdigit((unsigned char) *s)
 			    && (unsigned) (*s - '0') < DFONT_MIN) {
 				i = *s - '0';
 				swfac = cfmt.font_tb[i].swfac;
@@ -388,7 +388,7 @@ float tex_str(char *s)
 				if (--maxlen <= 0)
 					break;
 				*d++ = c1;
-				w += cwid(c1) * swfac;
+				w += cwid((unsigned char) c1) * swfac;
 				s++;
 			}
 			goto addchar1;
@@ -403,7 +403,7 @@ float tex_str(char *s)
 			if (--maxlen <= 0)
 				break;
 			*d++ = c1;
-			w += cwid(c1) * swfac;
+			w += cwid((unsigned char) c1) * swfac;
 			break;
 		}
 	}
@@ -419,7 +419,7 @@ void str_font(struct FONTSPEC *font)
 }
 
 /* -- output one string -- */
-static void str_ft_out1(unsigned char *p, int l)
+static void str_ft_out1(char *p, int l)
 {
 	if (strfc) {
 		if (strtx) {
@@ -437,14 +437,14 @@ static void str_ft_out1(unsigned char *p, int l)
 }
 
 /* -- output a string and the font changes -- */
-static void str_ft_out(unsigned char *p, int end)
+static void str_ft_out(char *p, int end)
 {
-	unsigned char *q;
+	char *q;
 
 	q = p;
 	while (*p != '\0') {
 		if (*p == '$') {
-			if (isdigit(p[1])
+			if (isdigit((unsigned char) p[1])
 			    && (unsigned) (p[1] - '0') < DFONT_MIN) {
 				if (p > q)
 					str_ft_out1(q, p - q);
@@ -472,7 +472,7 @@ static void str_ft_out(unsigned char *p, int end)
 }
 
 /* -- output a string, handling the font changes -- */
-void str_out(unsigned char *p, int action)
+void str_out(char *p, int action)
 {
 	if (strcf < 0) {		/* first call */
 		strcf = 0;
@@ -480,7 +480,7 @@ void str_out(unsigned char *p, int action)
 	}
 
 	/* special case when font change at start of text */
-	if (*p == '$' && isdigit(p[1])
+	if (*p == '$' && isdigit((unsigned char) p[1])
 	    && (unsigned) (p[1] - '0') < DFONT_MIN) {
 		if (strcf != p[1] - '0') {
 			strcf = p[1] - '0';
@@ -528,7 +528,7 @@ void str_out(unsigned char *p, int action)
 }
 
 /* -- output a string with TeX translation -- */
-void put_str(unsigned char *str, int action)
+void put_str(char *str, int action)
 {
 	tex_str(str);
 	str_out(tex_buf, action);
@@ -543,7 +543,7 @@ static void put_inf(struct SYMBOL *s)
 	p = s->as.text;
 	if (p[1] == ':')
 		p += 2;
-	while (isspace(*p))
+	while (isspace((unsigned char) *p))
 		p++;
 	put_str(p, A_LEFT);
 }
@@ -553,7 +553,7 @@ static void put_inf2r(struct SYMBOL *s1,
 		      struct SYMBOL *s2,
 		      int action)
 {
-	unsigned char buf[512], *p, *q;
+	char buf[512], *p, *q;
 
 	if (s1 == 0) {
 		s1 = s2;
@@ -563,14 +563,14 @@ static void put_inf2r(struct SYMBOL *s1,
 		p = s1->as.text;
 		if (p[1] == ':')
 			p += 2;
-		while (isspace(*p))
+		while (isspace((unsigned char) *p))
 			p++;
 	} else {
 		buf[sizeof buf - 1] = '\0';
 		p = s1->as.text;
 		if (p[1] == ':')
 			p += 2;
-		while (isspace(*p))
+		while (isspace((unsigned char) *p))
 			p++;
 		strncpy(buf, p, sizeof buf - 1);
 		q = buf + strlen(buf);
@@ -580,7 +580,7 @@ static void put_inf2r(struct SYMBOL *s1,
 			p = s2->as.text;
 			if (p[1] == ':')
 				p += 2;
-			while (isspace(*p))
+			while (isspace((unsigned char) *p))
 				p++;
 			strncpy(q, p, buf + sizeof buf - 2 - q);
 			q += strlen(q);
@@ -604,7 +604,7 @@ static void set_font_init(void)
 void add_to_text_block(char *s, int job)
 {
 	float baseskip, lw;
-	unsigned char *p, sep;
+	char *p, sep;
 	struct FONTSPEC *f;
 
 	/* if first line, set the fonts */
@@ -740,22 +740,22 @@ void write_text_block(int job, int abc_state)
 	buffer_eob();
 
 	/* next line to allow pagebreak after each paragraph */
-	if (!epsf && abc_state != ABC_S_TUNE && multicol_start == 0)
+	if (!epsf && abc_state != ABC_S_TUNE)
 		write_buffer();
 	font_init = 0;
 	strtw = -1;
 }
 
 /* -- output a line of words after tune -- */
-static int put_wline(unsigned char *p,
+static int put_wline(char *p,
 		     float x,
 		     int right)
 {
-	unsigned char *q, *r, sep;
+	char *q, *r, sep;
 
-	while (isspace(*p))
+	while (isspace((unsigned char) *p))
 		p++;
-	if (*p == '$' && isdigit(p[1])
+	if (*p == '$' && isdigit((unsigned char) p[1])
 	    && (unsigned) (p[1] - '0') < DFONT_MIN) {
 		if (strcf != p[1] - '0') {
 			strcf = p[1] - '0';
@@ -765,7 +765,7 @@ static int put_wline(unsigned char *p,
 	}
 	r = 0;
 	q = p;
-	if (isdigit(*p) || p[1] == '.') {
+	if (isdigit((unsigned char) *p) || p[1] == '.') {
 		while (*p != '\0') {
 			p++;
 			if (*p == ' '
@@ -803,7 +803,7 @@ static int put_wline(unsigned char *p,
 void put_words(struct SYMBOL *words)
 {
 	struct SYMBOL *s, *s_end, *s2;
-	unsigned char *p;
+	char *p;
 	int i, n, have_text, max2col;
 	float middle;
 
@@ -812,12 +812,12 @@ void put_words(struct SYMBOL *words)
 	/* see if we may have 2 columns */
 	middle = 0.5 * ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
 		- cfmt.leftmargin - cfmt.rightmargin) / cfmt.scale;
-	max2col = (middle - 45.) / (cwid('a') * cfmt.wordsfont.swfac);
+	max2col = (int) ((middle - 45.) / (cwid('a') * cfmt.wordsfont.swfac));
 	n = 0;
 	have_text = 0;
 	for (s = words; s != 0; s = s->next) {
 		p = &s->as.text[2];
-		while (isspace(*p))
+		while (isspace((unsigned char) *p))
 			p++;
 		if (strlen(p) > max2col) {
 			n = 0;
@@ -838,7 +838,7 @@ void put_words(struct SYMBOL *words)
 		s_end = words;
 		for (;;) {
 			p = &s_end->as.text[2];
-			while (isspace(*p))
+			while (isspace((unsigned char) *p))
 				p++;
 			if (*p == '\0') {
 				if (have_text && --i <= 0)
@@ -892,16 +892,17 @@ static void put_text(char *head,
 
 	if (s == 0)
 		return;
-	h = cfmt.textfont.size * cfmt.lineskipfac;
+	h = font_init->size * cfmt.lineskipfac;
 	bskip(h * 1.2);
-	strcf = -1;
-	PUT1("0 0 M(%s)show ", head);
-	if (strlen(head) > 6)
-		bskip(h);
+	if (strcf != 0) {
+		set_font(font_init);
+		strcf = 0;
+	}
+	PUT1("w%s ", head);
+	bskip(h);
 	do {
 		PUT0("50 0 M ");
 		put_inf(s);
-		PUT0("\n");
 		bskip(h);
 	} while ((s = s->next) != 0);
 	buffer_eob();
@@ -910,22 +911,27 @@ static void put_text(char *head,
 /* -- output history -- */
 void put_history(void)
 {
-	str_font(&cfmt.textfont);
 	bskip(cfmt.textspace);
+	font_init = &cfmt.historyfont;
+	str_font(&cfmt.historyfont);
+	set_font(&cfmt.historyfont);
+	strcf = 0;
+	set_font_init();
 	if (!cfmt.infoline)
-		put_text("Rhythm:", info.rhythm);
-	put_text("Book:", info.book);
-	put_text("Source:", info.src);
-	put_text("Discography: ", info.disco);
-	put_text("Notes: ", info.notes);
-	put_text("Transcription: ", info.trans);
-	put_text("History: ", info.histo);
+		put_text("rhythm", info.rhythm);
+	put_text("book", info.book);
+	put_text("source", info.src);
+	put_text("disco", info.disco);
+	put_text("notes", info.notes);
+	put_text("trans", info.trans);
+	put_text("histo", info.histo);
+	font_init = 0;
 }
 
 /* -- trim_title: move trailing "The" to front -- */
-static void trim_title(unsigned char *p)
+static void trim_title(char *p)
 {
-	unsigned char *q;
+	char *q;
 	int l;
 
 	l = strlen(p);
@@ -933,7 +939,7 @@ static void trim_title(unsigned char *p)
 	if (strcmp(q, "The") != 0)
 		return;
 	q--;
-	while (isspace(*q))
+	while (isspace((unsigned char) *q))
 		q--;
 	if (*q != ',')
 		return;
@@ -946,10 +952,10 @@ static void trim_title(unsigned char *p)
 /* -- write a title -- */
 void write_title(struct SYMBOL *s)
 {
-	unsigned char *p, t[200];
+	char *p, t[200];
 
 	p = &s->as.text[2];
-	while (isspace(*p))
+	while (isspace((unsigned char) *p))
 		p++;
 	if (*p == '\0' || strcmp(p, "(notitle)") == 0)
 		return;
@@ -977,9 +983,9 @@ void write_title(struct SYMBOL *s)
 }
 
 /* -- write_heading -- */
-void write_heading(void)
+void write_heading(struct abctune *t)
 {
-	struct SYMBOL *s, *rhythm;
+	struct SYMBOL *s, *rhythm, *area, *author;
 	float lwidth, down1, down2;
 
 	lwidth = ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
@@ -997,10 +1003,13 @@ void write_heading(void)
 		PUT1("0 %.1f M ",
 		     -(cfmt.composerspace + cfmt.composerfont.size));
 		put_inf(rhythm);
-		PUT0("\n");
 		down1 -= cfmt.composerfont.size;
 	}
-	if ((s = info.comp) != 0 || info.orig) {
+	area = author = 0;
+	if (t->abc_vers < 2)
+		area = info.area;
+	else	author = info.area;
+	if ((s = info.comp) != 0 || info.orig || author != 0) {
 		float xcomp;
 		int align;
 
@@ -1016,39 +1025,53 @@ void write_heading(void)
 			xcomp = lwidth;
 			align = A_RIGHT;
 		}
-		for (;;) {
-			bskip(cfmt.composerfont.size);
-			PUT1("%.1f 0 M ", xcomp);
-			put_inf2r(s,
-				  (s == 0 || s->next == 0) ? info.orig : 0,
-				  align);
-			PUT0("\n");
-			if (s == 0)
-				break;
-			if ((s = s->next) == 0)
-				break;
-			down1 += cfmt.composerfont.size;
+		down2 = down1;
+		if (author != 0) {
+			for (;;) {
+				bskip(cfmt.composerfont.size);
+				down2 += cfmt.composerfont.size;
+				PUT0("0 0 M ");
+				put_inf(author);
+				if ((author = author->next) == 0)
+					break;
+			}
+		}
+		if ((s = info.comp) != 0 || info.orig) {
+			if (cfmt.aligncomposer >= 0
+			    && down1 != down2)
+				bskip(down1 - down2);
+			for (;;) {
+				bskip(cfmt.composerfont.size);
+				PUT1("%.1f 0 M ", xcomp);
+				put_inf2r(s,
+					  (s == 0 || s->next == 0) ? info.orig : 0,
+					  align);
+				if (s == 0)
+					break;
+				if ((s = s->next) == 0)
+					break;
+				down1 += cfmt.composerfont.size;
+			}
+			if (down2 > down1)
+				bskip(down2 - down1);
 		}
 
 		rhythm = rhythm ? 0 : info.rhythm;
-		if ((rhythm || info.area) && cfmt.infoline) {
+		if ((rhythm || area) && cfmt.infoline) {
 
 			/* if only one of rhythm or area then do not use ()'s
 			 * otherwise output 'rhythm (area)' */
 			str_font(&cfmt.infofont);
 			bskip(cfmt.infofont.size + cfmt.infospace);
 			PUT1("%.1f 0 M ", lwidth);
-			put_inf2r(rhythm, info.area, A_RIGHT);
-			PUT0("\n");
+			put_inf2r(rhythm, area, A_RIGHT);
 			down1 += cfmt.infofont.size + cfmt.infospace;
 		}
 		down2 = 0;
-	} else {
-		down2 = cfmt.composerspace + cfmt.composerfont.size;
-	}
+	} else	down2 = cfmt.composerspace + cfmt.composerfont.size;
 
 	/* parts */
-	if (info.parts && cfmt.printparts) {
+	if (info.parts) {
 		down1 = cfmt.partsspace + cfmt.partsfont.size - down1;
 		if (down1 > 0)
 			down2 += down1;
@@ -1057,7 +1080,6 @@ void write_heading(void)
 		str_font(&cfmt.partsfont);
 		PUT0("0 0 M ");
 		put_inf(info.parts);
-		PUT0("\n");
 		down2 = 0;
 	}
 	bskip(down2 + cfmt.musicspace);
