@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2004 Jean-Franois Moine
+ * Copyright (C) 1998-2005 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -101,7 +101,7 @@ static char ps_head[] =
 
 	"/sbclef{exch 0.85 div exch 0.85 div gsave 0.85 dup scale 0 3 T bclef grestore}!\n"
 
-	"/cchalf{0 0 M 0 12 RM\n"
+	"/cchalf{0 12 M\n"
 	"	2.6 5 RL\n"
 	"	2.3 -5.8 5.2 -2.4 4.7 1.6 RC\n"
 	"	0.4 3.9 -3 6.7 -5.1 4 RC\n"
@@ -157,15 +157,15 @@ static char ps_head[] =
 
 	/* x y r4 - quarter rest */
 	"/r4{	xymove\n"
-	"	-0.5 8.9 RM\n"
-	"	1.3 -3.4 RL\n"
-	"	-2 -4.5 RL\n"
-	"	3.1 -4.8 RL\n"
-	"	-3.2 3.5 -5.8 -1.4 -1.4 -3.8 RC\n"
-	"	-1.9 2 -0.8 5 2.4 2.6 RC\n"
-	"	-2.2 4.2 RL\n"
-	"	0 0 2 4.7 2.1 4.7 RC\n"
-	"	-3.3 5 RL\n"
+	"	-1 8.5 RM\n"
+	"	3.6 -5.1 RL\n"
+	"	-2.1 -5.2 RL\n"
+	"	2.2 -4.3 RL\n"
+	"	-2.6 2.3 -5.1 0 -2.4 -2.6 RC\n"
+	"	-4.8 3 -1.5 6.9 1.4 4.1 RC\n"
+	"	-3.1 4.5 RL\n"
+	"	1.9 5.1 RL\n"
+	"	-1.5 3.5 RL\n"
 	"	fill}!\n"
 
 	/* 1/8 .. 1/64 rest element */
@@ -214,8 +214,8 @@ static char ps_head[] =
 	"	x y M 1.4 -2 RM r8e\n"
 	"	x y M -0.1 -8 RM r8e}!\n"
 
-	/* x dy dt - dot relative to head shifted by dy */
-	"/dt{y add M currentpoint 1.2 0 360 arc fill}!\n"
+	/* dx dy dt - dot relative to head */
+	"/dt{x y M RM currentpoint 1.2 0 360 arc fill}!\n"
 
 	/* x y hld - fermata */
 	"/hld{	1.5 add 2 copy 1.5 add M currentpoint 1.3 0 360 arc\n"
@@ -377,10 +377,10 @@ static char ps_head[] =
 	"			2.1 0 lineto\n"
 	"			1.9 0.8 1.4 0.7 0 -0.4 curveto\n"
 	"			fill\n"
-	"			pop 180 rotate -6 0 translate\n"
+	"			pop 180 rotate -6 0 T\n"
 	"		}for\n"
 	/*		% shift axes right one squiggle*/
-	"		pop 6 0 translate\n"
+	"		pop 6 0 T\n"
 	"	}for\n"
 	"	grestore}!\n"
 
@@ -394,13 +394,13 @@ static char ps_head[] =
 	"/opend{dlw M currentpoint 3 add 2.5 -90 270 arc stroke}!\n"
 
 	/* x y snap - 'snap' sign */
-	"/snap{	dlw M currentpoint -3 6 RM\n"
+	"/snap{	dlw 2 copy M -3 6 RM\n"
 	"	0 5 6 5 6 0 RC\n"
 	"	0 -5 -6 -5 -6 0 RC\n"
 	"	5 add M 0 -6 RL stroke}!\n"
 
 	/* x y thumb - 'thumb' sign */
-	"/thumb{	dlw M currentpoint -2.5 7 RM\n"
+	"/thumb{	dlw 2 copy M -2.5 7 RM\n"
 	"	0 6 5 6 5 0 RC\n"
 	"	0 -6 -5 -6 -5 0 RC\n"
 	"	2 add M 0 -4 RL stroke}!\n"
@@ -460,7 +460,7 @@ static char ps_head[] =
 	/* x y dft0 - double flat sign */
 	"/dft0{2 copy M -2.5 0 RM ftx M 1.5 0 RM ftx}!\n"
 	/* ancillary function for grace note accidentals */
-	"/gsc{gsave y T 0.7 dup scale 0 0}!\n"
+	"/gsc{gsave y T .7 dup scale 0 0}!\n"
 
 	/* some microtone accidentals */
 	/* 1/4 ton sharp */
@@ -485,17 +485,17 @@ static char ps_head[] =
 
 	/* accidentals in guitar chord */
 	"/sharp_glyph{\n"
-	"	fh 0.4 mul 0 RM currentpoint\n"
-	"	gsave T fh 0.08 mul dup scale 0 7 sh0 grestore\n"
-	"	fh 0.4 mul 0 RM}!\n"
+	"	fh .4 mul 0 RM currentpoint\n"
+	"	gsave T fh .08 mul dup scale 0 7 sh0 grestore\n"
+	"	fh .4 mul 0 RM}!\n"
 	"/flat_glyph{\n"
-	"	fh 0.4 mul 0 RM currentpoint\n"
-	"	gsave T fh 0.08 mul dup scale 0 5 ft0 grestore\n"
-	"	fh 0.4 mul 0 RM}!\n"
+	"	fh .4 mul 0 RM currentpoint\n"
+	"	gsave T fh .08 mul dup scale 0 5 ft0 grestore\n"
+	"	fh .4 mul 0 RM}!\n"
 	"/nat_glyph{\n"
-	"	fh 0.4 mul 0 RM currentpoint\n"
-	"	gsave T fh 0.08 mul dup scale 0 7 nt0 grestore\n"
-	"	fh 0.4 mul 0 RM}!\n"
+	"	fh .4 mul 0 RM currentpoint\n"
+	"	gsave T fh .08 mul dup scale 0 7 nt0 grestore\n"
+	"	fh .4 mul 0 RM}!\n"
 	/* str gcshow - guitar chord */
 	"/gchshow{\n"
 	"	pop pop\n"
@@ -511,7 +511,7 @@ static char ps_head[] =
 	"/box{.6 SLW rectstroke}!\n"
 
 	/* str anshow - annotation */
-	"/anshow{gcshow}!\n"
+	"/anshow{/gchshow load cshow}!\n"
 
 	/* -- lyrics under notes -- */
 	/* l x y wln - underscore line */
@@ -534,8 +534,8 @@ static char ps_head[] =
 	/* h x y thbar - thick bar */
 	"/thbar{3 -1 roll 3 exch rectfill}!\n"
 	/* x y rdots - repeat dots */
-	"/rdots{	9 add M currentpoint 2 copy 1.2 0 360 arc\n"
-	"	6 add M currentpoint 1.2 0 360 arc fill}!\n"
+	"/rdots{	2 copy 9 add M currentpoint 1.2 0 360 arc\n"
+	"	15 add M currentpoint 1.2 0 360 arc fill}!\n"
 
 	/* x y csig - C timesig */
 	"/csig{	M\n"
@@ -555,21 +555,19 @@ static char ps_head[] =
 	/* (top) (bot) x y tsig - time signature */
 	"/tsig{	M gsave/Times-Bold 16 selectfont 1.2 1 scale\n"
 	"	0 1 RM currentpoint 3 -1 roll showc\n"
-	"	M 0 12 RM showc grestore}!\n"
+	"	12 add M showc grestore}!\n"
 
 	/* (meter) x y stsig - single time signature */
 	"/stsig{	M gsave/Times-Bold 18 selectfont 1.2 1 scale\n"
 	"	0 6 RM showc grestore}!\n"
 
-	/* l x y staff - 5 lines staff */
-	"/staff{	M dlw dup 0 RL dup neg 6 RM\n"
-	"	dup 0 RL dup neg 6 RM\n"
-	"	dup 0 RL dup neg 6 RM\n"
-	"	dup 0 RL dup neg 6 RM\n"
-	"	0 RL stroke}!\n"
+	/* l y n staff - staff with n lines*/
+	"/staff{	dlw 1 1 3 -1 roll{\n"
+	"		pop 0 1 index M 1 index 0 RL 6 add\n"
+	"	}for pop pop stroke}!\n"
 
-	/* x1 x2 sep0 - hline separator */
-	"/sep0{dlw 0 M 0 lineto stroke}!\n"
+	/* l x sep0 - hline separator */
+	"/sep0{dlw 0 M 0 RL stroke}!\n"
 
 	"/hbrce{	-2.5 1 RM\n"
 	"	-4.5 -4.6 -7.5 -12.2 -4.4 -26.8 RC\n"
@@ -578,7 +576,7 @@ static char ps_head[] =
 	"	-2.7 9.5 -1.5 16.2 3 21.5 RC\n"
 	"	fill}!\n"
 	/* h x y brace */
-	"/brace{	gsave T 0 0 M 0.01 mul 1 exch scale hbrce\n"
+	"/brace{	gsave T 0 0 M .01 mul 1 exch scale hbrce\n"
 	"	0 -100 M 1 -1 scale hbrce grestore}!\n"
 
 	/* h x y bracket */
@@ -629,6 +627,15 @@ static char ps_head[] =
 	"/strw{stringwidth pop w add/w exch def}!\n"
 	"/jshow{w 0 32 4 -1 roll widthshow}!\n"
 
+	/* texts which may be translated */
+	"/wbook{0 0 M (Book:)show}!\n"
+	"/wdisco{0 0 M(Discography:)show}!\n"
+	"/whisto{0 0 M(History:)show}!\n"
+	"/wnotes{0 0 M(Notes:)show}!\n"
+	"/wrhythm{0 0 M(Rhythm:)show}!\n"
+	"/wsource{0 0 M(Source:)show}!\n"
+	"/wtrans{0 0 M(Transcription:)show}!\n"
+
 	/* -- note heads -- */
 	/* x y hd - full head */
 	"/hd{	xymove\n"
@@ -679,6 +686,23 @@ static char ps_head[] =
 	"/pflhd{/x 2 index def/y 1 index def dsh0\n"
 	"	.7 SLW x y 4 0 360 arc stroke}!\n"
 
+	/* x y ghd - grace note head */
+	"/ghd{	xymove\n"
+	"	1.7 1 RM\n"
+	"	-1 1.7 -4.5 -0.2 -3.4 -2 RC\n"
+	"	1 -1.7 4.5 0.2 3.4 2 RC fill}!\n"
+
+	/* dx dy gua / gda - acciaccatura */
+	"/gua{x y M -1 4 RM RL stroke}!\n"
+	"/gda{x y M -5 -4 RM RL stroke}!\n"
+
+	/* y ghl - grace note helper line */
+	"/ghl{	.6 SLW x -3 add exch M\n"
+	"	6 0 RL stroke}!\n"
+
+	/* x1 y2 x2 y2 x3 y3 x0 y0 gsl - grace note slur */
+	"/gsl{dlw M curveto stroke}!\n"
+
 	/* tin whistle */
 	"/tw_head{/Helvetica 8 selectfont\n"
 	"	0 -45 M 90 rotate(WHISTLE)show -90 rotate\n"
@@ -843,33 +867,31 @@ void define_font(char name[],
 		 int enc)
 {
 	if (enc == ENC_NATIVE || strcmp(name, "Symbol") == 0) {
-		fprintf(fout, "/F%d{dup 0.8 mul/fh exch def"
+		fprintf(fout, "/F%d{dup .8 mul/fh exch def"
 			"/%s exch selectfont}!\n",
 			num, name);
 		return;
 	}
 	fprintf(fout, "/%s%d/%s mkfontext%d\n"
-		"/F%d{dup 0.8 mul/fh exch def/%s%d exch selectfont}!\n",
+		"/F%d{dup .8 mul/fh exch def/%s%d exch selectfont}!\n",
 		name, enc, name, enc, num, name, enc);
 }
 
-/* -- def_stems -- */
-static void def_stems(FILE *fp)
+/* -- output the symbol definitions -- */
+void define_symbols(void)
 {
+	fputs(ps_head, fout);
+
 	/* len su - up stem */
-	fprintf(fp, "/su{dlw x y M %.1f %.1f RM %.1f sub 0 exch RL stroke}!\n",
+	fprintf(fout, "/su{dlw x y M %.1f %.1f RM %.1f sub 0 exch RL stroke}!\n",
 		STEM_XOFF, STEM_YOFF, STEM_YOFF);
 
 	/* len sd - down stem */
-	fprintf(fp, "/sd{dlw x y M %.1f %.1f RM %.1f add 0 exch RL stroke}!\n",
+	fprintf(fout, "/sd{dlw x y M %.1f %.1f RM %.1f add 0 exch RL stroke}!\n",
 		-STEM_XOFF, -STEM_YOFF, STEM_YOFF);
-}
 
-/* -- stem and flags -- */
-static void def_flags(FILE *fp)
-{
 	/* n len sfu - stem and n flags up */
-	fprintf(fp, "/sfu{	dlw x y M %.1f %.1f RM\n"
+	fprintf(fout, "/sfu{	dlw x y M %.1f %.1f RM\n"
 		"	%.1f sub 0 exch RL currentpoint stroke\n"
 		"	M dup 1 eq\n"
 		"	  {\n"
@@ -892,7 +914,7 @@ static void def_flags(FILE *fp)
 		STEM_XOFF, STEM_YOFF, STEM_YOFF);
 
 	/* n len sfd - stem and n flags down */
-	fprintf(fp, "/sfd{	dlw x y M %.1f %.1f RM\n"
+	fprintf(fout, "/sfd{	dlw x y M %.1f %.1f RM\n"
 		"	%.1f add 0 exch RL currentpoint stroke\n"
 		"	M dup 1 eq\n"
 		"	  {\n"
@@ -915,7 +937,7 @@ static void def_flags(FILE *fp)
 		-STEM_XOFF, -STEM_YOFF, STEM_YOFF);
 
 	/* n len sfs - stem and n straight flag down */
-	fprintf(fp, "/sfs{	dlw x y M %.1f %.1f RM\n"
+	fprintf(fout, "/sfs{	dlw x y M %.1f %.1f RM\n"
 		"	%.1f add 0 exch RL currentpoint stroke\n"
 		"	M 1 1 3 -1 roll{\n"
 		"		pop currentpoint\n"
@@ -926,19 +948,9 @@ static void def_flags(FILE *fp)
 		"	}for}!\n",
 		-STEM_XOFF, -STEM_YOFF, STEM_YOFF,
 		BEAM_DEPTH, BEAM_DEPTH, BEAM_DEPTH);
-}
-
-/* -- grace notes -- */
-static void def_gnote(FILE *fp)
-{
-	/* x y ghd - grace note head */
-	fprintf(fp, "/ghd{	xymove\n"
-		"	1.7 1 RM\n"
-		"	-1 1.7 -4.5 -0.2 -3.4 -2 RC\n"
-		"	1 -1.7 4.5 0.2 3.4 2 RC fill}!\n"
 
 	/* len gu - grace note stem up */
-		"/gu{	.6 SLW x y M\n"
+	fprintf(fout, "/gu{	.6 SLW x y M\n"
 		"	%.1f 0 RM 0 exch RL stroke}!\n"
 
 	/* len gd - grace note stem down */
@@ -947,7 +959,7 @@ static void def_gnote(FILE *fp)
 		GSTEM_XOFF, -GSTEM_XOFF);
 
 	/* n len sgu - gnote stem and n flag up */
-	fprintf(fp, "/sgu{	.6 SLW x y M %.1f 0 RM\n"
+	fprintf(fout, "/sgu{	.6 SLW x y M %.1f 0 RM\n"
 		"	0 exch RL currentpoint stroke\n"
 		"	M dup 1 eq\n"
 		"	  {\n"
@@ -967,7 +979,7 @@ static void def_gnote(FILE *fp)
 		GSTEM_XOFF);
 
 	/* n len sgd - gnote stem and n flag down */
-	fprintf(fp, "/sgd{	.6 SLW x y M %.1f 0 RM\n"
+	fprintf(fout, "/sgd{	.6 SLW x y M %.1f 0 RM\n"
 		"	0 exch RL currentpoint stroke\n"
 		"	M dup 1 eq\n"
 		"	  {\n"
@@ -987,7 +999,7 @@ static void def_gnote(FILE *fp)
 		-GSTEM_XOFF);
 
 	/* n len sgs - gnote stem and n straight flag up */
-	fprintf(fp, "/sgs{	.6 SLW x y M %.1f 0 RM\n"
+	fprintf(fout, "/sgs{	.6 SLW x y M %.1f 0 RM\n"
 		"	0 exch RL currentpoint stroke\n"
 		"	M 1 1 3 -1 roll{\n"
 		"		pop currentpoint\n"
@@ -995,25 +1007,4 @@ static void def_gnote(FILE *fp)
 		"		closepath fill -3 add M\n"
 		"	}for}!\n",
 		GSTEM_XOFF);
-
-	fprintf(fp,
-		/* dx dy gua / gda - acciaccatura */
-		"/gua{x y M -1 4 RM RL stroke}!\n"
-		"/gda{x y M -5 -4 RM RL stroke}!\n"
-
-		/* y ghl - grace note helper line */
-		"/ghl{	.6 SLW x -3 add exch M\n"
-		"	6 0 RL stroke}!\n"
-
-		/* x1 y2 x2 y2 x3 y3 x0 y0 gsl */
-		"/gsl{dlw M curveto stroke}!\n");
-}
-
-/* -- define_symbols: write postscript macros to file -- */
-void define_symbols(void)
-{
-	fputs(ps_head, fout);
-	def_stems(fout);
-	def_flags(fout);
-	def_gnote(fout);
 }
